# -*- encoding: UTF-8 -*-

# list of affix codes
_lTags = ['1', '4eur', '2r', '1|2ux', '2', '1ux', '3er', '2f', '0x', '2x', '2i', '2a', '2u', '3c', '0es', '', '3|2', '1i', '1ies', '2ux', '5ur', '2en', '3en', '3ef', '3n', '3et', '3x', '3s', '4is', '3', '3ek', '1e', '4ec']

# dictionary of words with uncommon plurals (-x, -ux, english, latin and italian plurals) and tags to generate them
_dPlurMisc = {u'addendum': 11, u'bandeau': 8, u'naseau': 8, u'héronneau': 8, u'latifundium': 11, u'jottereau': 8, u'pou': 8, u'hoyau': 8, u'processionnal': 5, u'travail': 19, u'verseau': 8, u'troupeau': 8, u'organeau': 8, u'joujou': 8, u'seau': 8, u'contre-feu': 8, u'penny': 18, u'hachereau': 8, u'piédestal': 5, u'cristal': 5, u'porte-manteau': 8, u'museau': 8, u'procès-verbal': 5, u'carneau': 8, u'speech': 14, u'majoral': 5, u'vocero': 17, u'brigandeau': 8, u'scotch': 14, u'adieu': 8, u'fieu': 8, u'tuyau': 8, u'tord-boyau': 8, u'têteau': 8, u'asseau': 8, u'doubleau': 8, u'junky': 18, u'loqueteau': 8, u'condottiere': 17, u'tennisman': 21, u'racinal': 5, u'panneau': 8, u'cerveau': 8, u'damoiseau': 8, u'jazzman': 21, u'stimulus': 10, u'tourtereau': 8, u'guideau': 8, u'conoyau': 8, u'chapeau': 8, u'cigogneau': 8, u'quintal': 5, u'hobby': 18, u'trullo': 17, u'diurnal': 5, u'sanctoral': 5, u'saumoneau': 8, u'mafioso': 17, u'cardinal': 5, u'porte-drapeau': 8, u'rondeau': 8, u'fabliau': 8, u'corbeau': 8, u'houseau': 8, u'traineau': 8, u'tréteau': 8, u'xénocristal': 5, u'poètereau': 8, u'flûtiau': 8, u'yachtman': 21, u'clergyman': 21, u'ramponeau': 8, u'wallaby': 18, u'javeau': 8, u'hébreu': 8, u'brandy': 18, u'yeoman': 21, u'gruppetto': 17, u'étourneau': 8, u'sketch': 14, u'ponceau': 8, u'merbau': 8, u'géosynclinal': 5, u'jouvenceau': 8, u'perdreau': 8, u'ronchonneau': 8, u'arsenal': 5, u'sherry': 18, u'chauffe-eau': 8, u'vanneau': 8, u'cadeau': 8, u'trousseau': 8, u'porte-chapeau': 8, u'éditorial': 5, u'grizzly': 18, u'pique-feu': 8, u'bihoreau': 8, u'eau': 8, u'rouleau': 8, u'cherry': 18, u'cache-museau': 8, u'pieu': 8, u'pintadeau': 8, u'meneau': 8, u'franco-provençal': 5, u'fourneau': 8, u'wattman': 21, u'cameraman': 21, u'réal': 5, u'camaïeu': 8, u'coupeau': 8, u'hâtiveau': 8, u'baleineau': 8, u'pruneau': 8, u'perchman': 21, u'flutiau': 8, u'jeu': 8, u'poireau': 8, u'gentleman': 21, u'simbleau': 8, u'coupe-feu': 8, u'pitch': 14, u'carreau': 8, u'paparazzo': 17, u'urinal': 5, u'vaisseau': 8, u'château': 8, u'partita': 31, u'pagus': 10, u'aveu': 8, u'recordman': 21, u'serdeau': 8, u'hôpital': 5, u'ouvreau': 8, u'flambeau': 8, u'self-made-man': 21, u'cheveu': 8, u'match': 14, u'linteau': 8, u'veau': 8, u'beau': 8, u'rinceau': 8, u'girafeau': 8, u'gerzeau': 8, u'conneau': 8, u'clash': 14, u'gagman': 21, u'caniveau': 8, u'arrière-vassal': 5, u'surjeu': 8, u'bocal': 5, u'fauconneau': 8, u'manteau': 8, u'boyau': 8, u'car-ferry': 18, u'rugbyman': 21, u'gemmail': 19, u'impedimentum': 11, u'épieu': 8, u'moineau': 8, u'gouttereau': 8, u'biscoteau': 8, u'triclinium': 11, u'idéal': 5, u'brise-copeau': 8, u'coach': 14, u'oripeau': 8, u'fricandeau': 8, u'corporal': 5, u'sèche-cheveu': 8, u'vigneau': 8, u'renardeau': 8, u'vipereau': 8, u'oral': 5, u'garde-feu': 8, u'maffioso': 17, u'jambonneau': 8, u'louveteau': 8, u'fléau': 8, u'non-métal': 5, u'yachtswoman': 21, u'desideratum': 11, u'lavatory': 18, u'animal': 5, u'ecstasy': 18, u'lazzarone': 17, u'hâtereau': 8, u'recordwoman': 21, u'dindonneau': 8, u'matériau': 8, u'pineau': 8, u'ferry': 18, u'sulky': 18, u'yeomanry': 18, u'bedeau': 8, u'ormeau': 8, u'céteau': 8, u'trijumeau': 8, u'feu': 8, u'anneau': 8, u'mail-coach': 14, u'boucau': 8, u'blaireau': 8, u'lunch': 14, u'pianissimo': 17, u'marsupial': 5, u'pare-feu': 8, u'arbrisseau': 8, u'sous-arbrisseau': 8, u'grémial': 5, u'vau': 8, u'tombereau': 8, u'fermail': 19, u'chalumeau': 8, u'smash': 14, u'tenniswoman': 21, u'rampeau': 8, u'exemplum': 11, u'paonneau': 8, u'businesswoman': 21, u'scrogneugneu': 8, u'vousseau': 8, u'vice-amiral': 5, u'maquereau': 8, u'pourceau': 8, u'hibou': 8, u'pupazzo': 17, u'sceau': 8, u'fromental': 5, u'étau': 8, u'couleuvreau': 8, u'listeau': 8, u'micronoyau': 8, u'sénéchal': 5, u'demi-mal': 5, u'ciseau': 8, u'fontainebleau': 8, u'bateau': 8, u'hottereau': 8, u'biveau': 8, u'troubleau': 8, u'caveau': 8, u'protège-genou': 8, u'self-made-woman': 21, u'erseau': 8, u'midshipman': 21, u'soprano': 17, u'cerneau': 8, u'ombudsman': 21, u'cuveau': 8, u'faisandeau': 8, u'bécasseau': 8, u'emporium': 11, u'speakeasy': 18, u'appeau': 8, u'chaudeau': 8, u'godiveau': 8, u'mémorial': 5, u'godelureau': 8, u'tourangeau': 8, u'vantail': 19, u'doleau': 8, u'canal': 5, u'coagulum': 11, u'éfourceau': 8, u'demi-dieu': 8, u'rameau': 8, u'plateau': 8, u'ruisseau': 8, u'émail': 19, u'acinus': 10, u'morceau': 8, u'putto': 17, u'madrigal': 5, u'antiviral': 5, u'trumeau': 8, u'sanatorium': 11, u'hippy': 18, u'yachtsman': 21, u'tuileau': 8, u'oiseau': 8, u'barbeau': 8, u'scutum': 11, u'lapereau': 8, u'tonlieu': 8, u'fanal': 5, u'symposium': 11, u'polycristal': 5, u'rejeu': 8, u'tory': 18, u'fesse-mathieu': 8, u'cranberry': 18, u'lanterneau': 8, u'flûteau': 8, u'pointeau': 8, u'boisseau': 8, u'boutefeu': 8, u'lionceau': 8, u'chemineau': 8, u'slash': 14, u'éléphanteau': 8, u'puceau': 8, u'arrière-cerveau': 8, u'non-lieu': 8, u'taureau': 8, u'râteau': 8, u'snoreau': 8, u'vipéreau': 8, u'poteau': 8, u'barreau': 8, u'pousse-caillou': 8, u'memorandum': 11, u'bouleau': 8, u'quasi-cristal': 5, u'hameau': 8, u'carbonaro': 17, u'poétereau': 8, u'enfaiteau': 8, u'aloyau': 8, u'ricercare': 17, u'box': 14, u'tourteau': 8, u'bitoniau': 8, u'sautereau': 8, u'winch': 14, u'grimpereau': 8, u'ramereau': 8, u'corail': 19, u'aboiteau': 8, u'whisky': 18, u'fardeau': 8, u'flush': 14, u'chenal': 5, u'roseau': 8, u'chevreau': 8, u'nobliau': 8, u'tire-feu': 8, u'terreau': 8, u'traîneau': 8, u'confessionnal': 5, u'brunch': 14, u'enfaîteau': 8, u'buggy': 18, u'boss': 14, u'neveu': 8, u'milieu': 8, u'touchau': 8, u'saute-ruisseau': 8, u'arioso': 17, u'contrefeu': 8, u'daleau': 8, u'dinghy': 18, u'guindeau': 8, u'escabeau': 8, u'husky': 18, u'official': 5, u'cerceau': 8, u'cuisseau': 8, u'jumeau': 8, u'moyeu': 8, u'plumasseau': 8, u'casseau': 8, u'colineau': 8, u'chrémeau': 8, u'prélinteau': 8, u'pontuseau': 8, u'tuffeau': 8, u'chanteau': 8, u'bail': 19, u'ramponneau': 8, u'penalty': 18, u'solo': 17, u'faisceau': 8, u'lady': 18, u'pizzaiolo': 17, u'bardeau': 8, u'temporal': 5, u'pifferaro': 17, u'cailleteau': 8, u'arceau': 8, u'sureau': 8, u'optimum': 11, u'esquimau': 8, u'porreau': 8, u'enjeu': 8, u'ranch': 14, u'cheval': 5, u'radeau': 8, u'canardeau': 8, u'tableau': 8, u'burgau': 8, u'coyau': 8, u'coulisseau': 8, u'fuseau': 8, u'phylum': 11, u'pinceau': 8, u'mâtereau': 8, u'arcdoubleau': 8, u'hobereau': 8, u'sandwich': 14, u'curriculum': 11, u'friponneau': 8, u'flash': 14, u'agneau': 8, u'antirétroviral': 5, u'berceau': 8, u'couvre-feu': 8, u'tempo': 17, u'gémeau': 8, u'chou': 8, u'liteau': 8, u'coupe-chou': 8, u'bigorneau': 8, u'fluteau': 8, u'marteau': 8, u'gentry': 18, u'prima-donna': 31, u'grau': 8, u'hirondeau': 8, u'sportsman': 21, u'métal': 5, u'hors-jeu': 8, u'chameau': 8, u'bluteau': 8, u'bull-finch': 14, u'joyau': 8, u'monsignore': 17, u'antislash': 14, u'réseau': 8, u'scratch': 14, u'dieu': 8, u'coteau': 8, u'genou': 8, u'lobby': 18, u'oculus': 10, u'pureau': 8, u'vassiveau': 8, u'baby': 18, u'libretto': 17, u'fécial': 5, u'panonceau': 8, u'gâteau': 8, u'peau': 8, u'nouveau': 8, u'pommeau': 8, u'nursery': 18, u'bourreau': 8, u'signal': 5, u'écheveau': 8, u'baliveau': 8, u'taxum': 11, u'juke-box': 14, u'bijou': 8, u'proxy': 18, u'bureau': 8, u'tufeau': 8, u'rabat-eau': 8, u'ypréau': 8, u'pectoral': 5, u'gruau': 8, u'coccus': 10, u'couteau': 8, u'dégueu': 8, u'copeau': 8, u'superman': 21, u'outardeau': 8, u'ripou': 8, u'vermisseau': 8, u'équatorial': 5, u'serpenteau': 8, u'faisanneau': 8, u'carpeau': 8, u'canzone': 17, u'porte-couteau': 8, u'allume-feu': 8, u'pecorino': 17, u'rousseau': 8, u'businessman': 21, u'juste-milieu': 8, u'orignal': 5, u'gluau': 8, u'mangonneau': 8, u'moreau': 8, u'zigoteau': 8, u'policeman': 21, u'barman': 21, u'mal': 5, u'clameau': 8, u'faiteau': 8, u'pigeonneau': 8, u'microcristal': 5, u'pastoureau': 8, u'chapiteau': 8, u'télétravail': 19, u'préau': 8, u'impresario': 17, u'marmenteau': 8, u'bush': 14, u'rideau': 8, u'bau': 8, u'yachtwoman': 21, u'pipeau': 8, u'écriteau': 8, u'aboteau': 8, u'ventail': 19, u'portemanteau': 8, u'toccata': 31, u'instrumental': 5, u'columbarium': 11, u'aisseau': 8, u'garden-party': 18, u'sous-réseau': 8, u'journal': 5, u'antijeu': 8, u'tonneau': 8, u'courlieu': 8, u'merleau': 8, u'tribunal': 5, u'haveneau': 8, u'biomatériau': 8, u'cordeau': 8, u'bigarreau': 8, u'tombeau': 8, u'lambeau': 8, u'lieu': 8, u'tommy': 18, u'oppidum': 11, u'jet-society': 18, u'bonneteau': 8, u'rational': 5, u'nuraghe': 17, u'vœu': 8, u'body': 18, u'caïeu': 8, u'alleu': 8, u'pizzicato': 17, u'sacramental': 5, u'quantum': 11, u'crève-tonneau': 8, u'vitrail': 19, u'claveau': 8, u'soupirail': 19, u'bestiau': 8, u'pointal': 5, u'dandy': 18, u'bravo': 17, u'souriceau': 8, u'plumeau': 8, u'fourreau': 8, u'nanomatériau': 8, u'lorry': 18, u'créneau': 8, u'présidial': 5, u'téléjournal': 5, u'noyau': 8, u'drapeau': 8, u'levreau': 8, u'passereau': 8, u'désaveu': 8, u'nanoréseau': 8, u'numéral': 5, u'bordereau': 8, u'cyclo-cross': 14, u'medium': 11, u'remix': 14, u'biseau': 8, u'cross-country': 18, u'monceau': 8, u'grumeau': 8, u'abdominal': 5, u'paisseau': 8, u'renouveau': 8, u'essieu': 8, u'fronteau': 8, u'rash': 14, u'tyranneau': 8, u'zingaro': 17, u'tumulus': 10, u'faîteau': 8, u'tasseau': 8, u'feld-maréchal': 5, u'caillou': 8, u'criterium': 11, u'phénocristal': 5, u'scriptorium': 11, u'boqueteau': 8, u'rush': 14, u'végétal': 5, u'local': 5, u'gerseau': 8, u'baggy': 18, u'soliveau': 8, u'niveau': 8, u'monocristal': 5, u'chéneau': 8, u'miss': 14, u'crash': 14, u'scenario': 17, u'batardeau': 8}

# dictionary of feminine forms and tags to generate masculine forms (singular and plural)
_dMasFW = {u'australo-néo-zélandaise': 0, u'coassante': 0, u'nocive': 7, u'prostituée': 0, u'décideuse': 2, u'plaine': 0, u'ménapienne': 4, u'pommée': 0, u'colonisée': 0, u'diserte': 0, u'semi-manufacturée': 0, u'diététicienne': 4, u'populeuse': 9, u'touchante': 0, u'dégonflarde': 0, u'monodimensionnelle': 4, u'émulatrice': 1, u'monodirectionnelle': 4, u'pincharde': 0, u'nattière': 6, u'dépoussiérante': 0, u'tranquillisante': 0, u'téléconseillère': 6, u'australo-américaine': 0, u'friande': 0, u'défileuse': 2, u'crackeuse': 2, u'thoraco-abdominale': 3, u'asexuée': 0, u'glosso-pharyngienne': 4, u'hypotensive': 7, u'rectificative': 7, u'petite': 0, u'foulonnière': 6, u'migratrice': 1, u'baptisée': 0, u'coopérante': 0, u'décrépite': 0, u'surencombrée': 0, u'provocante': 0, u'nautonière': 6, u'multilobée': 0, u'togolaise': 0, u'négatrice': 1, u'olfactive': 7, u'délinquante': 0, u'bistrotière': 6, u'traquée': 0, u'graphiteuse': 9, u'récréative': 7, u'annonciatrice': 1, u'solière': 6, u'radoteuse': 2, u'grimacière': 6, u'façadière': 6, u'guadeloupéenne': 4, u'porteuse': 2, u'voiturière': 6, u'innomée': 0, u'eurafricaine': 0, u'adextrée': 0, u'cacaoyère': 6, u'diagonale': 3, u'anti-aérienne': 4, u'discordante': 0, u'mordue': 0, u'auto-excitatrice': 1, u'complète': 25, u'mahousse': 4, u'flagellante': 0, u'estampeuse': 2, u'infractionnelle': 4, u'radiative': 7, u'taboue': 0, u'pattue': 0, u'médiane': 0, u'engrêlée': 0, u'organicienne': 4, u'traversière': 6, u'occlusive': 7, u'irréalisée': 0, u'hydroxylée': 0, u'transrhénane': 0, u'carolingienne': 4, u'aberrante': 0, u'synchroniseuse': 2, u'galeuse': 9, u'désagrégeante': 0, u'corinthienne': 4, u'maraichère': 6, u'oratrice': 1, u'ensellée': 0, u'grêleuse': 9, u'nosocomiale': 3, u'schleue': 0, u'massive': 7, u'monocomposante': 0, u'bienveillante': 0, u'pure': 0, u'londonienne': 4, u'intentionnelle': 4, u'chtarbée': 0, u'clabaudeuse': 2, u'voltairienne': 4, u'covalente': 0, u'zimbabwéenne': 4, u'carbonée': 0, u'excipée': 0, u'décérébrante': 0, u'engraisseuse': 2, u'prétorienne': 4, u'quérulente': 0, u'rénale': 3, u'coûtante': 0, u'probabilisée': 0, u'madrée': 0, u'galetteuse': 9, u'phylogénéticienne': 4, u'momentanée': 0, u'appréciatrice': 1, u'bigoudène': 22, u'plante': 0, u'talibane': 0, u'ellipsoïdale': 3, u'santonnière': 6, u'onusienne': 4, u'subtropicale': 3, u'émoustillante': 0, u'millimétrée': 0, u'inscrite': 0, u'sibylline': 0, u'punisseuse': 2, u'recenseuse': 2, u'danseuse': 2, u'enrichissante': 0, u'postformée': 0, u'pollinisatrice': 1, u'expédiente': 0, u'émiratie': 0, u'chapardeuse': 2, u'corrompue': 0, u'artificière': 6, u'casuelle': 4, u'hypertensive': 7, u'bonzesse': 16, u'psychoéducatrice': 1, u'bouffante': 0, u'négative': 7, u'polonaise': 0, u'félonne': 4, u'innovante': 0, u'monténégrine': 0, u'providente': 0, u'dahoméenne': 4, u'rolleuse': 2, u'divulgatrice': 1, u'meurtrière': 6, u'vaccinée': 0, u'généticienne': 4, u'saliveuse': 9, u'provocatrice': 1, u'distractive': 7, u'resarcelée': 0, u'apicale': 3, u'orpheline': 0, u'cajoleuse': 2, u'rhizocarpée': 0, u'bouffeuse': 2, u'pervertisseuse': 2, u'bonne': 4, u'tanneuse': 2, u'parfaite': 0, u'follette': 4, u'appréhensive': 7, u'chloreuse': 9, u'inexpiée': 0, u'tenancière': 6, u'augustine': 0, u'franco-algérienne': 4, u'tousseuse': 2, u'bénissante': 0, u'nouveau-née': 0, u'surréelle': 4, u'tordeuse': 2, u'scoriacée': 0, u'basse-courière': 6, u'coûteuse': 9, u'post-coloniale': 3, u'sénégalaise': 0, u'onguiculée': 0, u'attractive': 7, u'équidistante': 0, u'gomorrhéenne': 4, u'mâcheuse': 2, u'fructicultrice': 1, u'hibernante': 0, u'soule': 0, u'promeneuse': 2, u'blanche': 4, u'antitumorale': 3, u'citatrice': 1, u'épinceteuse': 2, u'bledarde': 0, u'proprioceptive': 7, u'truqueuse': 2, u'inspirée': 0, u'paqueteuse': 2, u'cylindreuse': 2, u'suborneuse': 2, u'médailleuse': 2, u'fantomale': 3, u'mal-aimée': 0, u'loirétaine': 0, u'multiconfessionnelle': 4, u'judéo-allemande': 0, u'traditionnelle': 4, u'algonquine': 0, u'inculpée': 0, u'zinzoline': 0, u'bosseuse': 2, u'néo-nazie': 0, u'inhospitalière': 6, u'entéro-rénale': 3, u'crétoise': 0, u'périorale': 0, u'fictionnelle': 4, u'ci-incluse': 0, u'imminente': 0, u'pulsante': 0, u'ongulée': 0, u'inutilisée': 0, u'indifférenciée': 0, u'hélicicultrice': 1, u'zoroastrienne': 4, u'chaudronnée': 0, u'scieuse': 2, u'bimensuelle': 4, u'avaliseuse': 2, u'chevillarde': 0, u'gifleuse': 2, u'arrangeuse': 2, u'pointeuse': 2, u'virale': 3, u'vérolée': 0, u'baladeuse': 2, u'incestueuse': 9, u'albanaise': 0, u'inamicale': 3, u'ligulée': 0, u'caporale': 3, u'nonchalante': 0, u'cérébrospinale': 3, u'dilatante': 0, u'incontrôlée': 0, u'nulle': 4, u'axiale': 3, u'adéquate': 0, u'criante': 0, u'surdouée': 0, u'pionnière': 6, u'observante': 0, u'duodénale': 3, u'gourmée': 0, u'socialisante': 0, u'grisonnante': 0, u'administratrice': 1, u'paresseuse': 9, u'acineuse': 9, u'extéroceptive': 7, u'aigüe': 12, u'parenthésée': 0, u'monoclinale': 3, u'convaincante': 0, u'impuissante': 0, u'austro-américaine': 0, u'rationnelle': 4, u'archi-fausse': 15, u'subsidiante': 0, u'doseuse': 2, u'valencienne': 4, u'franco-turque': 13, u'thoracoabdominale': 3, u'radionavigante': 0, u'thermohaline': 0, u'modélisatrice': 1, u'calamiteuse': 9, u'capitularde': 0, u'hensélienne': 4, u'convoquée': 0, u'tartreuse': 9, u'claquette': 4, u'abréviatrice': 1, u'stanneuse': 9, u'colicitante': 0, u'épaisse': 4, u'boulevardière': 6, u'laudatrice': 1, u'armurée': 0, u'ogresse': 16, u'immunisante': 0, u'inégale': 3, u'impubliée': 0, u'minière': 6, u'fibromateuse': 9, u'cautérisante': 0, u'équivariante': 0, u'abstinente': 0, u'usurpatrice': 1, u'autoporteuse': 2, u'épiscopalienne': 4, u'ultra-légère': 6, u'intersexuelle': 4, u'favorisante': 0, u'supra-humaine': 0, u'filandreuse': 9, u'saccharinée': 0, u'ligérienne': 4, u'grisouteuse': 9, u'défoliante': 0, u'communicatrice': 1, u'caponne': 4, u'albuginée': 0, u'pultacée': 0, u'satineuse': 2, u'codeuse': 2, u'optimale': 3, u'ulcérative': 7, u'augustale': 3, u'routarde': 0, u'transalpine': 0, u'quantitative': 7, u'turbosoufflante': 0, u'lettone': 0, u'refendeuse': 2, u'arienne': 4, u'estivante': 0, u'harceleuse': 2, u'dano-islandaise': 0, u'tractive': 7, u'abotée': 0, u'déterreuse': 2, u'survive': 7, u'foraminée': 0, u'eurasienne': 4, u'dégraissante': 0, u'basse': 4, u'farinière': 6, u'gloussante': 0, u'cassante': 0, u'obsessive': 7, u'rewriteuse': 2, u'miraculeuse': 9, u'directionnelle': 4, u'célébrante': 0, u'détractrice': 1, u'flambeuse': 2, u'craqueuse': 2, u'russo-italienne': 4, u'sacrificielle': 4, u'nasale': 3, u'affranchisseuse': 2, u'sacerdotale': 3, u'franco-anglaise': 0, u'démente': 0, u'économétricienne': 4, u'chétive': 7, u'irritative': 7, u'marteleuse': 2, u'châtelaine': 0, u'tontinière': 6, u'salariée': 0, u'désaltérante': 0, u'croyante': 0, u'halogénée': 0, u'abélienne': 4, u'parentale': 3, u'lettrée': 0, u'dégoûtante': 0, u'charentaise': 0, u'bio-ingénieure': 0, u'polluposteuse': 2, u'scripturale': 3, u'affolée': 0, u'emmerdante': 0, u'natoufienne': 4, u'trébuchante': 0, u'nominative': 7, u'babillarde': 0, u'poulaine': 0, u'plasticienne': 4, u'équipière': 6, u'biacuminée': 0, u'embaumeuse': 2, u'mollassonne': 4, u'ardoiseuse': 9, u'soupière': 6, u'infectée': 0, u'chalande': 0, u'anglo-polonaise': 0, u'multifamiliale': 3, u'dodécagonale': 3, u'chagrine': 0, u'verbeuse': 9, u'déclinante': 0, u'copernicienne': 4, u'quarante-huitarde': 0, u'peloteuse': 2, u'dévoratrice': 1, u'indicative': 7, u'répulsive': 7, u'soleilleuse': 9, u'autopunitive': 7, u'cossarde': 0, u'unifoliée': 0, u'bissexuée': 0, u'vulcanienne': 4, u'proverbiale': 3, u'exaspérée': 0, u'romande': 0, u'matutinale': 3, u'définitionnelle': 4, u'vocale': 3, u'postpalatale': 3, u'thyristorisée': 0, u'guignarde': 0, u'agencière': 6, u'bouffonne': 4, u'tradeuse': 2, u'ultra-plate': 0, u'concentreuse': 2, u'payante': 0, u'buguée': 0, u'bien-pensante': 0, u'impersonnelle': 4, u'improductive': 7, u'concubine': 0, u'veineuse': 9, u'clocharde': 0, u'biquette': 4, u'retardée': 0, u'gerbante': 0, u'sous-équipée': 0, u'prénasalisée': 0, u'promotrice': 1, u'asphyxiante': 0, u'intégrée': 0, u'innocente': 0, u'disruptive': 7, u'teufeuse': 2, u'graveleuse': 9, u'gastroduodénale': 3, u'comédienne': 4, u'attardée': 0, u'casanière': 6, u'chuintante': 0, u'régressive': 7, u'passive': 7, u'variqueuse': 9, u'anglo-iranienne': 4, u'méprisante': 0, u'secoureuse': 2, u'romanisante': 0, u'danubienne': 4, u'théâtrale': 3, u'invitante': 0, u'caribéenne': 4, u'notatrice': 1, u'lignifiante': 0, u'obéissante': 0, u'comateuse': 9, u'cacaotée': 0, u'protéinée': 0, u'allante': 0, u'harcelante': 0, u'rétinienne': 4, u'latitudinale': 3, u'tocarde': 0, u'scrabbleuse': 2, u'saintongeaise': 0, u'interprétante': 0, u'dissolue': 0, u'franco-haïtienne': 4, u'serine': 0, u'cordée': 0, u'ethmoïdale': 3, u'traductionnelle': 4, u'chauffeuse': 2, u'complexuelle': 4, u'brachiale': 3, u'agnate': 0, u'coassociée': 0, u'contre-productive': 7, u'bloqueuse': 2, u'technico-commerciale': 3, u'carrière': 6, u'hypertendue': 0, u'digitée': 0, u'lozérienne': 4, u'corrosive': 7, u'épinglière': 6, u'aréneuse': 9, u'coagulatrice': 1, u'emmanchée': 0, u'chanvrière': 6, u'godillante': 0, u'tétragonale': 3, u'spacieuse': 9, u'abonnée': 0, u'lénitive': 7, u'vendeuse': 2, u'gesticulatrice': 1, u'zonale': 3, u'héliomarine': 0, u'anarchisante': 0, u'aguichante': 0, u'nitreuse': 9, u'génuflectrice': 1, u'ghanéenne': 4, u'interosseuse': 9, u'actuelle': 4, u'enquiquinante': 0, u'génoise': 0, u'exerçante': 0, u'sidérante': 0, u'ronde': 0, u'paraguayenne': 4, u'orientatrice': 1, u'salonnarde': 0, u'maritale': 3, u'interministérielle': 4, u'hibernale': 3, u'parurière': 6, u'prémenstruelle': 4, u'pronominale': 3, u'coquine': 0, u'événementielle': 4, u'contestatrice': 1, u'totalisatrice': 1, u'furfuracée': 0, u'invasive': 7, u'éclaboussante': 0, u'amnistiante': 0, u'approchante': 0, u'vénitienne': 4, u'inoccupée': 0, u'participiale': 3, u'correctrice': 1, u'scrutatrice': 1, u'moralisante': 0, u'mystérieuse': 9, u'autorégulatrice': 1, u'azerbaïdjanaise': 0, u'minette': 4, u'charronne': 4, u'ivoirine': 0, u'électroacousticienne': 4, u'principale': 3, u'inétendue': 0, u'pomicultrice': 1, u'areligieuse': 9, u'dansante': 0, u'antillaise': 0, u'chartreuse': 9, u'insultée': 0, u'super-lourde': 0, u'accidentée': 0, u'composante': 0, u'désapprobatrice': 1, u'régénératrice': 1, u'ralliée': 0, u'boriquée': 0, u'manageuse': 2, u'humérale': 3, u'tupie': 0, u'distillatrice': 1, u'chargeuse': 2, u'adoptive': 7, u'porcelainière': 6, u'soupçonneuse': 9, u'greneuse': 2, u'muscinale': 3, u'guillerette': 4, u'abeillère': 6, u'interreliée': 0, u'phrygienne': 4, u'surgénératrice': 1, u'nancéienne': 4, u'postale': 3, u'acéricultrice': 1, u'centauresse': 16, u'habituée': 0, u'villafranchienne': 4, u'sulfurisée': 0, u'indésirée': 0, u'expatriée': 0, u'simplette': 4, u'fractionnelle': 4, u'séraphine': 0, u'paroissiale': 3, u'trigonale': 3, u'conspiratrice': 1, u'ponantaise': 0, u'fougueuse': 9, u'percheronne': 4, u'mésothéliale': 3, u'verviétoise': 0, u'colauréate': 0, u'sénégambienne': 4, u'lécheuse': 2, u'spéciale': 3, u'brune': 0, u'poinçonneuse': 2, u'métaxiale': 3, u'passée': 0, u'soudanienne': 4, u'rassembleuse': 2, u'thaïlandaise': 0, u'chatouilleuse': 2, u'incompétente': 0, u'marronne': 4, u'marmottante': 0, u'corporative': 7, u'germano-autrichienne': 4, u'maïsicultrice': 1, u'couchante': 0, u'colloïdale': 3, u'polyfonctionnelle': 4, u'assesseure': 0, u'fréquentative': 7, u'squirreuse': 9, u'drue': 0, u'résiliente': 0, u'altérante': 0, u'écossaise': 0, u'paranormale': 3, u'crayeuse': 9, u'endurante': 0, u'gutturale': 3, u'rétrosternale': 3, u'copulative': 7, u'interdimensionnelle': 4, u'cycloïdale': 3, u'révoltée': 0, u'éthicienne': 4, u'perlinguale': 3, u'kolkhozienne': 4, u'rebutante': 0, u'ombilicale': 3, u'gardienne': 4, u'bouseuse': 9, u'sabotière': 6, u'mucronée': 0, u'adjective': 7, u'acadienne': 4, u'lionnée': 0, u'tronçonneuse': 2, u'parigote': 0, u'arabisante': 0, u'contrastive': 7, u'anthraciteuse': 9, u'désosseuse': 2, u'composeuse': 2, u'superviseuse': 2, u'existante': 0, u'biseauteuse': 2, u'crispante': 0, u'aiguilleuse': 2, u'nivéale': 3, u'hégirienne': 4, u'teutonne': 4, u'méditerranée': 0, u'chienne': 4, u'collectionneuse': 2, u'riemannienne': 4, u'tarpéienne': 4, u'verte': 0, u'naisseuse': 2, u'nivopluviale': 3, u'markéteuse': 2, u'exaltante': 0, u'chlorophyllienne': 4, u'mendiante': 0, u'polyamoureuse': 2, u'franchiseuse': 2, u'aborale': 3, u'aviveuse': 2, u'discutante': 0, u'maxillofaciale': 3, u'rwandaise': 0, u'lugeuse': 2, u'offensée': 0, u'élastiquée': 0, u'dauphinoise': 0, u'désorganisatrice': 1, u'terrienne': 4, u'slalomeuse': 2, u'niaiseuse': 9, u'moulineuse': 2, u'déroulante': 0, u'épidurale': 3, u'irréfutée': 0, u'rhodienne': 4, u'latérale': 3, u'divagatrice': 1, u'mourante': 0, u'butyreuse': 9, u'sigillée': 0, u'voisée': 0, u'mendigote': 0, u'excito-motrice': 1, u'humiliante': 0, u'violeuse': 2, u'bilabiée': 0, u'maçonne': 4, u'exilée': 0, u'colleteuse': 2, u'branchue': 0, u'féodale': 3, u'pensive': 7, u'brutale': 3, u'maffieuse': 9, u'marbrière': 6, u'assoiffée': 0, u'éternelle': 4, u'consule': 0, u'serrurière': 6, u'pectorale': 3, u'déconstruisante': 0, u'oscillo-battante': 0, u'tubuleuse': 9, u'oublieuse': 9, u'croqueuse': 2, u'connée': 0, u'tordante': 0, u'cocompacte': 0, u'dépliante': 0, u'bisexuelle': 4, u'louvoyante': 0, u'charbonneuse': 9, u'suiveuse': 2, u'pharyngienne': 4, u'audiencière': 6, u'séquestrée': 0, u'évaluative': 7, u'perlière': 6, u'pédocriminelle': 4, u'disjonctive': 7, u'multi-catégorielle': 4, u'vosgienne': 4, u'conchoïdale': 3, u'conclusive': 7, u'colonisatrice': 1, u'éveilleuse': 2, u'prémontrée': 0, u'craniofaciale': 3, u'hospitalière': 6, u'antipodale': 3, u'engagée': 0, u'enguichée': 0, u'profective': 7, u'experte': 0, u'ennuyeuse': 9, u'benoite': 0, u'parente': 0, u'ismaélienne': 4, u'guindailleuse': 2, u'crurale': 3, u'lapone': 0, u'malvenue': 0, u'concupiscente': 0, u'décennale': 3, u'extra-légale': 3, u'abrutie': 0, u'alépine': 0, u'amiantée': 0, u'antipersonnelle': 4, u'courtisane': 0, u'mensongère': 6, u'spinalienne': 4, u'glucosée': 0, u'politicarde': 0, u'rétive': 7, u'miraculée': 0, u'vitale': 3, u'antinationale': 3, u'contredisante': 0, u'pierreuse': 9, u'positionnelle': 4, u'tracassante': 0, u'stockeuse': 2, u'glougloutante': 0, u'tenante': 0, u'sautillante': 0, u'cooccupante': 0, u'horrifiante': 0, u'osculatrice': 1, u'soûlote': 0, u'acheuléenne': 4, u'contraignante': 0, u'proactive': 7, u'correctionnelle': 4, u'dépénalisante': 0, u'oxydante': 0, u'couseuse': 2, u'thermopropulsive': 7, u'enculée': 0, u'déroutante': 0, u'patronnière': 6, u'seconde': 0, u'somptueuse': 9, u'obsédante': 0, u'parisienne': 4, u'titularisée': 0, u'bouillonnante': 0, u'duale': 3, u'désignative': 7, u'foudroyante': 0, u'grenue': 0, u'recruteuse': 2, u'vérifieuse': 2, u'redistributive': 7, u'prévalente': 0, u'confédérée': 0, u'annulative': 7, u'aminée': 0, u'formeuse': 2, u'logicielle': 4, u'ultradienne': 4, u'écocertifiée': 0, u'vaporeuse': 9, u'bêtatesteuse': 2, u'pantouflière': 6, u'sentencieuse': 9, u'champignonneuse': 9, u'possessionnelle': 4, u'poursuiveuse': 2, u'imperméabilisante': 0, u'superlégère': 6, u'exagératrice': 1, u'contre-maîtresse': 16, u'consubstantielle': 4, u'hockeyeuse': 2, u'originelle': 4, u'finasseuse': 2, u'gustative': 7, u'informelle': 4, u'superchampionne': 4, u'électroportative': 7, u'non-voyante': 0, u'perfoliée': 0, u'clitoridienne': 4, u'pardonnante': 0, u'soûlante': 0, u'fagnarde': 0, u'rabâcheuse': 2, u'véreuse': 9, u'mitoyenne': 4, u'hypercontinentale': 3, u'plaintive': 7, u'centralienne': 4, u'organophosphorée': 0, u'semestrielle': 4, u'pesante': 0, u'magnésienne': 4, u'saligaude': 0, u'drôlesse': 16, u'effluente': 0, u'épithéliale': 3, u'gardoise': 0, u'appalachienne': 4, u'nociceptive': 7, u'caquetante': 0, u'assistante': 0, u'péridurale': 3, u'quinaude': 0, u'inattaquée': 0, u'digestive': 7, u'ambigüe': 12, u'fugitive': 7, u'psychométricienne': 4, u'aligneuse': 2, u'balafrée': 0, u'tomenteuse': 9, u'sensuelle': 4, u'gibeline': 0, u'poignante': 0, u'modulante': 0, u'colitigante': 0, u'autorisatrice': 1, u'ouest-allemande': 0, u'inféconde': 0, u'rappelée': 0, u'chondrocostale': 3, u'admonitrice': 1, u'miroitée': 0, u'chocolatière': 6, u'potagère': 6, u'nictitante': 0, u'pétrée': 0, u'potestative': 7, u'extrasensorielle': 4, u'superlourde': 0, u'décorative': 7, u'téléactrice': 1, u'recercelée': 0, u'oubliée': 0, u'néozélandaise': 0, u'architravée': 0, u'addisonienne': 4, u'piteuse': 9, u'hachichine': 0, u'vicomtale': 3, u'auriculo-temporale': 3, u'skippeuse': 2, u'pacifiante': 0, u'survolante': 0, u'connecticienne': 4, u'acrimonieuse': 9, u'circassienne': 4, u'victorieuse': 9, u'moussante': 0, u'rassasiante': 0, u'scribouilleuse': 2, u'vergogneuse': 9, u'fascisante': 0, u'jeunette': 4, u'interannuelle': 4, u'aumônière': 6, u'improvisatrice': 1, u'définissante': 0, u'massacreuse': 2, u'indécise': 0, u'pyrograveuse': 2, u'fortunée': 0, u'golfeuse': 2, u'rétrospective': 7, u'immunodéficiente': 0, u'museuse': 2, u'jovienne': 4, u'frotteuse': 2, u'soiffarde': 0, u'délictueuse': 9, u'grondeuse': 2, u'enquêtée': 0, u'considérative': 7, u'aérospatiale': 3, u'dissidente': 0, u'bourdonnante': 0, u'ramasseuse': 2, u'maladroite': 0, u'inécoutée': 0, u'intersectorielle': 4, u'verglaçante': 0, u'significatrice': 1, u'dentofaciale': 3, u'adaptatrice': 1, u'soucieuse': 9, u'occitane': 0, u'fondatrice': 1, u'obsessionnelle': 4, u'automaticienne': 4, u'géophysicienne': 4, u'bismuthée': 0, u'socio-éducative': 7, u'cérébrale': 3, u'isospectrale': 3, u'contrélectromotrice': 1, u'régulatrice': 1, u'ensemblière': 6, u'frondeuse': 2, u'ammoniaquée': 0, u'grivoise': 0, u'pénitentielle': 4, u'marsupiale': 3, u'introductive': 7, u'aventurière': 6, u'ricanante': 0, u'franco-lettonne': 4, u'interdépartementale': 3, u'conditionneuse': 2, u'scoute': 0, u'bourgeonneuse': 9, u'hitlérienne': 4, u'malintentionnée': 0, u'indochinoise': 0, u'campinoise': 0, u'littérale': 3, u'bavochée': 0, u'ornementale': 3, u'imprécise': 0, u'néoconservatrice': 1, u'inconvenante': 0, u'bruiteuse': 2, u'adulescente': 0, u'équato-guinéenne': 4, u'accrocheuse': 2, u'abstractive': 7, u'tombeuse': 2, u'enfourneuse': 2, u'illustratrice': 1, u'contaminatrice': 1, u'recluse': 0, u'jointive': 7, u'huppée': 0, u'monacale': 3, u'aciéreuse': 9, u'traînante': 0, u'idéo-motrice': 1, u'vigneronne': 4, u'expulsive': 7, u'vivrée': 0, u'zélée': 0, u'lustrale': 3, u'marxisante': 0, u'martyre': 0, u'pliante': 0, u'viscérale': 3, u'distancielle': 4, u'néronienne': 4, u'décemvirale': 3, u'minéralisante': 0, u'lectrice': 1, u'inaugurale': 3, u'éthérée': 0, u'murie': 0, u'sternocléidomastoïdienne': 4, u'tempérante': 0, u'canonnière': 6, u'consonante': 0, u'hydrominérale': 3, u'correspondante': 0, u'côtière': 6, u'taularde': 0, u'suintante': 0, u'agrandissante': 0, u'imparidigitée': 0, u'déclarante': 0, u'blessée': 0, u'costumière': 6, u'larmoyeuse': 2, u'combinarde': 0, u'phosphorée': 0, u'médicoprofessionnelle': 4, u'affouagère': 6, u'graduelle': 4, u'finaude': 0, u'interindividuelle': 4, u'anémiante': 0, u'regardeuse': 2, u'ligamenteuse': 9, u'coadjutrice': 1, u'soutireuse': 2, u'divise': 0, u'niçoise': 0, u'prochaine': 0, u'méningée': 0, u'horsaine': 0, u'aérotransportée': 0, u'respectueuse': 9, u'caveuse': 2, u'abiétine': 0, u'radine': 0, u'écrasante': 0, u'veveysanne': 4, u'transafricaine': 0, u'tamponneuse': 2, u'rationalisante': 0, u'corruptrice': 1, u'pythienne': 4, u'transpositrice': 1, u'désabonnée': 0, u'ressemeleuse': 2, u'moghole': 0, u'mignonne': 4, u'cartonneuse': 9, u'fonctorielle': 4, u'indignée': 0, u'mytilicultrice': 1, u'vermiculée': 0, u'vectorielle': 4, u'mentonnière': 6, u'inactive': 7, u'irrationnelle': 4, u'souricière': 6, u'collective': 7, u'radio-active': 7, u'détraquée': 0, u'alcalinoterreuse': 9, u'apprentie': 0, u'banquière': 6, u'interconfessionnelle': 4, u'offensante': 0, u'gréco-romaine': 0, u'obstétricienne': 4, u'nippo-américaine': 0, u'américano-iranienne': 4, u'dribbleuse': 2, u'pharyngale': 3, u'revivifiante': 0, u'menine': 0, u'recrue': 0, u'namuroise': 0, u'fumeuse': 9, u'vanillée': 0, u'ministérielle': 4, u'consolatrice': 1, u'déprédatrice': 1, u'cyclonale': 3, u'junonienne': 4, u'arsenicale': 3, u'affainéantie': 0, u'salmonicultrice': 1, u'soufflée': 0, u'surcostale': 3, u'cistercienne': 4, u'basochienne': 4, u'conférencière': 6, u'modératrice': 1, u'besogneuse': 9, u'comparatrice': 1, u'artisanale': 3, u'contagieuse': 9, u'bondieusarde': 0, u'américano-canadienne': 4, u'angoissante': 0, u'synagogale': 3, u'hennissante': 0, u'obsidionale': 3, u'interreligieuse': 9, u'tardillonne': 4, u'étrangleuse': 2, u'accastilleuse': 2, u'incapacitante': 0, u'baby-boomeuse': 2, u'immigrée': 0, u'médico-chirurgicale': 3, u'écorchée': 0, u'piégeuse': 9, u'occidentale': 3, u'bahamienne': 4, u'franco-polonaise': 0, u'insidieuse': 9, u'décevante': 0, u'roulante': 0, u'lamelleuse': 9, u'auditive': 7, u'transcanadienne': 4, u'unidimensionnelle': 4, u'tremblante': 0, u'squatteuse': 2, u'dératiseuse': 2, u'araméenne': 4, u'farfelue': 0, u'facultative': 7, u'pyrénéenne': 4, u'penaude': 0, u'arriérée': 0, u'compilatrice': 1, u'dégradante': 0, u'inimitée': 0, u'gobeletière': 6, u'barbadienne': 4, u'partante': 0, u'télétravailleuse': 2, u'lotisseuse': 2, u'contiguë': 0, u'cultivatrice': 1, u'tibio-tarsienne': 4, u'stibiée': 0, u'revancharde': 0, u'vivifiante': 0, u'façonnière': 6, u'anglo-irakienne': 4, u'ultra-marine': 0, u'belgo-luxembourgeoise': 0, u'tombale': 3, u'férale': 3, u'charcutière': 6, u'fouilleuse': 2, u'germinatrice': 1, u'guignole': 0, u'douloureuse': 9, u'épinceuse': 2, u'tutrice': 1, u'acidifiante': 0, u'fastigiée': 0, u'chatte': 4, u'sterno-cléido-mastoïdienne': 4, u'intruse': 0, u'maraichine': 0, u'retrayante': 0, u'linnéenne': 4, u'dénominative': 7, u'maldivienne': 4, u'déchargeuse': 2, u'radioélectricienne': 4, u'pentathlonienne': 4, u'moricaude': 0, u'anthropisée': 0, u'dilatatrice': 1, u'cuproammoniacale': 3, u'fricoteuse': 2, u'bétonnière': 6, u'lusitanienne': 4, u'onctueuse': 9, u'compacte': 0, u'prieure': 0, u'microtonale': 0, u'coléreuse': 9, u'omnipotente': 0, u'actionnariale': 3, u'surveillante': 0, u'supranormale': 3, u'cémenteuse': 9, u'lotoise': 0, u'arrobase': 0, u'plaisancière': 6, u'amatrice': 1, u'envolée': 0, u'susdite': 0, u'paire': 0, u'malfamée': 0, u'connarde': 0, u'hérissonne': 4, u'tagueuse': 2, u'adjointe': 0, u'banale': 3, u'miauleuse': 2, u'pasteurienne': 4, u'haïe': 0, u'anormale': 3, u'visitatrice': 1, u'imagière': 6, u'redoublante': 0, u'beylicale': 3, u'mantelée': 0, u'coïncidente': 0, u'confessante': 0, u'pascalienne': 4, u'fourbue': 0, u'trayeuse': 2, u'craintive': 7, u'sarrasine': 0, u'râleuse': 2, u'lysosomale': 3, u'têtue': 0, u'aïeule': 0, u'paillarde': 0, u'flamingante': 0, u'abrutissante': 0, u'abrupte': 0, u'roubaisienne': 4, u'aventureuse': 9, u'bâfreuse': 2, u'huronienne': 4, u'fouetteuse': 2, u'railleuse': 2, u'mularde': 0, u'alizée': 0, u'limonadière': 6, u'gamine': 0, u'insoucieuse': 9, u'limeuse': 2, u'manichéenne': 4, u'lustrante': 0, u'comportementale': 3, u'tendineuse': 9, u'entrante': 0, u'spécificatrice': 1, u'lente': 0, u'maraîchère': 6, u'erronée': 0, u'moniale': 3, u'industrielle': 4, u'cartomancienne': 4, u'glorificatrice': 1, u'câblière': 6, u'cirière': 6, u'visqueuse': 9, u'amollissante': 0, u'gonflante': 0, u'uropygiale': 3, u'susmentionnée': 0, u'arlésienne': 4, u'déposante': 0, u'instante': 0, u'insouciante': 0, u'adamantine': 0, u'probante': 0, u'incuite': 0, u'contemporaine': 0, u'atténuante': 0, u'conversationnelle': 4, u'dorée': 0, u'surinamaise': 0, u'prévenante': 0, u'troyenne': 4, u'palestinienne': 4, u'regrattière': 6, u'clandestine': 0, u'juridictionnelle': 4, u'interdigitale': 3, u'luronne': 4, u'textuelle': 4, u'meilleure': 0, u'transversale': 3, u'richarde': 0, u'boutonnière': 6, u'draineuse': 2, u'bien-voulue': 0, u'acheteuse': 2, u'consanguine': 0, u'monomotrice': 1, u'embrigadée': 0, u'germaine': 0, u'chantante': 0, u'soignante': 0, u'flavescente': 0, u'furibarde': 0, u'léopardée': 0, u'marseillaise': 0, u'caudale': 3, u'décurrente': 0, u'anodiseuse': 2, u'balancière': 6, u'polyinsaturée': 0, u'costaricienne': 4, u'tapante': 0, u'cadrante': 0, u'sexagésimale': 3, u'détenue': 0, u'toxicodépendante': 0, u'ricinée': 0, u'clavelée': 0, u'abandonnatrice': 1, u'rêveuse': 2, u'arbalétrière': 6, u'aérodigestive': 7, u'achemineuse': 2, u'rôtisseuse': 2, u'mexico-américaine': 0, u'recombinante': 0, u'grenobloise': 0, u'informatrice': 1, u'seule': 0, u'dénigreuse': 2, u'qualitative': 7, u'préformatée': 0, u'terrifiante': 0, u'duveteuse': 9, u'koweïtienne': 4, u'amidonnière': 6, u'torse': 0, u'sculptrice': 1, u'flamande': 0, u'imparfaite': 0, u'wolofe': 0, u'laponne': 4, u'assyro-babylonienne': 4, u'tamoule': 0, u'sclérale': 3, u'inconsidérée': 0, u'doucette': 4, u'résolutive': 7, u'caméléonienne': 4, u'inemployée': 0, u'érosive': 7, u'mensuelle': 4, u'carentielle': 4, u'demi-morte': 0, u'léonine': 0, u'contentive': 7, u'plaideuse': 2, u'remuante': 0, u'freudienne': 4, u'immunocompétente': 0, u'iridescente': 0, u'phéniquée': 0, u'passionnante': 0, u'grammaticale': 3, u'tiercée': 0, u'sélecte': 0, u'dominiquaise': 0, u'sienne': 4, u'éleveuse': 2, u'chilienne': 4, u'chryséléphantine': 0, u'nubienne': 4, u'crépue': 0, u'possédante': 0, u'kanake': 0, u'anesthésiée': 0, u'gouvernante': 0, u'médulleuse': 9, u'superfine': 0, u'genouillée': 0, u'incurieuse': 9, u'expropriée': 0, u'pantropicale': 3, u'argilosiliceuse': 9, u'intergouvernementale': 3, u'reconnaissante': 0, u'prêcheuse': 2, u'jardinière': 6, u'moins-disante': 0, u'défenseure': 0, u'nycthémérale': 3, u'disculpée': 0, u'franco-afghane': 0, u'braqueuse': 2, u'pépiante': 0, u'métropolitaine': 0, u'zazoue': 0, u'nuitarde': 0, u'partielle': 4, u'volumineuse': 9, u'filamenteuse': 9, u'rizicultrice': 1, u'chiromancienne': 4, u'psychostimulante': 0, u'californienne': 4, u'bredouilleuse': 2, u'chapée': 0, u'oculomotrice': 1, u'belgo-allemande': 0, u'rockeuse': 2, u'échiquéenne': 4, u'sclérifiante': 0, u'sucrière': 6, u'sorbonnarde': 0, u'agente': 0, u'vasarde': 0, u'gondolière': 6, u'grévicultrice': 1, u'radicante': 0, u'exploitante': 0, u'soussignée': 0, u'menuisière': 6, u'enduiseuse': 2, u'enchanteresse': 20, u'indonésienne': 4, u'aiguë': 0, u'berlinoise': 0, u'navetteuse': 2, u'colombine': 0, u'levantine': 0, u'parachutée': 0, u'prudente': 0, u'ampliative': 7, u'fuchsienne': 4, u'ronflante': 0, u'armurière': 6, u'bissexuelle': 4, u'colorante': 0, u'pansue': 0, u'épicontinentale': 3, u'hispanisante': 0, u'fastueuse': 9, u'supérieure': 0, u'géomaticienne': 4, u'rétrécissante': 0, u'neurale': 3, u'allumée': 0, u'calédonienne': 4, u'astucieuse': 9, u'intransitive': 7, u'normalienne': 4, u'incubatrice': 1, u'planificatrice': 1, u'interronégative': 7, u'narrative': 7, u'saignante': 0, u'korrigane': 0, u'archiconnue': 0, u'aînée': 0, u'abdomino-génitale': 3, u'gastro-œsophagienne': 4, u'logicienne': 4, u'lessivielle': 4, u'hallstattienne': 4, u'présomptueuse': 9, u'pulpeuse': 9, u'générative': 7, u'consentante': 0, u'masquante': 0, u'oiseuse': 9, u'livreuse': 2, u'feuillardière': 6, u'feuillue': 0, u'cinglée': 0, u'cocréatrice': 1, u'intrigante': 0, u'dormitive': 7, u'gringolée': 0, u'émissive': 7, u'coupeuse': 2, u'pseudo-membraneuse': 9, u'francisante': 0, u'préservative': 7, u'opaline': 0, u'encroutante': 0, u'infectieuse': 9, u'caresseuse': 2, u'sanglante': 0, u'lobotomisante': 0, u'prématurée': 0, u'dépréciative': 7, u'opposée': 0, u'recouvrante': 0, u'jeunotte': 4, u'indienne': 4, u'affidée': 0, u'chthonienne': 4, u'murine': 0, u'polluante': 0, u'aromatisante': 0, u'berceuse': 2, u'concernée': 0, u'inédite': 0, u'béninoise': 0, u'galopeuse': 2, u'spécieuse': 9, u'raquetteuse': 2, u'multivariée': 0, u'circadienne': 4, u'chainetière': 6, u'ricaine': 0, u'chaudronnière': 6, u'échotière': 6, u'volatile': 0, u'pâlichonne': 4, u'ocellée': 0, u'bâtisseuse': 2, u'leadeuse': 2, u'lutéale': 3, u'interorbitale': 3, u'empêcheuse': 2, u'bivalente': 0, u'marrie': 0, u'uniformisante': 0, u'ampoulée': 0, u'tympanale': 3, u'une': 0, u'scapulo-humérale': 3, u'alternative': 7, u'subnormale': 3, u'hébraïsante': 0, u'tiqueuse': 2, u'contadine': 0, u'inconstitutionnelle': 4, u'entreprenante': 0, u'havanaise': 0, u'bosnienne': 4, u'extemporanée': 0, u'festivalière': 6, u'dévotieuse': 9, u'heureuse': 9, u'islandaise': 0, u'ultra-libérale': 3, u'interrogée': 0, u'latine': 0, u'castillonnaise': 0, u'obstétricale': 3, u'périurbaine': 0, u'teigneuse': 9, u'gironde': 0, u'surnaturelle': 4, u'franco-panaméenne': 4, u'perturbatrice': 1, u'domaniale': 3, u'surproductive': 7, u'bottière': 6, u'canardière': 6, u'certifiante': 0, u'retraitée': 0, u'prorogative': 7, u'tréflée': 0, u'enjambeuse': 2, u'révoltante': 0, u'jaillissante': 0, u'nombreuse': 9, u'étourdissante': 0, u'lavante': 0, u'réactionnelle': 4, u'sadducéenne': 4, u'tatillonne': 4, u'hésitante': 0, u'phéromonale': 3, u'primodélinquante': 0, u'mafieuse': 9, u'flippante': 0, u'pigeonne': 4, u'dysfonctionnelle': 4, u'entraîneuse': 2, u'démolisseuse': 2, u'mateuse': 2, u'pocharde': 0, u'syncopale': 3, u'sous-préfectorale': 3, u'ouvrière': 6, u'toscane': 0, u'officinale': 3, u'agonisante': 0, u'limitative': 7, u'diophantienne': 4, u'amélioratrice': 1, u'cornue': 0, u'albigeoise': 0, u'loupiote': 0, u'empoignante': 0, u'factieuse': 9, u'décalcifiante': 0, u'extradée': 0, u'boraine': 0, u'équarrisseuse': 2, u'écoutante': 0, u'décontractante': 0, u'résistante': 0, u'cybersquatteuse': 2, u'native': 7, u'fonceuse': 2, u'immersive': 7, u'envouteuse': 2, u'géomancienne': 4, u'pétaradante': 0, u'équationnelle': 4, u'incarnadine': 0, u'incohérente': 0, u'protestante': 0, u'extravagante': 0, u'ébouleuse': 9, u'vipérine': 0, u'lorraine': 0, u'régularisatrice': 1, u'colleuse': 2, u'épilogueuse': 2, u'cliente': 0, u'actancielle': 4, u'déplaisante': 0, u'ergoteuse': 2, u'basanée': 0, u'bathyale': 3, u'suroxygénée': 0, u'performative': 7, u'brigadière': 6, u'appelée': 0, u'égaliseuse': 2, u'voyante': 0, u'cantonale': 3, u'semi-conductrice': 1, u'parapublique': 13, u'néo-grecque': 29, u'subulée': 0, u'citoyenne': 4, u'majorante': 0, u'ourdoue': 0, u'dépourvue': 0, u'prêtresse': 16, u'adjudante': 0, u'papoue': 0, u'virile': 0, u'würmienne': 4, u'bourrative': 7, u'tardive': 7, u'brestoise': 0, u'volumatrice': 1, u'mort-née': 0, u'saurisseuse': 2, u'brasseuse': 2, u'dissimulatrice': 1, u'néocalédonienne': 4, u'désespérée': 0, u'décolorante': 0, u'ébarbeuse': 2, u'annulatrice': 1, u'parapharmacienne': 4, u'braillarde': 0, u'rusée': 0, u'costarmoricaine': 0, u'seulette': 4, u'regardante': 0, u'trottinante': 0, u'motivatrice': 1, u'post-industrielle': 4, u'grognonne': 4, u'crawlée': 0, u'phocéenne': 4, u'non-comparante': 0, u'convolutée': 0, u'puinée': 0, u'multinationale': 3, u'soyeuse': 9, u'béotienne': 4, u'propulsive': 7, u'formante': 0, u'croupale': 3, u'condescendante': 0, u'contraceptive': 7, u'hackeuse': 2, u'phylloxérienne': 4, u'coaxiale': 3, u'babyboumeuse': 2, u'vénézuélienne': 4, u'bouletée': 0, u'absolue': 0, u'cabotine': 0, u'trouillarde': 0, u'bipennée': 0, u'non-combattante': 0, u'apparente': 0, u'inintelligente': 0, u'lexicale': 3, u'englobante': 0, u'genevoise': 0, u'antichrétienne': 4, u'anticoncurrentielle': 4, u'suédée': 0, u'austro-hongroise': 0, u'papilleuse': 9, u'chaleureuse': 9, u'vile': 0, u'ordurière': 6, u'papillonneuse': 2, u'terrière': 6, u'frisquette': 4, u'frôleuse': 2, u'princesse': 16, u'germano-polonaise': 0, u'bileuse': 9, u'coquelucheuse': 9, u'obsolescente': 0, u'émigrante': 0, u'hiatale': 3, u'tuante': 0, u'compensatrice': 1, u'continentale': 3, u'dissolvante': 0, u'roturière': 6, u'transitive': 7, u'coordinatrice': 1, u'écumante': 0, u'associative': 7, u'moyée': 0, u'trembleuse': 2, u'statisticienne': 4, u'kalmouke': 0, u'visiteuse': 2, u'indue': 0, u'latifoliée': 0, u'profuse': 0, u'oblate': 0, u'triomphale': 3, u'égreneuse': 2, u'transneptunienne': 4, u'montpelliéraine': 0, u'disproportionnelle': 4, u'grenailleuse': 2, u'enchérisseuse': 2, u'latinisante': 0, u'reportrice': 1, u'israélienne': 4, u'branlante': 0, u'récurrente': 0, u'ripailleuse': 2, u'sulfureuse': 9, u'germinative': 7, u'sacchareuse': 9, u'somalie': 0, u'suractive': 7, u'gentillette': 4, u'segmentale': 3, u'minoenne': 4, u'snifeuse': 2, u'langagière': 6, u'constructrice': 1, u'antirétrovirale': 3, u'ozonée': 0, u'désastreuse': 9, u'birmane': 0, u'caloyère': 6, u'lyrée': 0, u'initiatrice': 1, u'tanzanienne': 4, u'celtisante': 0, u'ecclésiale': 3, u'affabulatrice': 1, u'séreuse': 9, u'magdalénienne': 4, u'galiléenne': 4, u'amérindienne': 4, u'salariale': 3, u'hémolysante': 0, u'valaisanne': 4, u'votante': 0, u'croissantière': 6, u'ténue': 0, u'camionneuse': 2, u'accouveuse': 2, u'inexaucée': 0, u'sondée': 0, u'brabançonne': 4, u'levrettée': 0, u'décidue': 0, u'clonale': 3, u'fulminante': 0, u'décrocheuse': 2, u'pitchounette': 4, u'italianisante': 0, u'wisigothe': 0, u'boulette': 4, u'jolie': 0, u'fondée': 0, u'régente': 0, u'algéroise': 0, u'métisse': 4, u'minimexée': 0, u'antiacarienne': 4, u'adénovirale': 3, u'conquérante': 0, u'zététicienne': 4, u'émotteuse': 2, u'canneuse': 2, u'hollywoodienne': 4, u'méritante': 0, u'psychosensorimotrice': 1, u'incréée': 0, u'toilière': 6, u'fruitière': 6, u'granuleuse': 9, u'épistolière': 6, u'hypotendue': 0, u'nigaude': 0, u'non-fumeuse': 2, u'braillante': 0, u'cracheuse': 2, u'sourde': 0, u'improbatrice': 1, u'cicatrisante': 0, u'marémotrice': 1, u'domoticienne': 4, u'grosse': 4, u'resplendissante': 0, u'dialogueuse': 2, u'menaçante': 0, u'ablative': 7, u'plieuse': 2, u'adversative': 7, u'ferroutière': 6, u'mosaïquée': 0, u'fourreuse': 2, u'survitaminée': 0, u'déconneuse': 2, u'renouvelante': 0, u'indicielle': 4, u'immunodéprimée': 0, u'animalière': 6, u'carreautée': 0, u'carpicultrice': 1, u'supraconductrice': 1, u'herborisée': 0, u'chamane': 0, u'porcine': 0, u'adultérine': 0, u'prédécédée': 0, u'fiscale': 3, u'crémeuse': 9, u'héritée': 0, u'foulante': 0, u'hideuse': 9, u'coulombienne': 4, u'skiffeuse': 2, u'transpyrénéenne': 4, u'appétitive': 7, u'bousilleuse': 2, u'frémissante': 0, u'ponctuelle': 4, u'crâneuse': 2, u'barlongue': 4, u'shogunale': 3, u'gauchère': 6, u'photograveuse': 2, u'sapientiale': 3, u'prépositive': 7, u'cochère': 6, u'suppliante': 0, u'excitée': 0, u'savoyarde': 0, u'cambrioleuse': 2, u'mordicante': 0, u'ferreuse': 9, u'franco-iranienne': 4, u'insultante': 0, u'gueularde': 0, u'désambigüisatrice': 1, u'bantoue': 0, u'tropicale': 3, u'débirentière': 6, u'durale': 3, u'crucifiée': 0, u'tangente': 0, u'paludière': 6, u'agrotechnicienne': 4, u'roselière': 6, u'vieillissante': 0, u'praticienne': 4, u'rhétienne': 4, u'glaciale': 3, u'exquise': 0, u'bégüe': 12, u'cortico-surrénale': 3, u'avisée': 0, u'convalescente': 0, u'vitelline': 0, u'tentatrice': 1, u'salienne': 4, u'ruchée': 0, u'viagère': 6, u'imbrulée': 0, u'croustilleuse': 9, u'pourrisseuse': 2, u'massière': 6, u'vireuse': 9, u'jacobine': 0, u'colossale': 3, u'traversante': 0, u'congrue': 0, u'innommée': 0, u'bienfaisante': 0, u'infractrice': 1, u'dénonciatrice': 1, u'balourde': 0, u'insaturée': 0, u'clignotante': 0, u'injuriée': 0, u'polisseuse': 2, u'capitulée': 0, u'staminée': 0, u'binomiale': 3, u'palmipartie': 0, u'haversienne': 4, u'conglomérale': 3, u'angoissée': 0, u'branquignole': 0, u'médico-sportive': 7, u'prédicatrice': 1, u'inimaginée': 0, u'tamiseuse': 2, u'stipitée': 0, u'marquise': 0, u'calibreuse': 2, u'astreignante': 0, u'zappeuse': 2, u'parqueuse': 2, u'équinoxiale': 3, u'défiante': 0, u'paralysante': 0, u'franco-canadienne': 4, u'biinvariante': 0, u'moutardière': 6, u'endiamantée': 0, u'froussarde': 0, u'thermocollante': 0, u'terminale': 3, u'agroenvironnementale': 3, u'clinquante': 0, u'intelligente': 0, u'tragédienne': 4, u'avocassière': 6, u'haletante': 0, u'puérile': 0, u'trifaciale': 3, u'souahélie': 0, u'abatteuse': 2, u'pressante': 0, u'frugale': 3, u'outragée': 0, u'baratineuse': 2, u'affine': 0, u'permanente': 0, u'azotée': 0, u'intéroceptive': 7, u'gourde': 0, u'zodiacale': 3, u'chleuhe': 0, u'prométhéenne': 4, u'irréelle': 4, u'certificatrice': 1, u'raffineuse': 2, u'américano-israélienne': 4, u'dopante': 0, u'gestante': 0, u'iroquoise': 0, u'efficiente': 0, u'intracérébrale': 3, u'déchirante': 0, u'camuse': 0, u'séante': 0, u'intersexuée': 0, u'tourneuse': 2, u'oblongue': 4, u'baraquée': 0, u'maréchale': 3, u'hilarante': 0, u'quadrimestrielle': 4, u'non-initiée': 0, u'anticonceptionnelle': 4, u'surannée': 0, u'autodestructrice': 1, u'officielle': 4, u'procureuse': 2, u'garante': 0, u'substantielle': 4, u'doctorante': 0, u'extraforte': 0, u'superficielle': 4, u'enleveuse': 2, u'havraise': 0, u'psychoactive': 7, u'mongolienne': 4, u'dépravante': 0, u'racinienne': 4, u'censière': 6, u'vocalisatrice': 1, u'lisseuse': 2, u'rescapée': 0, u'coralline': 0, u'grugeuse': 2, u'sous-marine': 0, u'pénicillinorésistante': 0, u'perdante': 0, u'brideuse': 2, u'smaragdine': 0, u'bûcheronne': 4, u'raccompagnatrice': 1, u'montreuse': 2, u'préenregistrée': 0, u'neuromotrice': 1, u'haussmannienne': 4, u'bicamérale': 3, u'partouzarde': 0, u'pointilleuse': 9, u'décatisseuse': 2, u'adventive': 7, u'réquisitoriale': 3, u'spongieuse': 9, u'pousseuse': 2, u'confiseuse': 2, u'grelotante': 0, u'finistérienne': 4, u'pharaonne': 4, u'flétrissante': 0, u'péquenaude': 0, u'naine': 0, u'rédemptrice': 1, u'électropositive': 7, u'hallucinante': 0, u'ensevelisseuse': 2, u'syrienne': 4, u'conarde': 0, u'frontalière': 6, u'néo-zélandaise': 0, u'glutineuse': 9, u'huilière': 6, u'câline': 0, u'aragonaise': 0, u'informationnelle': 4, u'italo-américaine': 0, u'rougeoleuse': 9, u'compulsionnelle': 4, u'languedocienne': 4, u'bimbelotière': 6, u'confirmande': 0, u'étouffante': 0, u'consistante': 0, u'agitatrice': 1, u'anglo-chinoise': 0, u'surcompensatrice': 1, u'vacharde': 0, u'alsacienne': 4, u'émotionnante': 0, u'sacrale': 3, u'absolutive': 7, u'sud-africaine': 0, u'chérie': 0, u'proclamatrice': 1, u'palatale': 3, u'dépaysante': 0, u'rangeuse': 2, u'syndiquée': 0, u'pensionnée': 0, u'maltraitante': 0, u'pondeuse': 2, u'baylienne': 4, u'navrante': 0, u'précambrienne': 4, u'saoudienne': 4, u'mérovingienne': 4, u'galopine': 0, u'bougresse': 16, u'nutritionnelle': 4, u'imprévue': 0, u'ricaneuse': 2, u'inexpliquée': 0, u'immortelle': 4, u'tâtonnante': 0, u'défenderesse': 20, u'fraisicultrice': 1, u'monocristalline': 0, u'dentellière': 6, u'chicaneuse': 2, u'incoordonnée': 0, u'préservatrice': 1, u'tristounette': 4, u'accordeuse': 2, u'franco-autrichienne': 4, u'agrégée': 0, u'coursonne': 4, u'truffière': 6, u'cotutrice': 1, u'sommelière': 6, u'antigrippale': 3, u'plaisante': 0, u'malencontreuse': 9, u'corrélative': 7, u'circonciseuse': 2, u'franco-finlandaise': 0, u'ribaude': 0, u'miteuse': 9, u'insubordonnée': 0, u'attributive': 7, u'régale': 0, u'texturale': 3, u'psycho-sensori-motrice': 1, u'poissonneuse': 9, u'hypophosphoreuse': 9, u'ritale': 0, u'rétroactive': 7, u'mandante': 0, u'polytraumatisée': 0, u'post-doctorale': 3, u'ivoirienne': 4, u'interpellante': 0, u'podcasteuse': 2, u'hallucinée': 0, u'contremanifestante': 0, u'catarrhale': 3, u'bilobée': 0, u'tremblotante': 0, u'marginale': 3, u'menottée': 0, u'retentissante': 0, u'stertoreuse': 9, u'saisonnière': 6, u'minimale': 3, u'paroissienne': 4, u'antéropostérieure': 0, u'fériée': 0, u'cahotante': 0, u'papillonnante': 0, u'actuarielle': 4, u'westphalienne': 4, u'monteuse': 2, u'polyphasée': 0, u'rouspéteuse': 2, u'frictionnelle': 4, u'adjudicatrice': 1, u'électronégative': 7, u'nodale': 3, u'glanduleuse': 9, u'bénédictine': 0, u'pleurante': 0, u'essoreuse': 2, u'indécente': 0, u'patagonne': 4, u'bégayante': 0, u'défavorisée': 0, u'usagée': 0, u'libidineuse': 9, u'mystico-religieuse': 9, u'sous-lieutenante': 0, u'vadrouilleuse': 2, u'nietzschéenne': 4, u'insulino-dépendante': 0, u'tendancieuse': 9, u'additionnelle': 4, u'non-figurative': 7, u'bien-aimée': 0, u'mixeuse': 2, u'désinfectante': 0, u'stomisée': 0, u'uro-génitale': 3, u'iléale': 3, u'rafraîchie': 0, u'stérilisante': 0, u'québécoise': 0, u'morveuse': 9, u'malpolie': 0, u'shipchandleuse': 2, u'anticonstitutionnelle': 4, u'enjambée': 0, u'gouverneure': 0, u'décapitée': 0, u'nationale': 3, u'couturière': 6, u'houillère': 6, u'aiguilleteuse': 2, u'cloacale': 3, u'inhibitive': 7, u'cessante': 0, u'nicheuse': 2, u'planquée': 0, u'diminutive': 7, u'pantelante': 0, u'surabondante': 0, u'sphinctérienne': 4, u'laminale': 3, u'coorbitale': 3, u'surdéterminante': 0, u'facilitatrice': 1, u'bohémienne': 4, u'gisante': 0, u'buandière': 6, u'injurieuse': 9, u'interstitielle': 4, u'cardinale': 3, u'épilaryngienne': 4, u'campeuse': 2, u'pérennante': 0, u'encrouée': 0, u'diablesse': 16, u'ardéchoise': 0, u'esthéticienne': 4, u'hardie': 0, u'étalière': 6, u'mixolydienne': 4, u'aurignacienne': 4, u'fringante': 0, u'irrévérencieuse': 9, u'chasséenne': 4, u'ébouriffante': 0, u'seychelloise': 0, u'plombière': 6, u'extraconjugale': 3, u'herbacée': 0, u'intercantonale': 3, u'fixatrice': 1, u'débrouillarde': 0, u'électromotrice': 1, u'trappeuse': 2, u'capiteuse': 9, u'gestuelle': 4, u'désintégrative': 7, u'thessalonicienne': 4, u'boudeuse': 2, u'luxuriante': 0, u'cloisonnante': 0, u'locative': 7, u'amère': 6, u'doucheuse': 2, u'iléocæcale': 3, u'rétentrice': 1, u'enficelée': 0, u'clairette': 4, u'quadrivalente': 0, u'ciliée': 0, u'anglo-australienne': 4, u'baveuse': 9, u'atonale': 3, u'additionneuse': 2, u'pluridimensionnelle': 4, u'mobilisante': 0, u'diffuseuse': 2, u'criminelle': 4, u'fusionnelle': 4, u'hargneuse': 9, u'acousticienne': 4, u'quinte': 0, u'trochléenne': 4, u'imposante': 0, u'cumulatrice': 1, u'panafricaine': 0, u'codétenue': 0, u'sulfateuse': 2, u'négligente': 0, u'blessante': 0, u'batifolante': 0, u'téléspectatrice': 1, u'patrimoniale': 3, u'vinificatrice': 1, u'palpébrale': 3, u'érugineuse': 9, u'culturale': 3, u'culottière': 6, u'impérieuse': 9, u'baleinière': 6, u'décongestive': 7, u'tarée': 0, u'fornicatrice': 1, u'autobloquante': 0, u'hebdomadière': 6, u'plisseuse': 2, u'ultra-résistante': 0, u'subite': 0, u'possessive': 7, u'retordeuse': 2, u'réconfortante': 0, u'armatrice': 1, u'comploteuse': 2, u'audacieuse': 9, u'est-allemande': 0, u'pèquenaude': 0, u'raconteuse': 2, u'challengeuse': 2, u'lamartinienne': 4, u'chaînetière': 6, u'hervienne': 4, u'quarte': 0, u'vaurienne': 4, u'consomptive': 7, u'grossière': 6, u'spitante': 0, u'électrisante': 0, u'hydratante': 0, u'tripière': 6, u'busquée': 0, u'accorte': 0, u'grippale': 3, u'patriarcale': 3, u'inexprimée': 0, u'berçante': 0, u'chitineuse': 9, u'onduleuse': 9, u'vexante': 0, u'estivale': 3, u'moireuse': 2, u'daltonienne': 4, u'iranienne': 4, u'caucasienne': 4, u'batelée': 0, u'moulante': 0, u'décotée': 0, u'taulière': 6, u'ultraviolette': 4, u'ovoïdale': 3, u'invalidante': 0, u'récalcitrante': 0, u'zélatrice': 1, u'malicieuse': 9, u'œstrale': 3, u'extracorporelle': 4, u'rémittente': 0, u'éclopée': 0, u'tourbière': 6, u'burelée': 0, u'glycosylée': 0, u'évaginée': 0, u'hachischée': 0, u'optionnelle': 4, u'vergetière': 6, u'magnétisante': 0, u'extravertie': 0, u'boyaudière': 6, u'ensacheuse': 2, u'plombante': 0, u'bulbeuse': 9, u'susvisée': 0, u'saine': 0, u'non-alignée': 0, u'entremetteuse': 2, u'tahitienne': 4, u'vert-de-grisée': 0, u'sanctionnée': 0, u'raquetière': 6, u'dreyfusarde': 0, u'discutailleuse': 2, u'goguenarde': 0, u'binominale': 3, u'encodeuse': 2, u'silicatée': 0, u'contingente': 0, u'trine': 0, u'saint-simonienne': 4, u'bidouilleuse': 2, u'ambivalente': 0, u'excellente': 0, u'assainisseuse': 2, u'stérilisatrice': 1, u'géométrale': 3, u'bouchonnière': 6, u'benoîte': 0, u'farinacée': 0, u'qualifiante': 0, u'ombreuse': 9, u'fuligineuse': 9, u'canulante': 0, u'malaisienne': 4, u'surfeuse': 2, u'pénalisante': 0, u'bigote': 0, u'bissau-guinéenne': 4, u'grégorienne': 4, u'affouagée': 0, u'pyramidale': 3, u'vernale': 3, u'instinctive': 7, u'losangée': 0, u'admiratrice': 1, u'crapuleuse': 9, u'restringente': 0, u'pulsative': 7, u'formosane': 0, u'adiposo-génitale': 3, u'ocreuse': 9, u'ismaïlienne': 4, u'investisseuse': 2, u'interlocutrice': 1, u'florissante': 0, u'intersidérale': 3, u'orbitale': 3, u'intertropicale': 3, u'agatisée': 0, u'expiatrice': 1, u'écailleuse': 9, u'antidérapante': 0, u'capitoline': 0, u'abélianisée': 0, u'conduite': 0, u'libidinale': 3, u'irrotationnelle': 4, u'alcoolodépendante': 0, u'concluante': 0, u'dévoyée': 0, u'cafarde': 0, u'prégnante': 0, u'jubilante': 0, u'euclidienne': 4, u'surqualifiée': 0, u'immodérée': 0, u'tibiotarsienne': 4, u'fournisseuse': 2, u'cymbalière': 6, u'micronésienne': 4, u'vénéneuse': 9, u'idempotente': 0, u'biennale': 3, u'hétérosexuelle': 4, u'déodorante': 0, u'siliqueuse': 9, u'obtentrice': 1, u'rhodanienne': 4, u'bise': 0, u'foireuse': 9, u'soûle': 0, u'tortueuse': 9, u'indiscutée': 0, u'perturbante': 0, u'afocale': 3, u'brunette': 4, u'aéronavale': 0, u'résonante': 0, u'branchiale': 3, u'apprêteuse': 2, u'ignominieuse': 9, u'gesticulante': 0, u'mécatronicienne': 4, u'furtive': 7, u'coronarienne': 4, u'révérencielle': 4, u'furonculeuse': 9, u'pieuse': 9, u'peigneuse': 2, u'contre-amirale': 3, u'terrorisante': 0, u'numérale': 3, u'sous-développée': 0, u'médicolégale': 3, u'démarcative': 7, u'maléficieuse': 9, u'libérale': 3, u'tisseuse': 2, u'cotidale': 3, u'indo-malaise': 0, u'bicarrée': 0, u'biquotidienne': 4, u'louvette': 4, u'ophidienne': 4, u'absconse': 0, u'apéritive': 7, u'antilibérale': 3, u'imaginative': 7, u'urineuse': 9, u'décanale': 3, u'décortiqueuse': 2, u'anglo-irlandaise': 0, u'esquintante': 0, u'chartraine': 0, u'imprévoyante': 0, u'villageoise': 0, u'transformatrice': 1, u'pêcheuse': 2, u'harangueuse': 2, u'détachante': 0, u'sous-cutanée': 0, u'orienteuse': 2, u'arobase': 0, u'préjudicielle': 4, u'antidétonante': 0, u'mélangeante': 0, u'parquetière': 6, u'roumaine': 0, u'mendélienne': 4, u'étonnante': 0, u'herminée': 0, u'saccarinée': 0, u'décisive': 7, u'esclavonne': 4, u'flasheuse': 2, u'coextensive': 7, u'inchoative': 7, u'chocolatée': 0, u'contramirale': 3, u'excrémentielle': 4, u'largueuse': 2, u'pénicillino-résistante': 0, u'exacte': 0, u'azérie': 0, u'désavantageuse': 9, u'sinisante': 0, u'successive': 7, u'préfectorale': 3, u'embêtante': 0, u'traqueuse': 2, u'sous-dominante': 0, u'dipneumonée': 0, u'cauteleuse': 9, u'itinérante': 0, u'dormeuse': 2, u'franciscaine': 0, u'ansérine': 0, u'poupine': 0, u'pateline': 0, u'fortifiante': 0, u'diamantine': 0, u'éburonne': 4, u'zélandaise': 0, u'projectivisée': 0, u'antiministérielle': 4, u'œdipienne': 4, u'jungienne': 4, u'ressortissante': 0, u'tireuse': 2, u'révérende': 0, u'surfondue': 0, u'impériale': 3, u'centésimale': 3, u'horripilante': 0, u'non-belligérante': 0, u'receleuse': 2, u'retraitante': 0, u'laryngale': 3, u'hindoustanie': 0, u'épicrânienne': 4, u'tétanisante': 0, u'obtuse': 0, u'dépendante': 0, u'bronchiteuse': 9, u'gobeline': 0, u'cévenole': 0, u'peltée': 0, u'plurinationale': 3, u'perruquière': 6, u'partiale': 3, u'maudite': 0, u'guyanaise': 0, u'pygméenne': 4, u'lippue': 0, u'fichante': 0, u'coquebine': 0, u'fessue': 0, u'fruitarienne': 4, u'bengalie': 0, u'allurée': 0, u'inachevée': 0, u'interféconde': 0, u'patricienne': 4, u'dérégulatrice': 1, u'thermale': 3, u'interinstitutionnelle': 4, u'minorante': 0, u'venante': 0, u'antiliante': 0, u'miroitante': 0, u'discrète': 25, u'préceptorale': 3, u'présidiale': 3, u'hivernante': 0, u'bienvenue': 0, u'vicomtesse': 16, u'droite': 0, u'mycorhizienne': 4, u'débroussailleuse': 2, u'conjonctivale': 3, u'émeutière': 6, u'soûlonne': 4, u'sécheuse': 2, u'déchiqueteuse': 2, u'zootechnicienne': 4, u'sensibilisatrice': 1, u'éclaireuse': 2, u'psychochirurgienne': 4, u'orpailleuse': 2, u'rapporteuse': 2, u'thermorégulatrice': 1, u'rentière': 6, u'verdelette': 4, u'eczémateuse': 9, u'soldate': 0, u'timorée': 0, u'joggeuse': 2, u'greffeuse': 2, u'contributive': 7, u'impétueuse': 9, u'imprimeuse': 2, u'turque': 13, u'optative': 7, u'branleuse': 2, u'sotte': 4, u'proviseure': 0, u'récognitive': 7, u'choisisseuse': 2, u'perfective': 7, u'tentante': 0, u'médicamenteuse': 9, u'auditrice': 1, u'cogneuse': 2, u'longue': 4, u'bufflonne': 4, u'punitive': 7, u'féculente': 0, u'chromeuse': 2, u'dégustatrice': 1, u'carnassière': 6, u'effusive': 7, u'débarbante': 0, u'empaffée': 0, u'montueuse': 9, u'priseuse': 2, u'irrigatrice': 1, u'buccale': 3, u'élisabéthaine': 0, u'imprimante': 0, u'flirteuse': 2, u'révoquée': 0, u'cavalière': 6, u'spontanée': 0, u'métaphysicienne': 4, u'immotivée': 0, u'intersubjective': 7, u'nécessitante': 0, u'coronale': 3, u'tellurienne': 4, u'zambienne': 4, u'géopoliticienne': 4, u'orchestratrice': 1, u'levronne': 4, u'supranationale': 3, u'réparatrice': 1, u'paumée': 0, u'neigeuse': 9, u'infoutue': 0, u'tisserande': 0, u'fécondatrice': 1, u'suburbaine': 0, u'fourrière': 6, u'disséqueuse': 2, u'pondérale': 3, u'franco-colombienne': 4, u'oniromancienne': 4, u'sonnante': 0, u'néerlandaise': 0, u'racketteuse': 2, u'alcalescente': 0, u'inhérente': 0, u'privatrice': 1, u'sélectrice': 1, u'sinueuse': 9, u'immanente': 0, u'avaleuse': 2, u'conchylienne': 4, u'mitrale': 3, u'comorienne': 4, u'dénoyauteuse': 2, u'atrière': 6, u'parapétrolière': 6, u'surcapitalisante': 0, u'ensorceleuse': 2, u'démoralisatrice': 1, u'chenillée': 0, u'cacaotière': 6, u'marocaine': 0, u'réduplicative': 7, u'gredine': 0, u'assonancée': 0, u'polyhandicapée': 0, u'agropastorale': 3, u'comtesse': 16, u'éducative': 7, u'couillue': 0, u'gestationnelle': 4, u'poussiéreuse': 9, u'verdurière': 6, u'franco-provençale': 3, u'présidentielle': 4, u'intérieure': 0, u'pinçarde': 0, u'malouine': 0, u'oppositive': 7, u'désœuvrée': 0, u'affameuse': 2, u'argilacée': 0, u'franco-suédoise': 0, u'française': 0, u'rochassière': 6, u'tourière': 6, u'somalienne': 4, u'déflectrice': 1, u'barbichue': 0, u'empaqueteuse': 2, u'démêlante': 0, u'médiatrice': 1, u'calière': 6, u'gréseuse': 9, u'hominisée': 0, u'belligérante': 0, u'interpolatrice': 1, u'munificente': 0, u'encline': 0, u'salante': 0, u'péagère': 6, u'rebouteuse': 2, u'peccante': 0, u'protohistorienne': 4, u'sirupeuse': 9, u'biactive': 7, u'salpêtreuse': 9, u'trichinée': 0, u'charbonnière': 6, u'roulottière': 6, u'aconière': 6, u'thébaine': 0, u'dévalorisante': 0, u'interférente': 0, u'trisaïeule': 0, u'impulsionnelle': 4, u'spatiotemporelle': 4, u'crénelée': 0, u'nord-coréenne': 4, u'vendéenne': 4, u'barytée': 0, u'revigorante': 0, u'évanescente': 0, u'plongeante': 0, u'abréviative': 7, u'applicative': 7, u'coqueleuse': 9, u'religieuse': 9, u'navigante': 0, u'impulsive': 7, u'strombolienne': 4, u'tangentielle': 4, u'dinosaurienne': 4, u'qatarienne': 4, u'expectorante': 0, u'défourneuse': 2, u'réceptrice': 1, u'torrentielle': 4, u'entendante': 0, u'réémergente': 0, u'paraveineuse': 9, u'pakistanaise': 0, u'géniale': 3, u'non-directive': 7, u'parieuse': 2, u'sustentatrice': 1, u'pluripotente': 0, u'panseuse': 2, u'admirative': 7, u'nidificatrice': 1, u'harmonieuse': 9, u'suppurante': 0, u'donneuse': 2, u'mangeuse': 2, u'yvelinoise': 0, u'bordante': 0, u'emprunteuse': 2, u'compatissante': 0, u'cafardeuse': 9, u'transcontinentale': 3, u'aigrie': 0, u'alluviale': 3, u'stripteaseuse': 2, u'spinorielle': 4, u'postindustrielle': 4, u'fabuleuse': 9, u'solsticiale': 3, u'épateuse': 2, u'claudélienne': 4, u'grogneuse': 2, u'sous-officière': 6, u'récapitulative': 7, u'suffocante': 0, u'primale': 3, u'martienne': 4, u'frisonne': 4, u'inexercée': 0, u'sociétale': 3, u'démotivante': 0, u'indéfinie': 0, u'brûlée': 0, u'cap-verdienne': 4, u'substantive': 7, u'caoutchouteuse': 9, u'sensibilisante': 0, u'neuronale': 3, u'trifoliée': 0, u'biscornue': 0, u'dîneuse': 2, u'fromagère': 6, u'agrafeuse': 2, u'suiffeuse': 9, u'fiérote': 0, u'antibrouillée': 0, u'facétieuse': 9, u'manilleuse': 2, u'discompteuse': 2, u'sélénienne': 4, u'parolière': 6, u'délurée': 0, u'ilienne': 4, u'armoriale': 3, u'doctrinale': 3, u'franco-vénézuélienne': 4, u'inapprivoisée': 0, u'inconditionnelle': 4, u'candidate': 0, u'ânière': 6, u'radiante': 0, u'hâtive': 7, u'intimée': 0, u'antioxydante': 0, u'interpersonnelle': 4, u'fausse': 26, u'profiteuse': 2, u'soûlaude': 0, u'conchylicultrice': 1, u'siffleuse': 2, u'surprenante': 0, u'puînée': 0, u'goulue': 0, u'déchétarienne': 4, u'vascularisée': 0, u'sororale': 3, u'grasseyante': 0, u'sabéenne': 4, u'implanteuse': 2, u'hessienne': 4, u'orante': 0, u'défensive': 7, u'antiinfectieuse': 9, u'foliacée': 0, u'néolibérale': 3, u'stimulatrice': 1, u'gigantale': 3, u'jugeuse': 2, u'bordée': 0, u'flapie': 0, u'ostracée': 0, u'exécutive': 7, u'fongueuse': 9, u'gaveuse': 2, u'boucheuse': 2, u'embarrassée': 0, u'sursalée': 0, u'antidépressive': 7, u'incrustante': 0, u'zigzagante': 0, u'chaldéenne': 4, u'concitoyenne': 4, u'humiliée': 0, u'inaperçue': 0, u'crânienne': 4, u'tiquetée': 0, u'dentale': 3, u'slameuse': 2, u'verruqueuse': 9, u'réticulo-endothéliale': 3, u'allitérative': 7, u'renchérisseuse': 2, u'monstrueuse': 9, u'maligne': 24, u'glissante': 0, u'directrice': 1, u'aggravante': 0, u'lianescente': 0, u'concessive': 7, u'swahilie': 0, u'anglo-italienne': 4, u'défaillante': 0, u'tueuse': 2, u'vespérale': 3, u'réchauffante': 0, u'rénovatrice': 1, u'basilicale': 3, u'portative': 7, u'fomentatrice': 1, u'lavoisienne': 4, u'explicative': 7, u'subjective': 7, u'pivotante': 0, u'privative': 7, u'botteleuse': 2, u'tombée': 0, u'naturelle': 4, u'phonéticienne': 4, u'prévenue': 0, u'prodigieuse': 9, u'carburière': 6, u'varoise': 0, u'vitrificative': 7, u'incorporelle': 4, u'féroïenne': 4, u'plongeuse': 2, u'emberlificoteuse': 2, u'transmutatrice': 1, u'consensuelle': 4, u'exténuante': 0, u'cloueuse': 2, u'fourchue': 0, u'papilionacée': 0, u'quincaillière': 6, u'multigraduée': 0, u'absente': 0, u'élogieuse': 9, u'névrosée': 0, u'chamailleuse': 2, u'timonière': 6, u'vociférante': 0, u'interrégionale': 3, u'exempte': 0, u'simulatrice': 1, u'horticultrice': 1, u'fortraite': 0, u'familière': 6, u'métatarsienne': 4, u'néoplatonicienne': 4, u'carillonnante': 0, u'impérative': 7, u'platonicienne': 4, u'cigarière': 6, u'sidéenne': 4, u'irrésolue': 0, u'inexacte': 0, u'vomérienne': 4, u'désolante': 0, u'capésienne': 4, u'microlocalisée': 0, u'anticlinale': 3, u'pastilleuse': 2, u'charolaise': 0, u'amputée': 0, u'annihilante': 0, u'revitalisante': 0, u'triennale': 3, u'sinistrée': 0, u'brocheuse': 2, u'simienne': 4, u'rétro-éclairée': 0, u'lamifiée': 0, u'prépondérante': 0, u'journaleuse': 9, u'fromagée': 0, u'interviewée': 0, u'phosphatière': 6, u'inviteuse': 2, u'repentie': 0, u'néonatale': 0, u'questionneuse': 2, u'prudhommale': 3, u'évènementielle': 4, u'dalleuse': 2, u'purulente': 0, u'guivrée': 0, u'locale': 3, u'labiodentale': 3, u'soudeuse': 2, u'prétoriale': 3, u'coquillère': 6, u'apostrophée': 0, u'incitatrice': 1, u'vaudoue': 0, u'disposante': 0, u'trachéenne': 4, u'operculée': 0, u'géodésienne': 4, u'vacillante': 0, u'interrogative': 7, u'conceptrice': 1, u'serpentine': 0, u'précontrainte': 0, u'bretonne': 4, u'nîmoise': 0, u'proéminente': 0, u'progestative': 7, u'gantière': 6, u'amincissante': 0, u'stagnante': 0, u'béarnaise': 0, u'serpigineuse': 9, u'éburnéenne': 4, u'saccharine': 0, u'libéralisante': 0, u'graffiteuse': 2, u'ovée': 0, u'malheureuse': 9, u'lunetière': 6, u'gymnasiale': 3, u'néo-libérale': 3, u'dilapidatrice': 1, u'antécambrienne': 4, u'succombante': 0, u'vultueuse': 9, u'bouleversante': 0, u'tatare': 0, u'promise': 0, u'productrice': 1, u'commise': 0, u'vindicative': 7, u'infecte': 0, u'grimpante': 0, u'ovarienne': 4, u'ramoneuse': 2, u'anatolienne': 4, u'affuteuse': 2, u'baconienne': 4, u'cocardière': 6, u'japonisante': 0, u'bénite': 0, u'saisonnale': 3, u'rate': 0, u'affreuse': 9, u'ivrognesse': 16, u'analysante': 0, u'diluante': 0, u'épargnante': 0, u'immunostimulante': 0, u'superordonnée': 0, u'franco-indonésienne': 4, u'glandée': 0, u'maintenicienne': 4, u'indélicate': 0, u'grandette': 4, u'comtoise': 0, u'accréditrice': 1, u'marmotteuse': 2, u'teenageuse': 2, u'contrariante': 0, u'transocéanienne': 4, u'bobinière': 6, u'manchote': 0, u'courageuse': 9, u'vestigiale': 3, u'technicocommerciale': 3, u'asinienne': 4, u'bouchère': 6, u'millésimale': 3, u'monopolisatrice': 1, u'échiquetée': 0, u'lampante': 0, u'florencée': 0, u'boxeuse': 2, u'enfançonne': 4, u'fouettarde': 0, u'arséniée': 0, u'aquitaine': 0, u'préimprimée': 0, u'parpaillote': 0, u'suffragante': 0, u'cosméticienne': 4, u'modelante': 0, u'désuète': 25, u'laideronne': 4, u'obédientielle': 4, u'détrousseuse': 2, u'vaticinatrice': 1, u'ramollissante': 0, u'ébouquetée': 0, u'magistrate': 0, u'birationnelle': 4, u'bateleuse': 2, u'utérine': 0, u'abbatiale': 3, u'ailière': 6, u'préemballée': 0, u'menteuse': 2, u'anticoagulante': 0, u'inobservée': 0, u'effleurie': 0, u'tridentée': 0, u'coulée': 0, u'cursive': 7, u'bufflesse': 16, u'coopératrice': 1, u'croûteuse': 9, u'soufreuse': 2, u'argumentative': 7, u'démultiplicatrice': 1, u'néogrammairienne': 4, u'pisseuse': 9, u'titubante': 0, u'extralégale': 3, u'défibreuse': 2, u'prudentielle': 4, u'flemmarde': 0, u'grassouillette': 4, u'noire': 0, u'tourmenteuse': 2, u'contre-manifestante': 0, u'invitée': 0, u'sous-durale': 3, u'briseuse': 2, u'grisante': 0, u'tarsienne': 4, u'argilo-siliceuse': 9, u'îlienne': 4, u'détonante': 0, u'nymphale': 3, u'excrémenteuse': 9, u'conative': 7, u'interdépendante': 0, u'autoréalisatrice': 1, u'confuse': 0, u'snowboardeuse': 2, u'évacuatrice': 1, u'démonstratrice': 1, u'formière': 6, u'senestrée': 0, u'préalpine': 0, u'rachidienne': 4, u'militante': 0, u'dérouleuse': 2, u'spéculative': 7, u'scrotale': 3, u'missilière': 6, u'biscaïenne': 4, u'skateuse': 2, u'déblayeuse': 2, u'ponceuse': 9, u'hagarde': 0, u'assimilative': 7, u'saponacée': 0, u'circonférentielle': 4, u'polyédrale': 3, u'expansive': 7, u'corporelle': 4, u'sourcière': 6, u'trifoliolée': 0, u'encourageante': 0, u'sous-maîtresse': 16, u'hormonale': 3, u'télémétreuse': 2, u'distinctive': 7, u'rassise': 0, u'autoadaptative': 7, u'goitreuse': 9, u'verbale': 3, u'azoteuse': 9, u'déverbale': 3, u'accidentelle': 4, u'impardonnée': 0, u'archipresbytérale': 3, u'mareyeuse': 2, u'adorée': 0, u'haussière': 6, u'clapotante': 0, u'filtrante': 0, u'designeuse': 2, u'piaffante': 0, u'tachante': 0, u'vagabonde': 0, u'estudiantine': 0, u'caprine': 0, u'progénitrice': 1, u'morfondue': 0, u'introvertie': 0, u'outrageante': 0, u'multiconductrice': 1, u'astrale': 3, u'pavimenteuse': 9, u'méniscale': 3, u'prénuptiale': 3, u'audio-orale': 3, u'écrivassière': 6, u'mussive': 7, u'saturnine': 0, u'flageolante': 0, u'champée': 0, u'loueuse': 2, u'affolante': 0, u'conjoncturelle': 4, u'incontinente': 0, u'placeuse': 2, u'inconnue': 0, u'compactifiée': 0, u'huîtrière': 6, u'guêpière': 6, u'constringente': 0, u'imitative': 7, u'abbevillienne': 4, u'hurluberlue': 0, u'aoûtienne': 4, u'suppresseuse': 2, u'tarbaise': 0, u'bruyante': 0, u'soldeuse': 2, u'volleyeuse': 2, u'anévrysmale': 3, u'foutrale': 0, u'rubéoleuse': 9, u'salvatrice': 1, u'lancinante': 0, u'noethérienne': 4, u'craquante': 0, u'congruente': 0, u'pétrinienne': 4, u'autonettoyante': 0, u'gâteuse': 9, u'intimidatrice': 1, u'sauve': 7, u'structurale': 3, u'sanctifiée': 0, u'inefficiente': 0, u'chevalière': 6, u'licière': 6, u'radiale': 3, u'subintrante': 0, u'épurative': 7, u'antimoniée': 0, u'tâcheronne': 4, u'timbrée': 0, u'coucheuse': 2, u'survireuse': 2, u'accentuelle': 4, u'pitchoune': 0, u'intercostale': 3, u'jersiaise': 0, u'diffamante': 0, u'pouparde': 0, u'inaccoutumée': 0, u'rejetonne': 4, u'cespiteuse': 9, u'bisimpliciale': 3, u'souillarde': 0, u'tarotée': 0, u'envoûtée': 0, u'distributrice': 1, u'buissonneuse': 9, u'nord-vietnamienne': 4, u'trieuse': 2, u'liseuse': 2, u'clunisienne': 4, u'brisante': 0, u'fréquente': 0, u'sondagière': 6, u'fossoyeuse': 2, u'impayée': 0, u'congestive': 7, u'carabinée': 0, u'usinière': 6, u'naïve': 7, u'crêpelée': 0, u'éléphante': 0, u'mutuelle': 4, u'sexuée': 0, u'plaqueuse': 2, u'commutative': 7, u'autoadhésive': 7, u'allaitante': 0, u'méditerranéenne': 4, u'ludo-éducative': 7, u'heptagonale': 3, u'aigrelette': 4, u'classificatrice': 1, u'franco-vietnamienne': 4, "prud'homale": 3, u'sodée': 0, u'résultative': 7, u'invertébrée': 0, u'wagonnière': 6, u'cyberdépendante': 0, u'animale': 3, u'mayennaise': 0, u'pubescente': 0, u'marguillière': 6, u'éprouvante': 0, u'institutionnelle': 4, u'embouteilleuse': 2, u'subjacente': 0, u'glabrescente': 0, u'interprofessionnelle': 4, u'psycho-éducative': 7, u'urétérale': 3, u'ânesse': 16, u'cryoconductrice': 1, u'plaignarde': 0, u'liquidienne': 4, u'ultra-violette': 4, u'lourdaude': 0, u'pourchassée': 0, u'familiale': 3, u'tumorale': 3, u'péronéotibiale': 3, u'éducationnelle': 4, u'matelassière': 6, u'herscheuse': 2, u'maghrébine': 0, u'employée': 0, u'précédente': 0, u'dispatcheuse': 2, u'élingueuse': 2, u'loculeuse': 9, u'métrosexuelle': 4, u'potentielle': 4, u'teilleuse': 2, u'localisante': 0, u'bretillienne': 4, u'personnelle': 4, u'offreuse': 2, u'meneuse': 2, u'sacrificatrice': 1, u'périclitante': 0, u'acharnée': 0, u'grincheuse': 9, u'défranchie': 0, u'sous-jacente': 0, u'autostoppeuse': 2, u'stéroïdienne': 4, u'sommeilleuse': 9, u'intersyndicale': 3, u'racoleuse': 2, u'doyenne': 4, u'espionne': 4, u'internationale': 3, u'anglo-brésilienne': 4, u'festoyeuse': 2, u'redistributrice': 1, u'gourmande': 0, u'heptathlonienne': 4, u'crue': 0, u'biinfinie': 0, u'bourbonienne': 4, u'coorganisatrice': 1, u'fanfaronne': 4, u'poissonnière': 6, u'bergère': 6, u'chevrière': 6, u'démouleuse': 2, u'rhizomateuse': 9, u'radieuse': 9, u'oxycarbonée': 0, u'aéropostale': 3, u'délatrice': 1, u'permutante': 0, u'intestinale': 3, u'locutrice': 1, u'napoléonienne': 4, u'magouilleuse': 2, u'séquanaise': 0, u'monovalente': 0, u'surfine': 0, u'intercontinentale': 3, u'barreuse': 2, u'fuyante': 0, u'mûre': 0, u'autoportante': 0, u'méconiale': 3, u'diamétrale': 3, u'pionne': 4, u'involutée': 0, u'substitutive': 7, u'calvadosienne': 4, u'antenaise': 0, u'franco-marocaine': 0, u'agricultrice': 1, u'formative': 7, u'rétractrice': 1, u'gazonneuse': 9, u'sadico-anale': 3, u'coureuse': 2, u'avicultrice': 1, u'boutonneuse': 9, u'dodécaphasée': 0, u'proliférante': 0, u'luthérienne': 4, u'interruptive': 7, u'évangélisatrice': 1, u'moustachue': 0, u'nourricière': 6, u'manquante': 0, u'pantouflarde': 0, u'monoflorale': 3, u'fondamentale': 3, u'chirale': 3, u'jet-setteuse': 2, u'théologale': 3, u'stuporeuse': 9, u'invendue': 0, u'dégraisseuse': 2, u'aménageuse': 2, u'neuve': 7, u'franco-ukrainienne': 4, u'exténuée': 0, u'globalisante': 0, u'cutanée': 0, u'goudronneuse': 9, u'graillonneuse': 9, u'préfète': 25, u'charnue': 0, u'maousse': 4, u'incarnate': 0, u'pompière': 6, u'abêtissante': 0, u'adjudicative': 7, u'chaussante': 0, u'globe-trotteuse': 2, u'choupinette': 4, u'fédérative': 7, u'écouteuse': 2, u'intra-artérielle': 4, u'baronne': 4, u'neurodégénérative': 7, u'mignarde': 0, u'américano-mexicaine': 0, u'répressive': 7, u'agro-environnementale': 3, u'contremaîtresse': 16, u'indélibérée': 0, u'binaurale': 3, u'grenadine': 0, u'froissante': 0, u'incorrecte': 0, u'graisseuse': 9, u'pileuse': 9, u'frétillante': 0, u'fouineuse': 2, u'microsomiale': 3, u'translationnelle': 4, u'factorielle': 4, u'métricienne': 4, u'basketteuse': 2, u'solfatarienne': 4, u'fruitée': 0, u'pyrrhonienne': 4, u'messine': 0, u'propositionnelle': 4, u'sniffeuse': 2, u'dégravelante': 0, u'versificatrice': 1, u'tabacultrice': 1, u'sikhe': 0, u'scapulohumérale': 3, u'inhabitée': 0, u'déperlante': 0, u'rajeunisseuse': 2, u'italienne': 4, u'terraquée': 0, u'endothéliale': 3, u'ichoreuse': 9, u'tribale': 3, u'passagère': 6, u'pesteuse': 9, u'saucière': 6, u'usufruitière': 6, u'porchère': 6, u'papelarde': 0, u'instinctuelle': 4, u'aveuglante': 0, u'matinale': 3, u'irienne': 4, u'hindoue': 0, u'famée': 0, u'martiniquaise': 0, u'linguale': 3, u'insensée': 0, u'coxo-fémorale': 3, u'vantarde': 0, u'altitudinale': 3, u'autoentrepreneuse': 2, u'bisonne': 4, u'électro-acousticienne': 4, u'khâgneuse': 9, u'muette': 4, u'affamante': 0, u'inopportune': 0, u'écrivaine': 0, u'abrogative': 7, u'tropho-microbienne': 4, u'vermeille': 4, u'photoréceptrice': 1, u'accaparante': 0, u'cordiale': 3, u'diffuse': 0, u'fransquillonne': 4, u'arêtière': 6, u'cratérisée': 0, u'semeuse': 2, u'catégorisante': 0, u'fumerollienne': 4, u'nuageuse': 9, u'mortelle': 4, u'mélaminée': 0, u'broussailleuse': 9, u'morutière': 6, u'volailleuse': 2, u'collante': 0, u'apiéceuse': 2, u'jaunissante': 0, u'franco-allemande': 0, u'mogole': 0, u'cadreuse': 2, u'nucale': 3, u'indiscrète': 25, u'grimpeuse': 2, u'décadente': 0, u'déciduale': 3, u'crevarde': 0, u'obligeante': 0, u'mauricienne': 4, u'encaisseuse': 2, u'importune': 0, u'toxi-infectieuse': 9, u'énonciative': 7, u'échauffante': 0, u'carpienne': 4, u'parenchymateuse': 9, u'structurelle': 4, u'achirale': 3, u'infraliminale': 3, u'plastiqueuse': 2, u'incomprise': 0, u'nabote': 0, u'émigrée': 0, u'lobuleuse': 9, u'individualisante': 0, u'vétilleuse': 9, u'serveuse': 2, u'dorsoventrale': 3, u'cadette': 4, u'bagnarde': 0, u'communisante': 0, u'enjôleuse': 2, u'révulsive': 7, u'prohibitive': 7, u'franco-espagnole': 0, u'saupoudreuse': 2, u'récollette': 4, u'laotienne': 4, u'kényane': 0, u'trilobée': 0, u'piroguière': 6, u'inexistante': 0, u'haleuse': 2, u'déclamatrice': 1, u'voisine': 0, u'creuse': 9, u'cohabitante': 0, u'compograveuse': 2, u'truculente': 0, u'maniaco-dépressive': 7, u'grassmannienne': 4, u'antifrançaise': 0, u'ulcéreuse': 9, u'exorbitante': 0, u'ombrageuse': 9, u'agitante': 0, u'amicale': 3, u'rosicrucienne': 4, u'iridiée': 0, u'pétante': 0, u'floue': 0, u'cancéreuse': 9, u'tadjike': 0, u'denchée': 0, u'viciatrice': 1, u'mémorielle': 4, u'descriptrice': 1, u'jordanienne': 4, u'dissuasive': 7, u'succulente': 0, u'soulote': 0, u'cataclysmale': 3, u'divertissante': 0, u'réhydratante': 0, u'franco-américaine': 0, u'farineuse': 9, u'ergative': 7, u'calleuse': 9, u'pétroleuse': 2, u'équilibreuse': 2, u'inconventionnelle': 4, u'végétale': 3, u'genrée': 0, u'fruiticultrice': 1, u'azimutale': 3, u'homosexuelle': 4, u'chrismale': 3, u'sprinteuse': 2, u'crocodilienne': 4, u'saturnienne': 4, u'automnale': 3, u'intacte': 0, u'préhistorienne': 4, u'habitante': 0, u'urcéolée': 0, u'russo-chinoise': 0, u'primordiale': 3, u'inconquise': 0, u'conjointe': 0, u'lénifiante': 0, u'acquéreuse': 2, u'énervée': 0, u'malendurante': 0, u'jennérienne': 4, u'orthonormée': 0, u'logisticienne': 4, u'dodue': 0, u'grognante': 0, u'escamoteuse': 2, u'miraillée': 0, u'cortico-spinale': 3, u'parlante': 0, u'socio-affective': 7, u'vertébrale': 3, u'doubleuse': 2, u'aboyeuse': 2, u'adipeuse': 9, u'jéjunale': 3, u'chaude': 0, u'dialectale': 3, u'coiffante': 0, u'comitiale': 3, u'cornéenne': 4, u'mandarinale': 3, u'dindonnière': 6, u'endettée': 0, u'transactionnelle': 4, u'motrice': 1, u'recycleuse': 2, u'centrafricaine': 0, u'impertinente': 0, u'amplificatrice': 1, u'alcoolo-résistante': 0, u'faïencière': 6, u'conviviale': 3, u'compréhensive': 7, u'châtreuse': 2, u'abougrie': 0, u'immémoriale': 3, u'artinienne': 4, u'grumeleuse': 9, u'hyaline': 0, u'scandalisée': 0, u'antisudorale': 3, u'austro-allemande': 0, u'byzantine': 0, u'cinglante': 0, u'melonnée': 0, u'hermitienne': 4, u'côtelée': 0, u'cordière': 6, u'saharienne': 4, u'intensive': 7, u'taiseuse': 9, u'transuranienne': 4, u'sarroise': 0, u'transculturelle': 4, u'thyroïdienne': 4, u'criméenne': 4, u'allumeuse': 2, u'antithyroïdienne': 4, u'vomitive': 7, u'spoliatrice': 1, u'historiale': 3, u'gastro-intestinale': 3, u'franchouillarde': 0, u'porracée': 0, u'pistoleuse': 2, u'bétonneuse': 2, u'filiale': 3, u'fomenteuse': 2, u'italo-albanaise': 0, u'sectionnelle': 4, u'vicésimale': 3, u'adhérente': 0, u'lamellée': 0, u'trottante': 0, u'pluriculturelle': 4, u'contrebandière': 6, u'collégienne': 4, u'coparticipante': 0, u'interpellatrice': 1, u'pelliculée': 0, u'inoubliée': 0, u'sénatoriale': 3, u'irradiée': 0, u'médaillée': 0, u'impudente': 0, u'milanaise': 0, u'narbonnaise': 0, u'disputeuse': 2, u'alezane': 0, u'bouquineuse': 2, u'inapaisée': 0, u'oppressive': 7, u'intumescente': 0, u'varappeuse': 2, u'toquarde': 0, u'pareuse': 2, u'verseuse': 2, u'différentiée': 0, u'exaltée': 0, u'euphorisante': 0, u'vivrière': 6, u'venteuse': 9, u'siamoise': 0, u'javanaise': 0, u'sud-américaine': 0, u'résultante': 0, u'emballeuse': 2, u'abondante': 0, u'prête': 0, u'contragestive': 7, u'préfacière': 6, u'empailleuse': 2, u'infradienne': 4, u'régnante': 0, u'préceptrice': 1, u'récente': 0, u'cérémoniale': 0, u'soularde': 0, u'cathédrale': 3, u'microgrenue': 0, u'féale': 3, u'sélectionneuse': 2, u'imparipennée': 0, u'mosane': 0, u'appelante': 0, u'parnassienne': 4, u'amphibienne': 4, u'new-yorkaise': 0, u'conquise': 0, u'heaumière': 6, u'romaine': 0, u'cornélienne': 4, u'cultuelle': 4, u'badigeonneuse': 2, u'déponente': 0, u'savoisienne': 4, u'franco-cubaine': 0, "baha'ie": 0, u'cisalpine': 0, u'voltigeante': 0, u'amodiatrice': 1, u'dérangeuse': 2, u'miséricordieuse': 9, u'envoyeuse': 2, u'prochinoise': 0, u'dévergondée': 0, u'légère': 6, u'âgée': 0, u'illative': 7, u'anglo-danoise': 0, u'consultative': 7, u'inorganisée': 0, u'stoppeuse': 2, u'viseuse': 9, u'studieuse': 9, u'palatine': 0, u'néphridienne': 4, u'meurtrissante': 0, u'ankylosante': 0, u'sanscrite': 0, u'écœurante': 0, u'celluleuse': 9, u'distante': 0, u'protectrice': 1, u'clinicienne': 4, u'componentielle': 4, u'concomitante': 0, u'impécunieuse': 9, u'putative': 7, u'brouillardeuse': 9, u'aboyante': 0, u'jugale': 3, u'bote': 0, u'artésienne': 4, u'souffreteuse': 9, u'coadaptatrice': 1, u'lassante': 0, u'surie': 0, u'opulente': 0, u'évadée': 0, u'fureteuse': 2, u'pubienne': 4, u'quinée': 0, u'protégée': 0, u'hypernerveuse': 9, u'hâbleuse': 2, u'édentée': 0, u'poméranienne': 4, u'allusive': 7, u'plantureuse': 9, u'impatientante': 0, u'accablante': 0, u'exanthémateuse': 9, u'affligeante': 0, u'indoue': 0, u'kidnappée': 0, u'décentralisatrice': 1, u'judicieuse': 9, u'anglo-espagnole': 0, u'intertidale': 3, u'frisotante': 0, u'géopotentielle': 4, u'égarée': 0, u'exceptionnelle': 4, u'shakespearienne': 4, u'voyageuse': 2, u'entrainante': 0, u'vaincue': 0, u'khmère': 6, u'meuleuse': 2, u'arsine': 0, u'maraudeuse': 2, u'rosissante': 0, u'néo-romane': 0, u'intervieweuse': 2, u'herculéenne': 4, u'persistante': 0, u'significative': 7, u'postdoctorale': 3, u'élyséenne': 4, u'enfileuse': 2, u'fluorescente': 0, u'gélive': 7, u'confirmative': 7, u'troublante': 0, u'garçonnière': 6, u'remorqueuse': 2, u'extra-professionnelle': 4, u'balzane': 0, u'insistante': 0, u'géante': 0, u'virevoltante': 0, u'créancière': 6, u'antinaturelle': 4, u'bourlingueuse': 2, u'maraude': 0, u'unilobée': 0, u'rotacée': 0, u'amazonienne': 4, u'subalpine': 0, u'inconditionnée': 0, u'censoriale': 3, u'audioguidée': 0, u'basale': 3, u'perceptuelle': 4, u'irradiante': 0, u'régressée': 0, u'bredouillante': 0, u'hyperboréenne': 4, u'parfumeuse': 2, u'dépouilleuse': 2, u'caoutchoutière': 6, u'frumentacée': 0, u'rutilante': 0, u'avilissante': 0, u'franco-indienne': 4, u'perverse': 0, u'hyperfocale': 3, u'fasciée': 0, u'dorso-ventrale': 3, u'résonnante': 0, u'essuyeuse': 2, u'goémonière': 6, u'frontale': 3, u'nippo-coréenne': 4, u'soupeuse': 2, u'agitée': 0, u'faignante': 0, u'sélective': 7, u'écrivaillée': 0, u'ragoûtante': 0, u'drapière': 6, u'chancelante': 0, u'pérégrine': 0, u'queutarde': 0, u'festive': 7, u'brûleuse': 2, u'escaladeuse': 2, u'gracieuse': 9, u'finale': 0, u'hercynienne': 4, u'bidirectionnelle': 4, u'tambourineuse': 2, u'minéralisatrice': 1, u'auto-entrepreneuse': 2, u'modificatrice': 1, u'synanthérée': 0, u'vigilante': 0, u'mélangeuse': 2, u'conteuse': 2, u'antibactérienne': 4, u'causale': 3, u'franco-pakistanaise': 0, u'esquichée': 0, u'placière': 6, u'pourprine': 0, u'contextuelle': 4, u'chatoyante': 0, u'cochonne': 4, u'antéro-postérieure': 0, u'lactée': 0, u'éreintante': 0, u'tellureuse': 9, u'anxieuse': 9, u'meusienne': 4, u'incessante': 0, u'injustifiée': 0, u'antihermitienne': 4, u'postière': 6, u'potelée': 0, u'roussillonnaise': 0, u'testatrice': 1, u'réto-romane': 0, u'inhumaine': 0, u'assoupissante': 0, u'grainière': 6, u'maitresse': 16, u'rudérale': 3, u'pinailleuse': 2, u'prétendante': 0, u'ruisselante': 0, u'accueillante': 0, u'autopropulsée': 0, u'illustrative': 7, u'hyperglycémiante': 0, u'boguée': 0, u'abréviée': 0, u'conductive': 7, u'bleue': 0, u'acuponctrice': 1, u'acescente': 0, u'boiteuse': 9, u'lutécienne': 4, u'fibrilleuse': 9, u'non-inscrite': 0, u'réfractrice': 1, u'inoffensive': 7, u'végétative': 7, u'casquettière': 6, u'dégourdissante': 0, u'variante': 0, u'impressionnante': 0, u'émouvante': 0, u'ambulancière': 6, u'fléchissante': 0, u'indigente': 0, u'monofocale': 3, u'séparatrice': 1, u'quartzeuse': 9, u'curative': 7, u'initiée': 0, u'chemisière': 6, u'malfaisante': 0, u'détectrice': 1, u'embue': 0, u'potencée': 0, u'saugrenue': 0, u'ougrienne': 4, u'ouateuse': 9, u'endocrinienne': 4, u'inébranlée': 0, u'interprétatrice': 1, u'grenadière': 6, u'godilleuse': 2, u'duodécimale': 3, u'draconienne': 4, u'innocentée': 0, u'œstroprogestative': 7, u'réversale': 3, u'asturienne': 4, u'inquisitoriale': 3, u'musculeuse': 9, u'versaillaise': 0, u'humorale': 3, u'crucifiante': 0, u'anglo-norvégienne': 4, u'tricoteuse': 2, u'muletière': 6, u'gagnante': 0, u'ptérygoïdienne': 4, u'hégélienne': 4, u'australienne': 4, u'archiépiscopale': 3, u'égrillarde': 0, u'bisannuelle': 4, u'surproductrice': 1, u'rouanne': 4, u'flamboyante': 0, u'cantonaise': 0, u'opératrice': 1, u'staffeuse': 2, u'alevinière': 6, u'locrienne': 4, u'fluctuante': 0, u'sensorimotrice': 1, u'palpitante': 0, u'hypoglycémiante': 0, u'issante': 0, u'filonienne': 4, u'râpeuse': 9, u'vice-championne': 4, u'agrandisseuse': 2, u'corpulente': 0, u'cybermilitante': 0, u'sasseuse': 2, u'amygdalienne': 4, u'empaillée': 0, u'insulinodépendante': 0, u'ilotière': 6, u'beugleuse': 2, u'horodatrice': 1, u'balinaise': 0, u'chevelée': 0, u'défunte': 0, u'disetteuse': 9, u'frissonnante': 0, u'mirobolante': 0, u'chenue': 0, u'outrageuse': 9, u'attractrice': 1, u'dévastatrice': 1, u'pommetée': 0, u'suppliciée': 0, u'souscriptrice': 1, u'augustinienne': 4, u'invariante': 0, u'prévisionnelle': 4, u'antigravitationnelle': 4, u'avionneuse': 2, u'affronteuse': 2, u'arménienne': 4, u'intimidée': 0, u'circonstancielle': 4, u'canado-américaine': 0, u'baissière': 6, u'lituanienne': 4, u'taxative': 7, u'reconstituante': 0, u'vassale': 3, u'larmoyante': 0, u'homofocale': 3, u'irakienne': 4, u'mormone': 0, u'covariante': 0, u'limougeaude': 0, u'co-orbitale': 3, u'abortive': 7, u'classeuse': 2, u'déliquescente': 0, u'algocultrice': 1, u'dédaigneuse': 9, u'interjective': 7, u'longitudinale': 3, u'éditrice': 1, u'nivo-pluviale': 3, u'clarificatrice': 1, u'provisionnelle': 4, u'insuffisante': 0, u'rectale': 3, u'éhontée': 0, u'propagatrice': 1, u'étampeuse': 2, u'pêchue': 0, u'cordonnière': 6, u'tempétueuse': 9, u'émergente': 0, u'égorgeuse': 2, u'plurilatérale': 3, u'continente': 0, u'monovariante': 0, u'barguigneuse': 2, u'intégrodifférentielle': 4, u'carillonneuse': 2, u'mitrée': 0, u'binoclarde': 0, u'conservative': 7, u'ratée': 0, u'ordonnatrice': 1, u'prospectrice': 1, u'turgescente': 0, u'antivénérienne': 4, u'arracheuse': 2, u'américano-turque': 13, u'phonatrice': 1, u'débordante': 0, u'élusive': 7, u'expulsée': 0, u'suggestive': 7, u'contrintuitive': 7, u'pataphysicienne': 4, u'mononucléée': 0, u'polarde': 0, u'recouvreuse': 2, u'mutante': 0, u'exemplative': 7, u'cubaine': 0, u'alcaline': 0, u'plurinominale': 3, u'teinturière': 6, u'résineuse': 9, u'périnatale': 0, u'quadrisaïeule': 0, u'vésicale': 3, u'faucardeuse': 2, u'cytogénéticienne': 4, u'abrasive': 7, u'fermentante': 0, u'sensitive': 7, u'évidente': 0, u'inaccomplie': 0, u'uxorilocale': 3, u'fédératrice': 1, u'flexionnelle': 4, u'roncière': 6, u'pétocharde': 0, u'contreperçante': 0, u'majeure': 0, u'adhésive': 7, u'thermicienne': 4, u'cagote': 0, u'paquetée': 0, u'redondée': 0, u'prélatine': 0, u'arborescente': 0, u'notificative': 7, u'relapse': 0, u'séronégative': 7, u'liégeoise': 0, u'oisive': 7, u'frutescente': 0, u'cruentée': 0, u'participante': 0, u'prédicative': 7, u'létale': 3, u'paniquante': 0, u'afro-américaine': 0, u'fovéale': 3, u'gluante': 0, u'bavaroise': 0, u'surcomposée': 0, u'opprimante': 0, u'lettreuse': 2, u'équipée': 0, u'surette': 4, u'caleuse': 2, u'gazonnante': 0, u'barrémienne': 4, u'ambianceuse': 2, u'brelandière': 6, u'souriante': 0, u'dispersante': 0, u'transparente': 0, u'abyssine': 0, u'prépotente': 0, u'cruciale': 3, u'torturante': 0, u'opalescente': 0, u'ultralibérale': 3, u'sauteuse': 2, u'rétrovirale': 3, u'charnelle': 4, u'cousine': 0, u'franco-macédonienne': 4, u'bourbeuse': 9, u'causeuse': 2, u'migraineuse': 9, u'députée': 0, u'fraichissante': 0, u'tour-opératrice': 1, u'taxatrice': 1, u'jeûneuse': 2, u'autoritative': 7, u'endêvée': 0, u'iridienne': 4, u'deltoïdienne': 4, u'focalisante': 0, u'intentionnée': 0, u'chérifienne': 4, u'sensationnelle': 4, u'disqualifiée': 0, u'bassinante': 0, u'claveleuse': 9, u'équatoguinéenne': 4, u'axonaise': 0, u'glénoïdale': 3, u'impromptue': 0, u'baudelairienne': 4, u'bilatérale': 3, u'bambocharde': 0, u'médicosociale': 3, u'désinhibitrice': 1, u'empoisonneuse': 2, u'contaminante': 0, u'tallée': 0, u'ratiocinante': 0, u'chélatrice': 1, u'gratteuse': 2, u'boutante': 0, u'déterminante': 0, u'préconsciente': 0, u'dangereuse': 9, u'fraîchissante': 0, u'maigrichonne': 4, u'violette': 4, u'alvéolée': 0, u'incitative': 7, u'requérante': 0, u'mocharde': 0, u'chieuse': 2, u'marmoréenne': 4, u'drupacée': 0, u'psycho-éducatrice': 1, u'représentante': 0, u'frappante': 0, u'varioleuse': 9, u'squirrheuse': 9, u'antigivrante': 0, u'martiale': 3, u'autoreproductrice': 1, u'encroûtante': 0, u'plaignante': 0, u'mystifiante': 0, u'murale': 3, u'odoriférante': 0, u'fidélisante': 0, u'diplômée': 0, u'cubitale': 3, u'tuilière': 6, u'indomptée': 0, u'crevante': 0, u'spumeuse': 9, u'médiale': 3, u'digressive': 7, u'musicienne': 4, u'gélifiante': 0, u'charmeuse': 2, u'préprogrammée': 0, u'finissante': 0, u'envoûtante': 0, u'époussetée': 0, u'hydrurée': 0, u'illusionnée': 0, u'orale': 3, u'équilatérale': 3, u'enquiquineuse': 2, u'coprésente': 0, u'piétonne': 4, u'joaillère': 6, u'pro-active': 7, u'wallonne': 4, u'incirconcise': 0, u'antifiscale': 3, u'euro-méditerranéenne': 4, u'difficultueuse': 9, u'enfantine': 0, u'routeuse': 2, u'éperdue': 0, u'défilante': 0, u'déloyale': 3, u'savante': 0, u'outsideuse': 2, u'pisteuse': 2, u'épuisante': 0, u'saunière': 6, u'continuatrice': 1, u'butineuse': 2, u'idéomotrice': 1, u'démersale': 3, u'kleinienne': 4, u'multiculturelle': 4, u'écolière': 6, u'papoteuse': 2, u'transsexuelle': 4, u'tropézienne': 4, u'modérée': 0, u'photorésistante': 0, u'presseuse': 2, u'ignée': 0, u'lapine': 0, u'parraineuse': 2, u'sanieuse': 9, u'décapeuse': 2, u'anti-ministérielle': 4, u'suédoise': 0, u'aérostière': 6, u'jacassière': 6, u'béante': 0, u'pyroxylée': 0, u'sentimentale': 3, u'crieuse': 2, u'inanimée': 0, u'radiocommandée': 0, u'autoroutière': 6, u'touranienne': 4, u'décroissante': 0, u'intellectuelle': 4, u'guetteuse': 2, u'ano-rectale': 3, u'monofonctionnelle': 4, u'addictive': 7, u'électroluminescente': 0, u'vénérienne': 4, u'plastifiante': 0, u'patiente': 0, u'précise': 0, u'roucoulante': 0, u'trainante': 0, u'malaimée': 0, u'passeuse': 2, u'buvetière': 6, u'pétrarquisante': 0, u'capée': 0, u'fracasseuse': 2, u'cohérente': 0, u'antireligieuse': 9, u'constitutive': 7, u'dégoutante': 0, u'ionienne': 4, u'barbouilleuse': 2, u'puncheuse': 2, u'post-soixante-huitarde': 0, u'lactescente': 0, u'afghane': 0, u'interactive': 7, u'fabulatrice': 1, u'déligneuse': 2, u'lanugineuse': 9, u'étameuse': 2, u'remueuse': 2, u'préférentielle': 4, u'exigüe': 12, u'vanuatuane': 0, u'quotidienne': 4, u'jurée': 0, u'neurocognitive': 7, u'crémière': 6, u'suprasegmentale': 3, u'cachottière': 6, u'charretière': 6, u'charmante': 0, u'transsaharienne': 4, u'trimestrielle': 4, u'présomptive': 7, u'renaissante': 0, u'éboueuse': 2, u'régalienne': 4, u'boursicoteuse': 2, u'buccogénitale': 3, u'conjecturale': 3, u'engainante': 0, u'danienne': 4, u'greffière': 6, u'œsophagienne': 4, u'impétrante': 0, u'législatrice': 1, u'hypertextuelle': 4, u'bâtarde': 0, u'indéchiffrée': 0, u'ailée': 0, u'nébuleuse': 9, u'supratemporale': 0, u'marketeuse': 2, u'ceinturière': 6, u'décanteuse': 2, u'libertarienne': 4, u'strasbourgeoise': 0, u'reposante': 0, u'dribleuse': 2, u'présidente': 0, u'musarde': 0, u'tripatouilleuse': 2, u'assourdissante': 0, u'cocontractante': 0, u'moucheuse': 2, u'croisetée': 0, u'poudreuse': 9, u'brulante': 0, u'malteuse': 2, u'directe': 0, u'rigoureuse': 9, u'mailleuse': 2, u'normale': 3, u'électroménagère': 6, u'ressemblante': 0, u'policière': 6, u'saligote': 0, u'surrénale': 3, u'responsoriale': 3, u'lilloise': 0, u'planante': 0, u'inquiète': 25, u'allumetière': 6, u'nourrissante': 0, u'caulescente': 0, u'anglaise': 0, u'commune': 0, u'malinoise': 0, u'contrefactrice': 1, u'éjective': 7, u'trisannuelle': 4, u'nippone': 0, u'gallo-romane': 0, u'bivaluée': 0, u'mignonnette': 4, u'palatiale': 3, u'érythémateuse': 9, u'triomphante': 0, u'lieutenante': 0, u'auto-réalisatrice': 1, u'alourdissante': 0, u'sabbathienne': 4, u'décoratrice': 1, u'catarrheuse': 9, u'guinéenne': 4, u'allergisante': 0, u'utilisatrice': 1, u'bluffante': 0, u'vicinale': 3, u'antéroventrale': 3, u'formatrice': 1, u'programmatrice': 1, u'laurée': 0, u'sortante': 0, u'pignorative': 7, u'illuviale': 3, u'bismarckienne': 4, u'ambroisienne': 4, u'aquicultrice': 1, u'prégénitale': 3, u'bernardine': 0, u'morbilleuse': 9, u'accompagnatrice': 1, u'inopinée': 0, u'filante': 0, u'maïeuticienne': 4, u'chamoiseuse': 2, u'schisteuse': 9, u'déterminative': 7, u'julienne': 4, u'entôleuse': 2, u'basculante': 0, u'soustractive': 7, u'objectale': 3, u'hollandaise': 0, u'joueuse': 2, u'langoureuse': 9, u'coutante': 0, u'anglo-française': 0, u'convoyeuse': 2, u'noiraude': 0, u'jouisseuse': 2, u'citrine': 0, u'péripatéticienne': 4, u'vapoteuse': 2, u'distributive': 7, u'ensileuse': 2, u'séropositive': 7, u'croissante': 0, u'sous-directrice': 1, u'orchestrale': 3, u'mariale': 3, u'explosive': 7, u'rapiate': 0, u'tarsale': 3, u'novatrice': 1, u'anglo-autrichienne': 4, u'virilocale': 3, u'figurative': 7, u'souffleuse': 2, u'fichue': 0, u'bagarreuse': 2, u'lettonne': 4, u'devancière': 6, u'défigurée': 0, u'occitanienne': 4, u'cézannienne': 4, u'arnaqueuse': 2, u'tensio-active': 7, u'pauvrette': 4, u'acuminée': 0, u'franco-paraguayenne': 4, u'entrepreneuse': 2, u'thermocollée': 0, u'fromentale': 3, u'désidérative': 7, u'précieuse': 9, u'afghano-pakistanaise': 0, u'inguinale': 3, u'univariante': 0, u'illimitée': 0, u'réveilleuse': 2, u'labiale': 3, u'tapineuse': 2, u'ossue': 0, u'insermentée': 0, u'hydrocarbonée': 0, u'trameuse': 2, u'disgracieuse': 9, u'cybercriminelle': 4, u'résurgente': 0, u'déprimante': 0, u'austronésienne': 4, u'blédarde': 0, u'adoratrice': 1, u'endormante': 0, u'germinale': 3, u'préadolescente': 0, u'vrombissante': 0, u'indicatrice': 1, u'youpine': 0, u'chaineuse': 2, u'sardinière': 6, u'lyncheuse': 2, u'ébavureuse': 2, u'rurbaine': 0, u'agioteuse': 2, u'ordonnancière': 6, u'calandreuse': 2, u'provinciale': 3, u'éventée': 0, u'volante': 0, u'hadale': 3, u'casernière': 6, u'idiote': 0, u'circonvoisine': 0, u'hypochloreuse': 9, u'parentérale': 3, u'tondeuse': 2, u'relative': 7, u'anévrismale': 3, u'mieux-disante': 0, u'profitante': 0, u'frustrante': 0, u'noblaillonne': 4, u'taupière': 6, u'oulipienne': 4, u'occasionnelle': 4, u'grecque': 29, u'loyale': 3, u'marchandeuse': 2, u'décalvante': 0, u'gothe': 0, u'récitatrice': 1, u'franco-tunisienne': 4, u'fessière': 6, u'adjacente': 0, u'zonarde': 0, u'implorante': 0, u'feignante': 0, u'déstabilisatrice': 1, u'joaillière': 6, u'bordière': 6, u'gantelée': 0, u'péronéo-tibiale': 3, u'amuseuse': 2, u'gâcheuse': 2, u'contremaitresse': 16, u'plurale': 3, u'denteleuse': 2, u'conformationnelle': 4, u'libyenne': 4, u'psychosociale': 3, u'extrafine': 0, u'approbatrice': 1, u'réactrice': 1, u'déviatrice': 1, u'gingivale': 3, u'révolutée': 0, u'stéréorégulière': 6, u'mésopotamienne': 4, u'myrmidone': 0, u'coxofémorale': 3, u'rustaude': 0, u'matriceuse': 2, u'prisonnière': 6, u'luciférienne': 4, u'patente': 0, u'préorale': 3, u'rédactrice': 1, u'lamaneuse': 2, u'arabo-musulmane': 0, u'inconsistante': 0, u'graniteuse': 9, u'riveuse': 2, u'pèquenotte': 4, u'péruvienne': 4, u'rubiconde': 0, u'vaillante': 0, u'omnipraticienne': 4, u'franco-hongroise': 0, u'conductrice': 1, u'prénatale': 0, u'séparative': 7, u'superattractive': 7, u'intercotidale': 3, u'apparitrice': 1, u'marine': 0, u'péjorative': 7, u'abdominale': 3, u'prussienne': 4, u'redentée': 0, u'suivante': 0, u'népérienne': 4, u'absidale': 3, u'gaffeuse': 2, u'courtoise': 0, u'rougissante': 0, u'microprogrammée': 0, u'lignarde': 0, u'ingrate': 0, u'iodurée': 0, u'inouïe': 0, u'bermudienne': 4, u'coquilleuse': 2, u'encalminée': 0, u'régularisante': 0, u'égalisatrice': 1, u'cimentière': 6, u'fière': 6, u'pastorale': 3, u'cogniticienne': 4, u'accapareuse': 2, u'intraartérielle': 4, u'immunisée': 0, u'bailleresse': 20, u'adjuvante': 0, u'plébéienne': 4, u'pharmacodépendante': 0, u'chalonnaise': 0, u'almandine': 0, u'consistoriale': 3, u'stimulante': 0, u'renardée': 0, u'médico-professionnelle': 4, u'rempailleuse': 2, u'diseuse': 2, u'dimissoriale': 3, u'omnisciente': 0, u'évasive': 7, u'polynomiale': 3, u'tabatière': 6, u'irrésignée': 0, u'quêteuse': 2, u'astrophysicienne': 4, u'afférente': 0, u'mouflette': 4, u'croupissante': 0, u'épieuse': 2, u'spiroïdale': 3, u'avoisinante': 0, u'nantie': 0, u'ostréicultrice': 1, u'conservatrice': 1, u'malfaitrice': 1, u'dédaléenne': 4, u'smicarde': 0, u'raisonnante': 0, u'lanceuse': 2, u'bonnarde': 0, u'verdoyante': 0, u'verrée': 0, u'bodybuildée': 0, u'tubéreuse': 9, u'bruinée': 0, u'généreuse': 9, u'intempérante': 0, u'diagnostiqueuse': 2, u'pourchasseuse': 2, u'divine': 0, u'amplifiante': 0, u'solidienne': 4, u'téléconduite': 0, u'louis-philipparde': 0, u'solutréenne': 4, u'usitée': 0, u'antimicrobienne': 4, u'salivante': 0, u'illettrée': 0, u'armoricaine': 0, u'thermidorienne': 4, u'gréeuse': 2, u'bachelière': 6, u'ci-jointe': 0, u'surhumaine': 0, u'corrodante': 0, u'adoptée': 0, u'rocheuse': 9, u'calomnieuse': 9, u'imbrûlée': 0, u'méridienne': 4, u'tonitruante': 0, u'apostate': 0, u'différente': 0, u'lutteuse': 2, u'pharaonienne': 4, u'éthiopienne': 4, u'conduiseuse': 2, u'pénarde': 0, u'sourceuse': 2, u'alliée': 0, u'mécontente': 0, u'citadine': 0, u'colorectale': 3, u'badine': 0, u'aldine': 0, u'expéditrice': 1, u'payse': 0, u'violente': 0, u'archétypale': 3, u'déformatrice': 1, u'pyroligneuse': 9, u'anguleuse': 9, u'lobulée': 0, u'convulsive': 7, u'infirmière': 6, u'exubérante': 0, u'suppressive': 7, u'namibienne': 4, u'nitrificatrice': 1, u'laitière': 6, u'houblonnière': 6, u'retraite': 0, u'bannie': 0, u'inspectrice': 1, u'ukrainienne': 4, u'franco-roumaine': 0, u'galvaudeuse': 9, u'indépendante': 0, u'divergente': 0, u'groupale': 3, u'calculatrice': 1, u'houleuse': 9, u'gliale': 3, u'gréco-latine': 0, u'khédivale': 3, u'tendeuse': 2, u'jalouse': 9, u'juteuse': 9, u'escompteuse': 2, u'concordante': 0, u'sclérosante': 0, u'multivaluée': 0, u'desséchante': 0, u'hussarde': 0, u'scélérate': 0, u'bruissante': 0, u'développée': 0, u'théologienne': 4, u'bombeuse': 2, u'normande': 0, u'ludoéducative': 7, u'gambadante': 0, u'honteuse': 9, u'roublarde': 0, u'incultivée': 0, u'usagère': 6, u'captieuse': 9, u'connective': 7, u'partisane': 0, u'panaméenne': 4, u'transnationale': 3, u'commensale': 3, u'cramoisie': 0, u'batailleuse': 2, u'éloquente': 0, u'stéphanoise': 0, u'inentamée': 0, u'correcte': 0, u'néocorticale': 3, u'bulleuse': 9, u'saxonne': 4, u'cabaretière': 6, u'herbagère': 6, u'extra-forte': 0, u'antirépublicaine': 0, u'slavisante': 0, u'volitionnelle': 4, u'hautelissière': 6, u'laryngienne': 4, u'subversive': 7, u'estroprogestative': 7, u'oléicultrice': 1, u'connaisseuse': 2, u'capitale': 3, u'abjecte': 0, u'neurovégétative': 7, u'indistincte': 0, u'abracadabrante': 0, u'rouennière': 6, u'insondée': 0, u'utriculeuse': 9, u'débarrasseuse': 2, u'réclamante': 0, u'délirante': 0, u'anglo-américaine': 0, u'poltronne': 4, u'rajeunissante': 0, u'assurée': 0, u'chahuteuse': 2, u'peccamineuse': 9, u'démaquillante': 0, u'papouasienne': 4, u'évolutive': 7, u'échafaudeuse': 2, u'testeuse': 2, u'dissolutive': 7, u'galoisienne': 4, u'privilégiée': 0, u'jurassienne': 4, u'délicate': 0, u'musulmane': 0, u'reptilienne': 4, u'déshumanisante': 0, u'multicanale': 3, u'professionnelle': 4, u'fiévreuse': 9, u'unilatérale': 3, u'gradée': 0, u'maximisatrice': 1, u'contemptrice': 1, u'nicaraguayenne': 4, u'artérielle': 4, u'cadastrale': 3, u'illégale': 3, u'dorienne': 4, u'sulpicienne': 4, u'crustale': 3, u'gaspilleuse': 2, u'discuteuse': 2, u'oléagineuse': 9, u'franco-surinamaise': 0, u'frangeante': 0, u'alléchante': 0, u'savonnière': 6, u'irrassasiée': 0, u'décente': 0, u'ombiliquée': 0, u'reproductive': 7, u'proudhonienne': 4, u'inchangée': 0, u'renversante': 0, u'cohésive': 7, u'factrice': 1, u'feutrante': 0, u'récoltante': 0, u'vertébrée': 0, u'calorifugeuse': 2, u'finlandaise': 0, u'occlusale': 3, u'bélizienne': 4, u'mâtine': 0, u'marchande': 0, u'matheuse': 9, u'marchante': 0, u'abductrice': 1, u'ultérieure': 0, u'captive': 7, u'grenadienne': 4, u'tumescente': 0, u'contrastante': 0, u'encaissante': 0, u'distributionnelle': 4, u'verglacée': 0, u'laie': 0, u'inférovariée': 0, u'interventionnelle': 4, u'croustillante': 0, u'patronne': 4, u'vasoconstrictrice': 1, u'faunesse': 16, u'gêneuse': 2, u'monopoleuse': 2, u'désodorisante': 0, u'plâtreuse': 9, u'cigarettière': 6, u'rectificatrice': 1, u'monocamérale': 3, u'glorieuse': 9, u'finisseuse': 2, u'opticienne': 4, u'raccommodeuse': 2, u'bobineuse': 2, u'européenne': 4, u'encadrante': 0, u'captative': 7, u'découpeuse': 2, u'réductrice': 1, u'châtelperronienne': 4, u'concrescente': 0, u'obligée': 0, u'palée': 0, u'étudiante': 0, u'picoreuse': 2, u'finassière': 6, u'contre-intuitive': 7, u'beauceronne': 4, u'ventrale': 3, u'dresseuse': 2, u'antiaérienne': 4, u'couvrante': 0, u'conflictuelle': 4, u'rotulienne': 4, u'organisatrice': 1, u'geignante': 0, u'sauvagine': 0, u'gravitationnelle': 4, u'administrative': 7, u'exorbitée': 0, u'sujette': 4, u'daubeuse': 2, u'préconçue': 0, u'tripoteuse': 2, u'ethnoculturelle': 4, u'déshydratante': 0, u'pantoise': 0, u'marqueteuse': 2, u'fouinarde': 0, u'réconciliatrice': 1, u'marbreuse': 2, u'tonale': 0, u'pendue': 0, u'incrémentielle': 4, u'théâtreuse': 9, u'possédée': 0, u'griotte': 4, u'ruminante': 0, u'cribleuse': 2, u'poisseuse': 9, u'chaloupée': 0, u'moutonnée': 0, u'faucheuse': 2, u'cuniculicultrice': 1, u'excommuniée': 0, u'fanatiseuse': 2, u'kafkaïenne': 4, u'incidentelle': 4, u'rengagée': 0, u'ployante': 0, u'inégalée': 0, u'hyperactive': 7, u'stoïcienne': 4, u'transcendante': 0, u'sectatrice': 1, u'crollée': 0, u'fréjusienne': 4, u'doucereuse': 9, u'empaleuse': 2, u'impure': 0, u'involucrée': 0, u'joliette': 4, u'herniée': 0, u'dénotationnelle': 4, u'éclusière': 6, u'envoyée': 0, u'partitive': 7, u'collégiale': 3, u'monoparentale': 3, u'étasunienne': 4, u'liquoreuse': 9, u'échauffée': 0, u'primo-accédante': 0, u'avignonnaise': 0, u'talqueuse': 9, u'friponne': 4, u'vicennale': 3, u'assidue': 0, u'encapuchonnée': 0, u'étalonneuse': 2, u'boursière': 6, u'pâteuse': 9, u'cintreuse': 2, u'cégésimale': 3, u'numérative': 7, u'rouée': 0, u'pipeuse': 2, u'détartrante': 0, u'antisociale': 3, u'impunie': 0, u'dépravée': 0, u'vernisseuse': 2, u'malplaisante': 0, u'phylloxérée': 0, u'suraigüe': 12, u'encolleuse': 2, u'excavatrice': 1, u'bicarbonatée': 0, u'gloutonne': 4, u'soumise': 0, u'bookmakeuse': 2, u'participative': 7, u'cofondatrice': 1, u'machinale': 3, u'émotionnelle': 4, u'liechtensteinoise': 0, u'encombrante': 0, u'privée': 0, u'polylobée': 0, u'paraverbale': 3, u'biculturelle': 4, u'échantillonneuse': 2, u'baie': 0, u'coraco-brachiale': 3, u'voluptueuse': 9, u'récolteuse': 2, u'cueilleuse': 2, u'conjonctive': 7, u'lésionnelle': 4, u'dolosive': 7, u'manganeuse': 9, u'saburrale': 3, u'anglo-indienne': 4, u'rabattue': 0, u'contente': 0, u'lionne': 4, u'auto-stoppeuse': 2, u'magyare': 0, u'peseuse': 2, u'coparentale': 3, u'javeleuse': 2, u'surale': 3, u'ambiguë': 0, u'qualificative': 7, u'immaculée': 0, u'frioulane': 0, u'légale': 3, u'cafetière': 6, u'bas-jointée': 0, u'auriculotemporale': 3, u'cérémonieuse': 9, u'croquignolette': 4, u'interalliée': 0, u'sourcilière': 6, u'sécante': 0, u'anti-seigneuriale': 3, u'déracinée': 0, u'bigleuse': 9, u'médiopalatale': 3, u'délictuelle': 4, u'osseuse': 9, u'psychoaffective': 7, u'féconde': 0, u'irrégulière': 6, u'anabolisante': 0, u'irritante': 0, u'extractive': 7, u'xiphoïdienne': 4, u'timoraise': 0, u'involutive': 7, u'perçante': 0, u'angolaise': 0, u'inclusive': 7, u'trachéotomisée': 0, u'phonocaptrice': 1, u'épousée': 0, u'viennoise': 0, u'annonceuse': 2, u'nord-africaine': 0, u'aristotélicienne': 4, u'quadrilobée': 0, u'connotative': 7, u'enivrante': 0, u'dispersive': 7, u'bananière': 6, u'tendancielle': 4, u'désireuse': 9, u'autosuffisante': 0, u'fuyarde': 0, u'avocate': 0, u'défricheuse': 2, u'championne': 4, u'cotonneuse': 9, u'djiboutienne': 4, u'nageuse': 2, u'usuelle': 4, u'médiévale': 3, u'philosophale': 0, u'trotteuse': 2, u'céruléenne': 4, u'bissextile': 0, u'attentive': 7, u'lézarde': 0, u'suturale': 3, u'polypeuse': 9, u'huissière': 6, u'bancheuse': 2, u'renforçante': 0, u'spectrale': 3, u'écopaysagère': 6, u'pastorienne': 4, u'immolatrice': 1, u'inauguratrice': 1, u'trivalente': 0, u'brocanteuse': 2, u'clicheuse': 2, u'modulatrice': 1, u'ravissante': 0, u'multirésistante': 0, u'tuyauteuse': 2, u'euskarienne': 4, u'sépulcrale': 3, u'médullosurrénale': 3, u'anglo-saxonne': 4, u'franco-coréenne': 4, u'autobronzante': 0, u'supplétive': 7, u'alternante': 0, u'concise': 0, u'infectante': 0, u'grelottante': 0, u'sénatrice': 1, u'malletière': 6, u'floconneuse': 9, u'toiletteuse': 2, u'cérébelleuse': 9, u'manadière': 6, u'relectrice': 1, u'méditative': 7, u'tutorale': 0, u'laplacienne': 4, u'globetrotteuse': 2, u'glaneuse': 2, u'germano-danoise': 0, u'germano-américaine': 0, u'pleureuse': 2, u'départementale': 3, u'stabilisatrice': 1, u'sorteuse': 2, u'chirurgicale': 3, u'pénicillée': 0, u'magrébine': 0, u'impotente': 0, u'séricicultrice': 1, u'décérébrée': 0, u'furieuse': 9, u'giletière': 6, u'décodeuse': 2, u'verticillée': 0, u'comparante': 0, u'interférentielle': 4, u'avantageuse': 9, u'effarante': 0, u'sumérienne': 4, u'commettante': 0, u'fastidieuse': 9, u'anglo-allemande': 0, u'camerounaise': 0, u'commandeuse': 2, u'insinuante': 0, u'attrayante': 0, u'instructrice': 1, u'vice-chancelière': 6, u'additive': 7, u'landaise': 0, u'ensorcelante': 0, u'suante': 0, u'horizontale': 3, u'bobeuse': 2, u'démarcheuse': 2, u'patronale': 3, u'épigée': 0, u'médicalisante': 0, u'saisissante': 0, u'pourvoyeuse': 2, u'prometteuse': 2, u'épierreuse': 2, u'stressante': 0, u'gravillonneuse': 2, u'naufragée': 0, u'extinctive': 7, u'bêta-testeuse': 2, u'ribosomale': 3, u'circonspecte': 0, u'crayonneuse': 9, u'mondiale': 3, u'beethovénienne': 4, u'fouleuse': 2, u'astronauticienne': 4, u'vivante': 0, u'médicochirurgicale': 3, u'outrancière': 6, u'appellative': 7, u'descriptive': 7, u'littératrice': 1, u'conversive': 7, u'flacheuse': 9, u'offensive': 7, u'factuelle': 4, u'baragouineuse': 2, u'œuvée': 0, u'norroise': 0, u'projective': 7, u'chansonnière': 6, u'incrémentale': 3, u'spirante': 0, u'injonctive': 7, u'rétreinte': 0, u'bioluminescente': 0, u'chauvine': 0, u'recto-sigmoïdienne': 4, u'puerpérale': 3, u'investigatrice': 1, u'moutonnante': 0, u'aléoutienne': 4, u'traficoteuse': 2, u'paysagée': 0, u'sereine': 0, u'barbue': 0, u'préquelle': 4, u'centrifugeuse': 2, u'sournoise': 0, u'harkie': 0, u'travailleuse': 2, u'ravigotante': 0, u'délibérative': 7, u'subséquente': 0, u'résiduelle': 4, u'colistière': 6, u'justificative': 7, u'vilaine': 0, u'ragoutante': 0, u'dénicheuse': 2, u'chleue': 0, u'sous-clavière': 6, u'boiseuse': 2, u'italo-autrichienne': 4, u'couenneuse': 9, u'velouteuse': 9, u'revive': 7, u'narcotrafiquante': 0, u'chiffonnière': 6, u'bourgeonnante': 0, u'américaine': 0, u'observationnelle': 4, u'azuréenne': 4, u'vive': 7, u'organochlorée': 0, u'prépositionnelle': 4, u'réactive': 7, u'demie': 0, u'gouleyante': 0, u'hertzienne': 4, u'pharisienne': 4, u'glottale': 3, u'enrouleuse': 2, u'criblière': 6, u'accoucheuse': 2, u'paniquarde': 0, u'hélicoïdale': 3, u'butanée': 0, u'pipière': 6, u'rafraîchissante': 0, u'amnistiée': 0, u'confiturière': 6, u'packageuse': 2, u'clairvoyante': 0, u'penchante': 0, u'réitérative': 7, u'footballeuse': 2, u'parleuse': 2, u'nigériane': 0, u'disparue': 0, u'oscillo-coulissante': 0, u'mudéjare': 0, u'perforatrice': 1, u'croassante': 0, u'matérielle': 4, u'accrescente': 0, u'anamorphosée': 0, u'délitescente': 0, u'tensorielle': 4, u'dijonnaise': 0, u'survivante': 0, u'imposeuse': 2, u'énervante': 0, u'primitive': 7, u'exponentielle': 4, u'dominée': 0, u'semencière': 6, u'équitante': 0, u'aviatrice': 1, u'remplaçante': 0, u'téléopératrice': 1, u'reconstructive': 7, u'nuptiale': 3, u'rédactionnelle': 4, u'garibaldienne': 4, u'rhomboïdale': 3, u'universelle': 4, u'omniprésente': 0, u'coexistante': 0, u'respective': 7, u'sétacée': 0, u'foreuse': 2, u'usurière': 6, u'cotisante': 0, u'épuratrice': 1, u'faisandière': 6, u'slavonne': 4, u'chirurgienne': 4, u'franco-chilienne': 4, u'libertine': 0, u'récriminatrice': 1, u'anglo-normande': 0, u'rassurante': 0, u'décathlonienne': 4, u'térébrante': 0, u'fluviale': 3, u'secouante': 0, u'déhiscente': 0, u'transfuseuse': 2, u'damnée': 0, u'exploratrice': 1, u'inductive': 7, u'ennemie': 0, u'droguée': 0, u'attenante': 0, u'paternelle': 4, u'vaccinale': 3, u'bichonne': 4, u'brownienne': 4, u'pharamineuse': 9, u'néocoloniale': 3, u'fascinante': 0, u'aliénatrice': 1, u'métallière': 6, u'extraterritoriale': 3, u'khalifale': 3, u'époustouflante': 0, u'équidimensionnelle': 4, u'pique-niqueuse': 2, u'climatisante': 0, u'adolescente': 0, u'récifale': 3, u'gaufreuse': 2, u'corédactrice': 1, u'aromaticienne': 4, u'gypseuse': 9, u'tessellée': 0, u'contuse': 0, u'débatteuse': 2, u'coffreuse': 2, u'centralisatrice': 1, u'agrégative': 7, u'déstressante': 0, u'outrecuidante': 0, u'puante': 0, u'énergéticienne': 4, u'philippienne': 4, u'centreuse': 2, u'haline': 0, u'logeuse': 2, u'subconsciente': 0, u'romane': 0, u'lupercale': 3, u'hongreuse': 2, u'corticosurrénale': 3, u'délicieuse': 9, u'moelleuse': 9, u'serpentante': 0, u'fleurissante': 0, u'scléreuse': 9, u'tonifiante': 0, u'anglo-néerlandaise': 0, u'angoisseuse': 9, u'libéroligneuse': 9, u'highlandeuse': 2, u'hernieuse': 9, u'préindustrielle': 4, u'communale': 3, u'spiralée': 0, u'nantaise': 0, u'bitonale': 3, u'agissante': 0, u'hypercorrecte': 0, u'façonneuse': 2, u'receveuse': 2, u'récessive': 7, u'bégayeuse': 2, u'dessiccatrice': 1, u'identificatrice': 1, u'angoreuse': 9, u'intendante': 0, u'sensée': 0, u'rossarde': 0, u'affétée': 0, u'ouvrante': 0, u'souterraine': 0, u'relaxée': 0, u'retardatrice': 1, u'modificative': 7, u'foisonnante': 0, u'empoisonnante': 0, u'maorie': 0, u'élaboratrice': 1, u'kosovare': 0, u'culminante': 0, u'équipotentielle': 4, u'anomale': 3, u'interagissante': 0, u'râteleuse': 2, u'isolatrice': 1, u'voleuse': 2, u'petiote': 0, u'hypothéticodéductive': 7, u'sifflante': 0, u'flicarde': 0, u'pleurnicheuse': 2, u'haschichée': 0, u'colossienne': 4, u'diagnosticienne': 4, u'changeante': 0, u'affleurante': 0, u'séculière': 6, u'astacicultrice': 1, u'succédanée': 0, u'malayo-polynésienne': 4, u'grimaçante': 0, u'loucheuse': 2, u'fasciculée': 0, u'céréalicultrice': 1, u'moraliseuse': 2, u'inférieure': 0, u'géothermale': 3, u'chrétienne': 4, u'coagulante': 0, u'sérielle': 4, u'macédonienne': 4, u'transamazonienne': 4, u'transfiguratrice': 1, u'tigresse': 16, u'limoneuse': 9, u'péri-urbaine': 0, u'frimeuse': 2, u'masticatrice': 1, u'caritative': 7, u'portugaise': 0, u'turbulente': 0, u'endormeuse': 2, u'éléphantine': 0, u'baptiseuse': 2, u'débardeuse': 2, u'éveillée': 0, u'hercheuse': 2, u'cornière': 6, u'dévideuse': 2, u'désinentielle': 4, u'extrovertie': 0, u'lipschitzienne': 4, u'peureuse': 9, u'méchante': 0, u'infamante': 0, u'curieuse': 9, u'impartiale': 3, u'antimorale': 3, u'persulfurée': 0, u'elzévirienne': 4, u'ramendeuse': 2, u'tapeuse': 2, u'alimentatrice': 1, u'énonciatrice': 1, u'pernicieuse': 9, u'archère': 6, u'pénale': 3, u'claironnante': 0, u'lourde': 0, u'jointoyeuse': 2, u'snipeuse': 2, u'photo-émettrice': 1, u'chauffante': 0, u'médiate': 0, u'métamérisée': 0, u'anglo-pakistanaise': 0, u'confessionnelle': 4, u'châtaine': 0, u'texane': 0, u'relationnelle': 4, u'subsaharienne': 4, u'formicante': 0, u'sudorale': 3, u'prévaricatrice': 1, u'componctueuse': 9, u'sous-calibrée': 0, u'camérière': 6, u'dévorante': 0, u'adjectivale': 3, u'parchemineuse': 9, u'bolivarienne': 4, u'confidentielle': 4, u'couarde': 0, u'vertueuse': 9, u'zozoteuse': 2, u'écureuse': 2, u'aspiratrice': 1, u'serricultrice': 1, u'pollueuse': 2, u'réceptive': 7, u'approximative': 7, u'émondeuse': 2, u'citronnée': 0, u'constitutionnelle': 4, u'tamisière': 6, u'pélagienne': 4, u'partouzeuse': 2, u'virginale': 3, u'asphalteuse': 2, u'journalière': 6, u'sucrante': 0, u'rinceuse': 2, u'chercheuse': 2, u'ratière': 6, u'retorse': 0, u'deviseuse': 2, u'disciplinée': 0, u'pâlissante': 0, u'expansée': 0, u'excitante': 0, u'cruelle': 4, u'éclairante': 0, u'suréminente': 0, u'américano-australienne': 4, u'bienheureuse': 9, u'poivrote': 0, u'dégressive': 7, u'bringuebalante': 0, u'refondatrice': 1, u'cocréancière': 6, u'larvée': 0, u'navigatrice': 1, u'fortuite': 0, u'boursicotière': 6, u'auxiliatrice': 1, u'tridimensionnelle': 4, u'photocopieuse': 2, u'sauveuse': 2, u'inclémente': 0, u'perpétuelle': 4, u'parvenue': 0, u'archiprêtresse': 16, u'semi-marathonienne': 4, u'agaçante': 0, u'fissurale': 3, u'costaude': 0, u'confondante': 0, u'trafiqueuse': 2, u'rectosigmoïdienne': 4, u'chichiteuse': 9, u'perforelieuse': 2, u'avertisseuse': 2, u'compromettante': 0, u'coréalisatrice': 1, u'coécrite': 0, u'cascadeuse': 2, u'séquentielle': 4, u'ardue': 0, u'fabricatrice': 1, u'impensée': 0, u'délégatrice': 1, u'déviante': 0, u'visigothe': 0, u'guichetière': 6, u'gazière': 6, u'lépreuse': 9, u'émettrice': 1, u'inspiratrice': 1, u'mûrie': 0, u'théodosienne': 4, u'destinatrice': 1, u'émulsifiante': 0, u'bractéale': 3, u'adoptante': 0, u'homoparentale': 3, u'totipotente': 0, u'coutumière': 6, u'humaine': 0, u'débosseleuse': 2, u'sèche': 32, u'fraîche': 28, u'référentielle': 4, u'clubbeuse': 2, u'stalinienne': 4, u'tamile': 0, u'hiémale': 3, u'anesthésiante': 0, u'malsonnante': 0, u'batelière': 6, u'déhontée': 0, u'dealeuse': 2, u'antidreyfusarde': 0, u'nécromancienne': 4, u'traitante': 0, u'vétérane': 0, u'nauséeuse': 9, u'immigrante': 0, u'faciale': 3, u'opprimée': 0, u'érudite': 0, u'condruzienne': 4, u'empotée': 0, u'bloquante': 0, u'éruptive': 7, u'ruineuse': 9, u'vietnamienne': 4, u'jouteuse': 2, u'laborieuse': 9, u'brodeuse': 2, u'inappliquée': 0, u'stridulante': 0, u'transporteuse': 2, u'besantée': 0, u'équatorienne': 4, u'mentale': 3, u'tricamérale': 3, u'poulotte': 4, u'sibilante': 0, u'oléfiante': 0, u'hesbignonne': 4, u'solennelle': 4, u'algérienne': 4, u'rougeaude': 0, u'inférentielle': 4, u'subrogée': 0, u'buissonnante': 0, u'marrante': 0, u'détaillante': 0, u'sidérale': 3, u'artificieuse': 9, u'énumérative': 7, u'vénielle': 4, u'ajaccienne': 4, u'campagnarde': 0, u'mahométane': 0, u'tiraillante': 0, u'chassieuse': 9, u'choquante': 0, u'assassine': 0, u'entière': 6, u'psychotechnicienne': 4, u'mentholée': 0, u'répétitrice': 1, u'expropriante': 0, u'ventripotente': 0, u'pénitente': 0, u'prétranchée': 0, u'totale': 3, u'dentelière': 6, u'variationnelle': 4, u'proscrite': 0, u'traîneuse': 2, u'pelleteuse': 2, u'élagueuse': 2, u'piqueniqueuse': 2, u'bronchodilatatrice': 1, u'variolée': 0, u'agréeuse': 2, u'courtille': 4, u'écaillère': 6, u'niqueuse': 2, u'pseudodifférentielle': 4, u'picoleuse': 2, u'particulière': 6, u'falsificatrice': 1, u'cananéenne': 4, u'antirationnelle': 4, u'apollinienne': 4, u'arnaquée': 0, u'bessonne': 4, u'tabletière': 6, u'pigeonnante': 0, u'bruineuse': 9, u'suppléante': 0, u'autopsiée': 0, u'faîtière': 6, u'débauchée': 0, u'capucine': 0, u'informaticienne': 4, u'antituberculeuse': 9, u'pariétale': 3, u'médullo-surrénale': 0, u'hivernale': 3, u'économico-sociale': 3, u'vésicante': 0, u'nasarde': 0, u'ivoirière': 6, u'préexistante': 0, u'piqueuse': 9, u'vairée': 0, u'enfonceuse': 2, u'remettante': 0, u'quiète': 25, u'ammoniacale': 3, u'antiadhésive': 7, u'contemplatrice': 1, u'matoise': 0, u'introductrice': 1, u'préhospitalière': 6, u'pestiférée': 0, u'primo-délinquante': 0, u'toulousaine': 0, u'aérienne': 4, u'débutante': 0, u'scaphandrière': 6, u'annuelle': 4, u'grisonne': 4, u'hypersustentatrice': 1, u'ultralégère': 6, u'valeureuse': 9, u'luminescente': 0, u'sénestrée': 0, u'évoluée': 0, u'traceuse': 2, u'desservante': 0, u'raseuse': 2, u'bluffeuse': 2, u'scribouillarde': 0, u'rectorale': 3, u'quinteuse': 9, u'étalonnière': 6, u'brillante': 0, u'tourbeuse': 9, u'amibienne': 4, u'buteuse': 2, u'transpirante': 0, u'compressive': 7, u'diaconale': 3, u'sauvageonne': 4, u'sexuelle': 4, u'crevetticultrice': 1, u'locatrice': 1, u'commémorative': 7, u'vielleuse': 9, u'chasseuse': 2, u'liftière': 6, u'marécageuse': 9, u'rhabilleuse': 2, u'autolissante': 0, u'laudative': 7, u'bonnetière': 6, u'productive': 7, u'parsie': 0, u'sempiternelle': 4, u'gémisseuse': 2, u'franco-norvégienne': 4, u'vivificatrice': 1, u'pouilleuse': 9, u'architecturale': 3, u'gagneuse': 2, u'mammalienne': 4, u'pressureuse': 2, u'quercitrine': 0, u'passionnelle': 4, u'forestière': 6, u'tractrice': 1, u'mûrissante': 0, u'enthousiasmante': 0, u'reproductrice': 1, u'salonnière': 6, u'mineure': 0, u'réjouissante': 0, u'associée': 0, u'convolutive': 7, u'énergisante': 0, u'agenceuse': 2, u'îlotière': 6, u'chaisière': 6, u'dento-faciale': 3, u'ultraconservatrice': 1, u'pataude': 0, u'vectrice': 1, u'roborative': 7, u'froufroutante': 0, u'anténatale': 0, u'sanctificatrice': 1, u'polynésienne': 4, u'provençale': 3, u'rançonneuse': 2, u'ondoyante': 0, u'dénigrante': 0, u'glaireuse': 9, u'germée': 0, u'rhumatismale': 3, u'siliceuse': 9, u'inversive': 7, u'justificatrice': 1, u'sondeuse': 2, u'interhumaine': 0, u'pyrotechnicienne': 4, u'créative': 7, u'wesleyenne': 4, u'pékinée': 0, u'agglutinante': 0, u'pascale': 0, u'transfinie': 0, u'athéromateuse': 9, u'démoulante': 0, u'engazonneuse': 2, u'paniculée': 0, u'pétiolée': 0, u'khédiviale': 3, u'franco-albanaise': 0, u'microfibrée': 0, u'détoxifiante': 0, u'surenchérisseuse': 2, u'amirale': 3, u'médico-légale': 3, u'versante': 0, u'systémicienne': 4, u'galante': 0, u'criailleuse': 2, u'régénérative': 7, u'liliacée': 0, u'manipulatrice': 1, u'proportionnelle': 4, u'battante': 0, u'rappeuse': 2, u'fusilière': 6, u'chelléenne': 4, u'coalescente': 0, u'carieuse': 9, u'multiraciale': 3, u'argileuse': 9, u'cuissarde': 0, u'dépenaillée': 0, u'coproductrice': 1, u'prévoyante': 0, u'zingueuse': 2, u'sélacienne': 4, u'abrutisseuse': 2, u'rampante': 0, u'phalangienne': 4, u'cliveuse': 2, u'votive': 7, u'radioamatrice': 1, u'recourante': 0, u'glisseuse': 2, u'pleurnicharde': 0, u'camarde': 0, u'tensioactive': 7, u'uruguayenne': 4, u'protéagineuse': 9, u'auvergnate': 0, u'conditionnelle': 4, u'costaricaine': 0, u'rayonnante': 0, u'cristallisante': 0, u'atomicienne': 4, u'concurrente': 0, u'oiselière': 6, u'autocaravanière': 6, u'constante': 0, u'smurfeuse': 2, u'ancestrale': 3, u'généralisatrice': 1, u'nigérienne': 4, u'instrumentale': 3, u'consciente': 0, u'rouleuse': 2, u'obturatrice': 1, u'givreuse': 9, u'ingénieuse': 9, u'manducatrice': 1, u'plurivalente': 0, u'faraude': 0, u'césarienne': 4, u'affadissante': 0, u'vidienne': 4, u'multivalente': 0, u'denticulée': 0, u'prenante': 0, u'ptoléméenne': 4, u'vulnérante': 0, u'viveuse': 2, u'crapoteuse': 9, u'audiovisuelle': 4, u'cendreuse': 9, u'balisticienne': 4, u'intolérante': 0, u'perceptive': 7, u'incorporatrice': 1, u'horlogère': 6, u'navale': 0, u'manieuse': 2, u'stercorale': 3, u'réfugiée': 0, u'observatrice': 1, u'puéricultrice': 1, u'arthurienne': 4, u'batteuse': 2, u'gallicane': 0, u'organisationnelle': 4, u'secrète': 25, u'sérieuse': 9, u'rupine': 0, u'astéroïdale': 3, u'réale': 3, u'biduale': 3, u'incandescente': 0, u'fulgurante': 0, u'monnayeuse': 2, u'équipotente': 0, u'conjugale': 3, u'nominante': 0, u'dissipative': 7, u'soulonne': 4, u'comparative': 7, u'isolée': 0, u'paulicienne': 4, u'bafouilleuse': 2, u'frileuse': 9, u'flaireuse': 2, u'croissantée': 0, u'stomacale': 3, u'agro-industrielle': 4, u'bergsonienne': 4, u'portière': 6, u'persécutée': 0, u'robineuse': 9, u'condylienne': 4, u'courtaude': 0, u'désabusée': 0, u'carrossière': 6, u'intercommunale': 3, u'prévôtale': 3, u'linéale': 3, u'pénétrante': 0, u'laudanisée': 0, u'surintendante': 0, u'fervente': 0, u'pacsée': 0, u'ougandaise': 0, u'glorifiante': 0, u'poreuse': 9, u'grande': 0, u'fatale': 0, u'abstruse': 0, u'grise': 0, u'aéroréfrigérante': 0, u'conventuelle': 4, u'pilo-sébacée': 0, u'alcalino-terreuse': 9, u'mycélienne': 4, u'italo-éthiopienne': 4, u'assiégée': 0, u'constructive': 7, u'saillante': 0, u'morale': 3, u'modeleuse': 2, u'stricte': 0, u'minérale': 3, u'essorante': 0, u'dynamisante': 0, u'allodiale': 3, u'onciale': 3, u'anodine': 0, u'pronatrice': 1, u'bruleuse': 2, u'désarmante': 0, u'papale': 3, u'dépossédée': 0, u'socinienne': 4, u'échangeuse': 2, u'multispectrale': 3, u'apéritrice': 1, u'paysagère': 6, u'veilleuse': 2, u'urticarienne': 4, u'développeuse': 2, u'australe': 3, u'manuelle': 4, u'fécondante': 0, u'effileuse': 2, u'tripartie': 0, u'puritaine': 0, u'amaigrissante': 0, u'inhumée': 0, u'filleule': 0, u'chalengeuse': 2, u'chiqueuse': 2, u'sartrienne': 4, u'gaullienne': 4, u'judaïsante': 0, u'vanneuse': 2, u'transformationnelle': 4, u'pipelette': 4, u'recréatrice': 1, u'liquéfiante': 0, u'égrisée': 0, u'nigéro-congolaise': 0, u'béquillarde': 0, u'flammée': 0, u'boulotte': 4, u'bondissante': 0, u'transformante': 0, u'orwellienne': 4, u'luisante': 0, u'inductrice': 1, u'cunicultrice': 1, u'spirale': 3, u'palléale': 3, u'agrammaticale': 3, u'nasillarde': 0, u'suboptimale': 3, u'révélatrice': 1, u'violatrice': 1, u'rectifieuse': 2, u'aérodynamicienne': 4, u'périœsophagienne': 4, u'primoaccédante': 0, u'acceptante': 0, u'vivandière': 6, u'propharmacienne': 4, u'brancardière': 6, u'castillane': 0, u'russo-américaine': 0, u'vasouillarde': 0, u'hyposulfureuse': 9, u'dubitative': 7, u'éclatante': 0, u'générale': 3, u'qualiticienne': 4, u'acquisitive': 7, u'suraiguë': 0, u'théoricienne': 4, u'ronfleuse': 2, u'boulangère': 6, u'maximale': 3, u'démarqueuse': 2, u'nazillonne': 4, u'vocationnelle': 4, u'nerveuse': 9, u'laitée': 0, u'encartoucheuse': 2, u'hexadécimale': 3, u'maquignonne': 4, u'matinière': 6, u'déchiffreuse': 2, u'geignarde': 0, u'fêlée': 0, u'brûlante': 0, u'grouillante': 0, u'maxillo-faciale': 3, u'briqueteuse': 2, u'préœdipienne': 4, u'créditrice': 1, u'frisottante': 0, u'incomplète': 25, u'carrée': 0, u'inharmonieuse': 9, u'sanguine': 0, u'éprise': 0, u'transcendantale': 3, u'marguillère': 6, u'flatteuse': 2, u'psychomotricienne': 4, u'hiératisée': 0, u'détergente': 0, u'chafouine': 0, u'télévendeuse': 2, u'couteuse': 9, u'consécutive': 7, u'audio-visuelle': 4, u'coruscante': 0, u'franco-française': 0, u'court-vêtue': 0, u'dentue': 0, u'ducale': 3, u'plumetée': 0, u'opportune': 0, u'autoéquilibrante': 0, u'griffeuse': 2, u'héraclitéenne': 4, u'caséeuse': 9, u'interraciale': 3, u'injective': 7, u'remémorative': 7, u'éliminatrice': 1, u'coordonnatrice': 1, u'vannière': 6, u'presciente': 0, u'négociatrice': 1, u'spatiale': 3, u'malchanceuse': 9, u'louangeuse': 2, u'oxycoupeuse': 2, u'attacheuse': 2, u'chuchotante': 0, u'illuminée': 0, u'plasmolysée': 0, u'nettoyante': 0, u'constrictive': 7, u'coticée': 0, u'confiante': 0, u'hispano-américaine': 0, u'esseulée': 0, u'cagnarde': 0, u'extensionnelle': 4, u'déconnante': 0, u'blanchissante': 0, u'déréelle': 4, u'sempervirente': 0, u'encadreuse': 2, u'édulcorante': 0, u'endogée': 0, u'vérificative': 7, u'ronchonneuse': 2, u'corticale': 3, u'possessionnée': 0, u'opacifiante': 0, u'artificielle': 4, u'asociale': 3, u'tuberculeuse': 9, u'syndicale': 3, u'stadière': 6, u'ultraconfidentielle': 4, u'patriciale': 3, u'cornemuseuse': 2, u'préteuse': 2, u'superstitieuse': 9, u'anoblissante': 0, u'extra-fine': 0, u'inattentive': 7, u'alfatière': 6, u'haïtienne': 4, u'durative': 7, u'jambée': 0, u'exigeante': 0, u'expectante': 0, u'publique': 13, u'avare': 0, u'sud-vietnamienne': 4, u'théière': 6, u'ordovicienne': 4, u'pékinoise': 0, u'surprotectrice': 1, u'napolitaine': 0, u'espagnole': 0, u'nettoyeuse': 2, u'regimbeuse': 2, u'brosseuse': 2, u'uropygienne': 4, u'diffamatrice': 1, u'multilatérale': 3, u'poplitée': 0, u'réflexive': 7, u'rotationnelle': 4, u'positive': 7, u'profanatrice': 1, u'alitée': 0, u'frappeuse': 2, u'contraposée': 0, u'maïeure': 0, u'gravatière': 6, u'trésorière': 6, u'ombellée': 0, u'converse': 0, u'agneline': 0, u'multifactorielle': 4, u'échéante': 0, u'incisive': 7, u'augmentative': 7, u'univalente': 0, u'zostérienne': 4, u'transpadane': 0, u'auto-référentielle': 4, u'antédiluvienne': 4, u'vagissante': 0, u'adragante': 0, u'cérulée': 0, u'décourageante': 0, u'diocésaine': 0, u'nominale': 3, u'débraillée': 0, u'supercavitante': 0, u'peluchée': 0, u'écrue': 0, u'tufière': 6, u'croupie': 0, u'désinscrite': 0, u'sahraouie': 0, u'biphasée': 0, u'muscardine': 0, u'résidente': 0, u'nanisante': 0, u'coiffeuse': 2, u'rogneuse': 2, u'maltaise': 0, u'héritière': 6, u'loseuse': 2, u'muséale': 3, u'crétine': 0, u'boutiquière': 6, u'transitionnelle': 4, u'volitive': 7, u'tubulée': 0, u'grinçante': 0, u'envahisseuse': 2, u'palladienne': 4, u'thermisée': 0, u'perquisitionnée': 0, u'pétrisseuse': 2, u'arroseuse': 2, u'ordinatrice': 1, u'physicienne': 4, u'écorceuse': 2, u'rechignée': 0, u'arboricultrice': 1, u'exploitée': 0, u'marneuse': 9, u'robinetière': 6, u'stabilisante': 0, u'immunosuppressive': 7, u'télémécanicienne': 4, u'usante': 0, u'indisciplinée': 0, u'trapézoïdale': 3, u'batracienne': 4, u'tourtelée': 0, u'ardennaise': 0, u'benjamine': 0, u'moutonnière': 6, u'maladive': 7, u'surlignante': 0, u'désherbante': 0, u'abyssinienne': 4, u'pellagreuse': 9, u'jargonneuse': 9, u'randonneuse': 2, u'annotatrice': 1, u'griffonneuse': 2, u'carnée': 0, u'coulante': 0, u'bouilleuse': 2, u'obstinée': 0, u'enculeuse': 2, u'agenouillée': 0, u'choyée': 0, u'jacassante': 0, u'polytechnicienne': 4, u'immune': 0, u'ravageuse': 2, u'binationale': 3, u'ourdisseuse': 2, u'surfaceuse': 2, u'poivrière': 6, u'matricielle': 4, u'proximale': 3, u'jaseuse': 2, u'litigieuse': 9, u'géorgienne': 4, u'sacrifiée': 0, u'inconsolée': 0, u'courte': 0, u'jaunette': 4, u'gérante': 0, u'aimantine': 0, u'balbutiante': 0, u'confucéenne': 4, u'communarde': 0, u'vauclusienne': 4, u'kinoise': 0, u'caussenarde': 0, u'restauratrice': 1, u'populicultrice': 1, u'régisseuse': 2, u'bienséante': 0, u'pentue': 0, u'agrippante': 0, u'qualifiée': 0, u'défonceuse': 2, u'mousseuse': 9, u'ingressive': 7, u'agresseuse': 2, u'pailleuse': 9, u'pouponnée': 0, u'restrictive': 7, u'contributrice': 1, u'impermanente': 0, u'détentrice': 1, u'intermodale': 3, u'abêtifiée': 0, u'exfoliante': 0, u'militaro-industrielle': 4, u'surpuissante': 0, u'reçue': 0, u'décisionnelle': 4, u'assembleuse': 2, u'valseuse': 2, u'éventuelle': 4, u'déstabilisante': 0, u'sous-frutescente': 0, u'déférente': 0, u'découverte': 0, u'gymnasienne': 4, u'dissonante': 0, u'passante': 0, u'brouilleuse': 2, u'fibreuse': 9, u'faiblarde': 0, u'lilliputienne': 4, u'bancale': 0, u'paléochrétienne': 4, u'communicante': 0, u'chevaline': 0, u'habilleuse': 2, u'interpariétale': 3, u'téléprospectrice': 1, u'meunière': 6, u'trilatérale': 3, u'dupeuse': 2, u'octale': 3, u'annoncière': 6, u'bordelaise': 0, u'louchonne': 4, u'bardeuse': 2, u'samaritaine': 0, u'toxiinfectieuse': 9, u'malvoyante': 0, u'ménine': 0, u'morbihannaise': 0, u'gardiane': 0, u'rondelette': 4, u'sternale': 3, u'moqueuse': 2, u'touffue': 0, u'vésico-utérine': 0, u'berneuse': 2, u'septennale': 3, u'tortue': 0, u'performante': 0, u'piétonnière': 6, u'irraisonnée': 0, u'défoliatrice': 1, u'aléseuse': 2, u'panetière': 6, u'controlatérale': 3, u'baroudeuse': 2, u'cracovienne': 4, u'psychosensorielle': 4, u'interruptrice': 1, u'néo-platonicienne': 4, u'immorale': 3, u'belgo-congolaise': 0, u'bijective': 7, u'impaludée': 0, u'scintillante': 0, u'transférentielle': 4, u'intrarachidienne': 4, u'mouvante': 0, u'rescindante': 0, u'inattendue': 0, u'cérébro-spinale': 3, u'créatrice': 1, u'crolée': 0, u'douillette': 4, u'grandiloquente': 0, u'sure': 0, u'archetière': 6, u'oscillatrice': 1, u'dérivative': 7, u'pélasgienne': 4, u'trompeuse': 2, u'triphasée': 0, u'énarquienne': 4, u'américano-danoise': 0, u'parachutale': 3, u'latente': 0, u'fouteuse': 2, u'coprésidente': 0, u'gente': 0, u'murmurante': 0, u'séléniteuse': 9, u'concurrentielle': 4, u'semainière': 6, u'remblayeuse': 2, u'assonante': 0, u'conceptuelle': 4, u'gauchisante': 0, u'prénormale': 3, u'embrasseuse': 2, u'basse-lissière': 6, u'compteuse': 2, u'blousante': 0, u'décompensée': 0, u'rosière': 6, u'trigéminée': 0, u'assertive': 7, u'septale': 3, u'crédirentière': 6, u'intoxiquée': 0, u'déshéritée': 0, u'feinteuse': 2, u'infernale': 3, u'postdoctorante': 0, u'consultrice': 1, u'diadémée': 0, u'ignorante': 0, u'curarisante': 0, u'extra-sensorielle': 4, u'turriculée': 0, u'pétanqueuse': 2, u'herborisatrice': 1, u'babysitteuse': 2, u'gangréneuse': 9, u'chaînière': 6, u'demeurée': 0, u'assureuse': 2, u'gigotée': 0, u'ouzbèke': 30, u'cristalline': 0, u'préélectorale': 3, u'arachnoïdienne': 4, u'lésinante': 0, u'campignienne': 4, u'uvale': 3, u'incrusteuse': 2, u'sincipitale': 3, u'anglicane': 0, u'roboticienne': 4, u'discoureuse': 2, u'assommée': 0, u'canadienne': 4, u'boratée': 0, u'modernisatrice': 1, u'affaiblissante': 0, u'picarde': 0, u'capillicultrice': 1, u'lumineuse': 9, u'susurrante': 0, u'boisselière': 6, u'antivénéneuse': 9, u'glaçante': 0, u'unitarienne': 4, u'métreuse': 2, u'rouleautée': 0, u'embobineuse': 2, u'frisante': 0, u'gommeuse': 9, u'céréalière': 6, u'lydienne': 4, u'débitante': 0, u'bhoutanaise': 0, u'coutelière': 6, u'merdeuse': 9, u'sphéroïdale': 3, u'alphabétiseuse': 2, u'tienne': 4, u'pillarde': 0, u'carpettière': 6, u'marqueuse': 2, u'crapoussine': 0, u'pistarde': 0, u'crooneuse': 2, u'enchaînée': 0, u'irrespectueuse': 9, u'rosacée': 0, u'inopérante': 0, u'professorale': 3, u'assaillante': 0, u'italo-française': 0, u'égoutière': 6, u'adaptative': 7, u'coaccusée': 0, u'ennéagonale': 3, u'marcheuse': 2, u'conserveuse': 2, u'assamaise': 0, u'supermassive': 7, u'sus-jacente': 0, u'gueuse': 9, u'rhino-pharyngienne': 4, u'poilante': 0, u'lauréate': 0, u'irrévérente': 0, u'irrigante': 0, u'thermodynamicienne': 4, u'abolitive': 7, u'parastatale': 3, u'maniacodépressive': 7, u'néolamarckienne': 4, u'migrante': 0, u'bouvière': 6, u'pulvérulente': 0, u'gitane': 0, u'meulière': 6, u'prolétarienne': 4, u'infralittorale': 3, u'immunosupprimée': 0, u'hypnotiseuse': 2, u'racleuse': 2, u'ottomane': 0, u'téléassistante': 0, u'traçante': 0, u'anglo-japonaise': 0, u'gageuse': 2, u'chiffreuse': 2, u'surmultipliée': 0, u'angevine': 0, u'amicrobienne': 4, u'jaboteuse': 2, u'nestorienne': 4, u'carcérale': 3, u'moulinière': 6, u'contractuelle': 4, u'délégante': 0, u'spatio-temporelle': 4, u'divalente': 0, u'stéréosélective': 7, u'inintéressante': 0, u'chevelue': 0, u'préoccupante': 0, u'locorégionale': 3, u'composteuse': 2, u'indéhiscente': 0, u'engageante': 0, u'inaccentuée': 0, u'exémateuse': 9, u'neutralisante': 0, u'lyonnaise': 0, u'effrénée': 0, u'mildiousée': 0, u'pondératrice': 1, u'culturelle': 4, u'dissociatrice': 1, u'légitimée': 0, u'ottonienne': 4, u'fileuse': 2, u'postulante': 0, u'démoralisante': 0, u'polissonne': 4, u'tatoueuse': 2, u'limitante': 0, u'intergénérationnelle': 4, u'furette': 4, u'presbytérale': 3, u'états-unienne': 4, u'futée': 0, u'néo-conservatrice': 1, u'colonelle': 4, u'vitaminée': 0, u'mortaiseuse': 2, u'unifamiliale': 3, u'auto-immune': 0, u'sommatrice': 1, u'rocailleuse': 9, u'levée': 0, u'narratrice': 1, u'mécanicienne': 4, u'hondurienne': 4, u'criarde': 0, u'informulée': 0, u'rondouillarde': 0, u'malentendante': 0, u'causante': 0, u'implosée': 0, u'aliénante': 0, u'quadrilatérale': 3, u'complétive': 7, u'détacheuse': 2, u'susjacente': 0, u'rhinopharyngienne': 4, u'inhibée': 0, u'échevinale': 3, u'isolante': 0, u'péteuse': 9, u'sociale': 3, u'brulée': 0, u'multidimensionnelle': 4, u'outilleuse': 2, u'cuisinière': 6, u'actrice': 1, u'néo-hébridaise': 0, u'huileuse': 9, u'québecoise': 0, u'ébaucheuse': 2, u'marxienne': 4, u'massicotière': 6, u'extraprofessionnelle': 4, u'auto-adaptative': 7, u'aéroportée': 0, u'désertifiante': 0, u'piocheuse': 2, u'contrevenante': 0, u'séduisante': 0, u'blasphématrice': 1, u'rouergate': 0, u'érubescente': 0, u'rurale': 3, u'sabine': 0, u'répugnante': 0, u'metteuse': 2, u'hypersensitive': 7, u'améliorante': 0, u'rétienne': 4, u'grandelette': 4, u'auto-adhésive': 7, u'vaniteuse': 9, u'graffeuse': 2, u'sertisseuse': 2, u'anticléricale': 3, u'déséquilibrante': 0, u'bahreïnie': 0, u'formelle': 4, u'fondeuse': 2, u'démonteuse': 2, u'forcenée': 0, u'patrilocale': 3, u'seuillée': 0, u'hyperfine': 0, u'convergente': 0, u'astringente': 0, u'chainière': 6, u'ansée': 0, u'corallienne': 4, u'adlérienne': 4, u'diphasée': 0, u'scriptrice': 1, u'exclusive': 7, u'charançonnée': 0, u'dialecticienne': 4, u'fusionnante': 0, u'bandante': 0, u'anticommutative': 7, u'dérivante': 0, u'régatière': 6, u'survirante': 0, u'grignarde': 0, u'médicinale': 3, u'pelvienne': 4, u'paumière': 6, u'couillonne': 4, u'méfiante': 0, u'dissociative': 7, u'dionysienne': 4, u'girondine': 0, u'débourgeoisée': 0, u'paraxiale': 3, u'jouissive': 7, u'démerdarde': 0, u'dirimante': 0, u'teneuse': 2, u'imperforée': 0, u'sous-qualifiée': 0, u'imperfective': 7, u'prescriptrice': 1, u'pétéchiale': 3, u'philippine': 0, u'audoise': 0, u'spinale': 3, u'adoucissante': 0, u'inépuisée': 0, u'dispensatrice': 1, u'érythréenne': 4, u'destructrice': 1, u'quiescente': 0, u'mastoïdienne': 4, u'victorienne': 4, u'subrogative': 7, u'amie': 0, u'itérative': 7, u'marshallaise': 0, u'telle': 4, u'forte': 0, u'subaiguë': 0, u'corégente': 0, u'olivacée': 0, u'funérale': 3, u'cicéronienne': 4, u'affluente': 0, u'périlleuse': 9, u'audioorale': 3, u'putassière': 6, u'legendrienne': 4, u'cantinière': 6, u'codominante': 0, u'diluviale': 3, u'présentielle': 4, u'extrémale': 3, u'germanopratine': 0, u'botswanaise': 0, u'cérumineuse': 9, u'palissonneuse': 2, u'persiffleuse': 2, u'insinuative': 7, u'prosociale': 3, u'mornée': 0, u'rimbaldienne': 4, u'crissante': 0, u'féminisante': 0, u'babylonienne': 4, u'couperosée': 0, u'affriolante': 0, u'démentielle': 4, u'torrentueuse': 9, u'braseuse': 2, u'villeuse': 9, u'affamée': 0, u'fameuse': 9, u'fertilisante': 0, u'saucissonneuse': 2, u'éducatrice': 1, u'bougonne': 4, u'semi-chenillée': 0, u'rouette': 4, u'portoricaine': 0, u'antiréflexive': 7, u'athénienne': 4, u'arborisée': 0, u'parodontale': 3, u'dauphine': 0, u'bastiaise': 0, u'ambiante': 0, u'jaugeuse': 2, u'fâcheuse': 9, u'somnolente': 0, u'toupilleuse': 2, u'roumie': 0, u'inavouée': 0, u'bichonneuse': 2, u'invaincue': 0, u'bikeuse': 2, u'appréciative': 7, u'auscultatrice': 1, u'cireuse': 9, u'sensorielle': 4, u'charpentière': 6, u'saigneuse': 9, u'réfringente': 0, u'impressive': 7, u'réprouvée': 0, u'coadministratrice': 1, u'lycéenne': 4, u'valdoisienne': 4, u'bivitelline': 0, u'lieuse': 2, u'fermante': 0, u'inabritée': 0, u'suffisante': 0, u'pontificale': 3, u'percussive': 7, u'éburnée': 0, u'gersoise': 0, u'fuguée': 0, u'oléoprotéagineuse': 9, u'bayésienne': 4, u'résurrectionnelle': 4, u'tumultueuse': 9, u'baptismale': 3, u'nuncupative': 7, u'mathématicienne': 4, u'tavernière': 6, u'héliograveuse': 2, u'infinie': 0, u'malthusienne': 4, u'franco-brésilienne': 4, u'nomenclatrice': 1, u'trafiquante': 0, u'malvenante': 0, u'insurgée': 0, u'réorganisatrice': 1, u'paneuropéenne': 4, u'perdue': 0, u'avalante': 0, u'afro-cubaine': 0, u'broyeuse': 2, u'écornifleuse': 2, u'simonienne': 4, u'sibérienne': 4, u'civilisatrice': 1, u'résonatrice': 1, u'surbookée': 0, u'cyclopéenne': 4, u'sympodiale': 3, u'coéternelle': 4, u'séquestrante': 0, u'loqueteuse': 9, u'notariale': 3, u'programmeuse': 2, u'boitillante': 0, u'gémissante': 0, u'atterrante': 0, u'tourbillonnante': 0, u'contenante': 0, u'valentine': 0, u'forgeronne': 4, u'diédrale': 3, u'mate': 0, u'matrilocale': 3, u'omnipuissante': 0, u'efflorescente': 0, u'permissive': 7, u'saccarine': 0, u'gastroœsophagienne': 4, u'univitelline': 0, u'rongeuse': 2, u'danoise': 0, u'solliciteuse': 2, u'fixative': 7, u'euryhaline': 0, u'argenteuse': 2, u'gouvernementale': 3, u'glomérulée': 0, u'komsomole': 0, u'récursive': 7, u'dysendocrinienne': 4, u'épagneule': 0, u'flosculeuse': 9, u'avenue': 0, u'bluette': 4, u'triboluminescente': 0, u'piailleuse': 2, u'gobeuse': 2, u'manuéline': 0, u'cristallinienne': 4, u'adulatrice': 1, u'champenoise': 0, u'vasculeuse': 9, u'sauvée': 0, u'purificatrice': 1, u'inélégante': 0, u'récupératrice': 1, u'hydrothermale': 3, u'tailleuse': 2, u'raboteuse': 9, u'austro-française': 0, u'raquettière': 6, u'colmarienne': 4, u'dégivrante': 0, u'saunante': 0, u'cauchemardeuse': 9, u'lombarde': 0, u'natale': 0, u'néolatine': 0, u'vilipendeuse': 2, u'importatrice': 1, u'prétentiarde': 0, u'différenciatrice': 1, u'concertante': 0, u'générationnelle': 4, u'purifiante': 0, u'prompte': 0, u'ronchonne': 4, u'carthaginoise': 0, u'prêteuse': 2, u'rhino-pharyngée': 0, u'gesticulée': 0, u'pelletière': 6, u'bombante': 0, u'chanoinesse': 16, u'cédante': 0, u'aveyronnaise': 0, u'demi-fine': 0, u'déléguée': 0, u'fine': 0, u'naufrageuse': 2, u'perspective': 7, u'israélo-américaine': 0, u'mélodieuse': 9, u'vitupératrice': 1, u'céphalo-rachidienne': 4, u'perceuse': 2, u'mollette': 4, u'enlumineuse': 2, u'dévonienne': 4, u'intermittente': 0, u'désoxydante': 0, u'antitussive': 7, u'manœuvrière': 6, u'panaméricaine': 0, u'abottée': 0, u'sous-vireuse': 2, u'bucheronne': 4, u'éminente': 0, u'bêta-bloquante': 0, u'qatarie': 0, u'vice-première': 6, u'anorectale': 3, u'mugissante': 0, u'débilitante': 0, u'intracrânienne': 4, u'nécessiteuse': 9, u'faitière': 6, u'hacheuse': 2, u'songeuse': 2, u'capitalisante': 0, u'brunéienne': 4, u'flagellatrice': 1, u'nilpotente': 0, u'bâtonnière': 6, u'dénutrie': 0, u'diplômante': 0, u'vanisée': 0, u'suractivée': 0, u'objective': 7, u'dirigeante': 0, u'brumale': 3, u'matelote': 0, u'vibrante': 0, u'gélatineuse': 9, u'argentière': 6, u'santoméenne': 4, u'juxtaposante': 0, u'cogérante': 0, u'coupée': 0, u'transcutanée': 0, u'blêmissante': 0, u'attentionnée': 0, u'contrarotative': 7, u'pataugeuse': 2, u'handballeuse': 2, u'interzonale': 3, u'chouineuse': 2, u'rougeoyante': 0, u'hamiltonienne': 4, u'extorqueuse': 2, u'feutrière': 6, u'admise': 0, u'anticonjoncturelle': 4, u'obscure': 0, u'inter-institutionnelle': 4, u'isoclinale': 3, u'rigolarde': 0, u'interpelée': 0, u'contondante': 0, u'prédigérée': 0, u'veinarde': 0, u'pharyngée': 0, u'académicienne': 4, u'servante': 0, u'guillotinée': 0, u'aigrettée': 0, u'cérémonielle': 4, u'flatulente': 0, u'néohébridaise': 0, u'castrale': 3, u'approvisionneuse': 2, u'restante': 0, u'mandchoue': 0, u'sanglotante': 0, u'freinatrice': 1, u'maïorale': 3, u'dolente': 0, u'spéculatrice': 1, u'financière': 6, u'gaumaise': 0, u'poilue': 0, u'menue': 0, u'méditante': 0, u'conséquente': 0, u'visuelle': 4, u'lambine': 0, u'américano-néerlandaise': 0, u'déserte': 0, u'péquenotte': 4, u'gardeuse': 2, u'photoémettrice': 1, u'chevrotante': 0, u'écorcheuse': 2, u'braconnière': 6, u'dameuse': 2, u'fendante': 0, u'cressicultrice': 1, u'pendarde': 0, u'neurochirurgienne': 4, u'saint-cyrienne': 4, u'cicatricielle': 4, u'antivenimeuse': 9, u'déwattée': 0, u'neuro-endocrinienne': 4, u'malgracieuse': 9, u'altérative': 7, u'interlinguale': 3, u'notariée': 0, u'groenlandaise': 0, u'multiplicatrice': 1, u'homogénéisatrice': 1, u'penseuse': 2, u'gauloise': 0, u'assimilatrice': 1, u'périapicale': 3, u'ultramontaine': 0, u'mal-famée': 0, u'environnementale': 3, u'pertinente': 0, u'tuvaluane': 0, u'émotive': 7, u'imaginale': 3, u'cataméniale': 3, u'palmilobée': 0, u'envinée': 0, u'intéressée': 0, u'exhaustive': 7, u'homoxylée': 0, u'antidéflagrante': 0, u'vermineuse': 9, u'promotionnelle': 4, u'réfléchissante': 0, u'canalisatrice': 1, u'thaïe': 0, u'catalogueuse': 2, u'dédoubleuse': 2, u'curatrice': 1, u'faisane': 0, u'norvégienne': 4, u'palefrenière': 6, u'maricultrice': 1, u'chipeuse': 2, u'crasseuse': 9, u'allemande': 0, u'tricennale': 3, u'tibéto-birmane': 0, u'vicariante': 0, u'dévoreuse': 2, u'invoquée': 0, u'axonale': 3, u'béatifiante': 0, u'amarile': 0, u'magnétiseuse': 2, u'arithméticienne': 4, u'caractérisante': 0, u'cocue': 0, u'obséquieuse': 9, u'évocatrice': 1, u'cossue': 0, u'référente': 0, u'média-planneuse': 2, u'maxwellienne': 4, u'herseuse': 2, u'unipotente': 0, u'menstruelle': 4, u'mondaine': 0, u'mercière': 6, u'toussoteuse': 9, u'imbue': 0, u'cloutière': 6, u'poursuiteuse': 2, u'hircine': 0, u'mi-partie': 0, u'paperassière': 6, u'intervertébrale': 3, u'sainte': 0, u'cuiseuse': 2, u'dénotative': 7, u'engreneuse': 2, u'biscottière': 6, u'melliflue': 0, u'intemporelle': 4, u'divinatrice': 1, u'étireuse': 2, u'subjonctive': 7, u'cagoularde': 0, u'cumulative': 7, u'bouquetière': 6, u'retourneuse': 2, u'concourante': 0, u'indirecte': 0, u'occipitale': 3, u'ragréeuse': 2, u'méticuleuse': 9, u'antigouvernementale': 3, u'routinière': 6, u'paysanne': 4, u'civile': 0, u'médicosportive': 7, u'hyoïdienne': 4, u'seyante': 0, u'érectrice': 1, u'bibliotechnicienne': 4, u'niaise': 0, u'kiosquière': 6, u'vue': 0, u'chevaucheuse': 2, u'claire': 0, u'électricienne': 4, u'unguéale': 3, u'lorgneuse': 2, u'dateuse': 2, u'gammée': 0, u'profileuse': 2, u'icarienne': 4, u'intersectionnelle': 4, u'étouffeuse': 2, u'légiférée': 0, u'rôdeuse': 2, u'ferro-magnésienne': 4, u'procréative': 7, u'chevauchante': 0, u'pourfendeuse': 2, u'bradeuse': 2, u'récriminée': 0, u'calomniée': 0, u'relieuse': 2, u'sculpturale': 3, u'lainière': 6, u'dispose': 0, u'claquante': 0, u'apprivoiseuse': 2, u'flexueuse': 9, u'égéenne': 4, u'extinctrice': 1, u'acquittée': 0, u'censuelle': 4, u'déflagrante': 0, u'officieuse': 9, u'anale': 3, u'bambocheuse': 2, u'uncinée': 0, u'endoctrinée': 0, u'rhodiée': 0, u'auto-équilibrante': 0, u'grognarde': 0, u'vibrationnelle': 4, u'dévolutive': 7, u'fumée': 0, u'taquine': 0, u'driveuse': 2, u'recarburée': 0, u'oppositionnelle': 4, u'crawleuse': 2, u'simpliciale': 3, u'triathlonienne': 4, u'incidente': 0, u'cooccurrente': 0, u'épiscopale': 3, u'susdénommée': 0, u'anglo-égyptienne': 4, u'sauveteuse': 2, u'écocitoyenne': 4, u'folioteuse': 2, u'libournaise': 0, u'amochée': 0, u'végétalienne': 4, u'zonière': 6, u'géotechnicienne': 4, u'clavière': 6, u'haschischée': 0, u'castratrice': 1, u'dineuse': 2, u'bourrelière': 6, u'boueuse': 9, u'sous-occipitale': 3, u'singulative': 7, u'biomédicale': 3, u'avantagée': 0, u'caravanière': 6, u'crémée': 0, u'doreuse': 2, u'immunotolérante': 0, u'aidante': 0, u'extractrice': 1, u'courbatue': 0, u'congénitale': 3, u'urticante': 0, u'carbonylée': 0, u'stationnée': 0, u'robinétière': 6, u'scarieuse': 9, u'miellée': 0, u'présentatrice': 1, u'bactérienne': 4, u'vitrioleuse': 2, u'striduleuse': 9, u'opérante': 0, u'professeure': 0, u'booléenne': 4, u'transgénérationnelle': 4, u'illuminative': 7, u'innominée': 0, u'inéprouvée': 0, u'confédérative': 7, u'eurodéputée': 0, u'grilladine': 0, u'incertaine': 0, u'magnanière': 6, u'délibérante': 0, u'charognarde': 0, u'enchainée': 0, u'chagrinante': 0, u'cambodgienne': 4, u'juive': 7, u'catalane': 0, u'assimilante': 0, u'néo-natale': 0, u'skieuse': 2, u'asexuelle': 4, u'infiltrée': 0, u'grammairienne': 4, u'embusquée': 0, u'absidiale': 3, u'patriotarde': 0, u'polarisante': 0, u'cartilagineuse': 9, u'gênante': 0, u'bretteuse': 2, u'retreinte': 0, u'agaillardie': 0, u'invocatrice': 1, u'bangladaise': 0, u'bienpensante': 0, u'insolente': 0, u'fœtale': 3, u'paroxysmale': 3, u'empileuse': 2, u'préposée': 0, u'portante': 0, u'rameuse': 9, u'insomnieuse': 9, u'enfoirée': 0, u'tuniquée': 0, u'soudanaise': 0, u'accédée': 0, u'temporale': 3, u'cognitive': 7, u'innée': 0, u'ininterrompue': 0, u'blafarde': 0, u'limonière': 6, u'miroitière': 6, u'irréligieuse': 9, u'prooccidentale': 3, u'végétarienne': 4, u'impondérée': 0, u'chanceuse': 9, u'garnisseuse': 2, u'incitante': 0, u'sinusale': 3, u'préardennaise': 0, u'brunisseuse': 2, u'américano-néo-zélandaise': 0, u'consultante': 0, u'biréfringente': 0, u'prédominante': 0, u'péronière': 6, u'brailleuse': 2, u'collectrice': 1, u'chiropraticienne': 4, u'calomniatrice': 1, u'aurorale': 3, u'musicale': 3, u'certaine': 0, u'écrivailleuse': 2, u'géospatiale': 3, u'soufflante': 0, u'accoupleuse': 2, u'minorative': 7, u'conne': 4, u'puissante': 0, u'démineuse': 2, u'odieuse': 9, u'odorante': 0, u'sinapisée': 0, u'déformante': 0, u'tibétaine': 0, u'fractale': 0, u'pourrissante': 0, u'législative': 7, u'affileuse': 2, u'réfrigérante': 0, u'essonnienne': 4, u'judéo-espagnole': 0, u'ambrosienne': 4, u'sino-japonaise': 0, u'orientale': 3, u'éolienne': 4, u'illuminatrice': 1, u'inadaptée': 0, u'successorale': 3, u'gallinacée': 0, u'américano-irlandaise': 0, u'rentrante': 0, u'édifiante': 0, u'exorciseuse': 2, u'toarcienne': 4, u'inséminatrice': 1, u'néoromane': 0, u'grattante': 0, u'estimative': 7, u'gazeuse': 9, u'archaïsante': 0, u'dominatrice': 1, u'copartageante': 0, u'sourcilleuse': 9, u'trophomicrobienne': 4, u'polarisatrice': 1, u'radioactive': 7, u'bâloise': 0, u'amante': 0, u'condamnée': 0, u'dépassante': 0, u'contrôleuse': 2, u'inapparente': 0, u'exhortative': 7, u'darwinienne': 4, u'phénoménale': 3, u'truitée': 0, u'droitière': 6, u'bifocale': 3, u'technicienne': 4, u'tricheuse': 2, u'arachnéenne': 4, u'avorteuse': 2, u'ourleuse': 2, u'neuroendocrinienne': 4, u'rageante': 0, u'relayeuse': 2, u'bossue': 0, u'guillocheuse': 2, u'intra-utérine': 0, u'chineuse': 2, u'alizéenne': 4, u'mexicaine': 0, u'magnétomotrice': 1, u'fermière': 6, u'systématicienne': 4, u'malaisée': 0, u'tonnante': 0, u'plumeuse': 9, u'suzeraine': 0, u'anglo-canadienne': 4, u'contractante': 0, u'prédatée': 0, u'donatrice': 1, u'écuyère': 6, u'boulinière': 6, u'italo-africaine': 0, u'bricoleuse': 2, u'orageuse': 9, u'barisienne': 4, u'querelleuse': 2, u'professante': 0, u'pénienne': 4, u'fricasseuse': 2, u'amylacée': 0, u'coépouse': 9, u'camarguaise': 0, u'ondine': 0, u'tresseuse': 2, u'trochantérienne': 4, u'dégénérescente': 0, u'caféière': 6, u'transmetteuse': 2, u'drainante': 0, u'rectrice': 1, u'consortiale': 3, u'turonienne': 4, u'modale': 3, u'priante': 0, u'percutanée': 0, u'mystificatrice': 1, u'étiqueteuse': 2, u'monologueuse': 2, u'vaudoise': 0, u'grand-ducale': 3, u'chiridienne': 4, u'liégeuse': 9, u'proeuropéenne': 4, u'méprisée': 0, u'releveuse': 2, u'tracassière': 6, u'fricative': 7, u'modifiante': 0, u'anti-infectieuse': 9, u'ultra-confidentielle': 4, u'antivirale': 3, u'psychosomaticienne': 4, u'bourguignonne': 4, u'dompteuse': 2, u'chère': 6, u'américano-européenne': 4, u'fiancée': 0, u'égrotante': 0, u'éclateuse': 2, u'délocalisante': 0, u'soudaine': 0, u'paulienne': 4, u'applaudisseuse': 2, u'soigneuse': 9, u'cavaleuse': 2, u'percheuse': 2, u'soupirante': 0, u'quilleuse': 2, u'colzatière': 6, u'ariégeoise': 0, u'transcaucasienne': 4, u'antisportive': 7, u'inexpressive': 7, u'grandissante': 0, u'moyen-orientale': 3, u'abornée': 0, u'brute': 0, u'coulissante': 0, u'exultante': 0, u'minaudière': 6, u'pasteurisatrice': 1, u'dictatoriale': 3, u'minimisante': 0, u'émulsive': 7, u'rhinopharyngée': 0, u'swahélie': 0, u'coréenne': 4, u'péléenne': 4, u'saleuse': 2, u'massacrante': 0, u'acéteuse': 9, u'traductrice': 1, u'sableuse': 9, u'psychophysicienne': 4, u'alcalifiante': 0, u'intéressante': 0, u'flingueuse': 2, u'définitive': 7, u'microporeuse': 9, u'demi-entière': 6, u'épineuse': 9, u'nutritive': 7, u'éplorée': 0, u'nonpareille': 4, u'archéenne': 4, u'autocollante': 0, u'hawaïenne': 4, u'alpine': 0, u'représentative': 7, u'nouménale': 3, u'russienne': 4, u'céphalorachidienne': 4, u'exclamative': 7, u'primesautière': 6, u'africaine': 0, u'barboteuse': 2, u'poitevine': 0, u'pratiquante': 0, u'hyperémotive': 7, u'malséante': 0, u'avouée': 0, u'abusive': 7, u'capverdienne': 4, u'bodybuildeuse': 2, u'pédicellée': 0, u'localisatrice': 1, u'unidirectionnelle': 4, u'signifiante': 0, u'bénigne': 24, u'négrière': 6, u'préconisatrice': 1, u'pluriannuelle': 4, u'inadéquate': 0, u'tranchante': 0, u'monitrice': 1, u'contrefaisante': 0, u'rousse': 26, u'ascensionnelle': 4, u'chambreuse': 2, u'meublante': 0, u'contrite': 0, u'fournière': 6, u'buronnière': 6, u'postérieure': 0, u'pulsionnelle': 4, u'dernière': 6, u'insurgente': 0, u'keynésienne': 4, u'mesmérienne': 4, u'expropriatrice': 1, u'scabieuse': 9, u'bénisseuse': 2, u'resquilleuse': 2, u'éditoriale': 3, u'fibrineuse': 9, u'vaccinatrice': 1, u'tullière': 6, u'absorptive': 7, u'super-légère': 6, u'amortisseuse': 2, u'limousine': 0, u'vicelarde': 0, u'indulgente': 0, u'monopolisante': 0, u'subtile': 0, u'isarienne': 4, u'pendante': 0, u'paralysée': 0, u'électrice': 1, u'surrénalienne': 4, u'cæcale': 3, u'maquilleuse': 2, u'surirradiée': 0, u'indéterminée': 0, u'psychosexuelle': 4, u'foliée': 0, u'inarticulée': 0, u'balzacienne': 4, u'antinazie': 0, u'rancunière': 6, u'alcoolorésistante': 0, u'australo-française': 0, u'ennuyante': 0, u'dragueuse': 2, u'maternelle': 4, u'presbytérienne': 4, u'capillotractée': 0, u'salinière': 6, u'racheteuse': 2, u'transfusionnelle': 4, u'carcinomateuse': 9, u'plâtrière': 6, u'contemplative': 7, u'normative': 7, u'moribonde': 0, u'monumentale': 3, u'pondéreuse': 9, u'préromane': 0, u'éparse': 0, u'pistonnée': 0, u'dialogale': 3, u'boycotteuse': 2, u'cannoise': 0, u'hardeuse': 2, u'fécale': 3, u'réacquise': 0, u'ascendante': 0, u'turbocompressée': 0, u'fourragère': 6, u'boucholeuse': 2, u'brouillassée': 0, u'fragilisante': 0, u'ostrogothe': 0, u'parkinsonienne': 4, u'coauteure': 0, u'pupitreuse': 2, u'multinomiale': 3, u'freineuse': 2, u'rase': 0, u'cagneuse': 9, u'sectorielle': 4, u'chronométreuse': 2, u'zéphyrienne': 4, u'apaisante': 0, u'fluette': 4, u'coracoïdienne': 4, u'aphteuse': 9, u'non-engagée': 0, u'critiqueuse': 2, u'canoniale': 3, u'demandeuse': 2, u'velue': 0, u'camouflante': 0, u'calabraise': 0, u'estimatrice': 1, u'zurichoise': 0, u'plane': 0, u'billetée': 0, u'biparentale': 3, u'congolaise': 0, u'infréquentée': 0, u'factitive': 7, u'aqueuse': 9, u'loubarde': 0, u'pinéale': 3, u'ménagère': 6, u'emballante': 0, u'rébarbative': 7, u'zende': 0, u'taïwanaise': 0, u'patoisante': 0, u'pétrolière': 6, u'mariée': 0, u'excitatrice': 1, u'subaérienne': 4, u'non-euclidienne': 4, u'indépassée': 0, u'distincte': 0, u'océanienne': 4, u'toulonnaise': 0, u'corrézienne': 4, u'membraneuse': 9, u'huronne': 4, u'épandeuse': 2, u'japonaise': 0, u'rhabdomancienne': 4, u'antibiorésistante': 0, u'insatisfaisante': 0, u'daronne': 4, u'légumière': 6, u'sicilienne': 4, u'dimensionnelle': 4, u'noduleuse': 9, u'piaffeuse': 2, u'cantonnière': 6, u'vésiculeuse': 9, u'commissurale': 3, u'permanencière': 6, u'pseudoriemannienne': 4, u'papesse': 16, u'atemporelle': 4, u'luthière': 6, u'répartitrice': 1, u'opposante': 0, u'racée': 0, u'basselissière': 6, u'compagnonne': 4, u'régénérante': 0, u'diffluente': 0, u'saducéenne': 4, u'pompéienne': 4, u'prédiffusée': 0, u'dessinatrice': 1, u'mercurielle': 4, u'télé-actrice': 1, u'institutrice': 1, u'civilisationnelle': 4, u'banqueteuse': 2, u'quadriennale': 3, u'remplisseuse': 2, u'politico-sociale': 3, u'grutière': 6, u'supportrice': 1, u'ancienne': 4, u'cartusienne': 4, u'drômoise': 0, u'souffrante': 0, u'fesseuse': 2, u'catégoriale': 0, u'kirghize': 0, u'cléchée': 0, u'unisexuelle': 4, u'bucco-génitale': 3, u'renarde': 0, u'rythmicienne': 4, u'mésenchymateuse': 9, u'écumeuse': 9, u'payeuse': 2, u'débourreuse': 2, u'intransigeante': 0, u'reconstructrice': 1, u'siennoise': 0, u'continuelle': 4, u'sarmenteuse': 9, u'dentée': 0, u'levageuse': 2, u'inhabituelle': 4, u'laide': 0, u'originale': 3, u'bio-industrielle': 4, u'tabulatrice': 1, u'franco-mexicaine': 0, u'banlieusarde': 0, u'frauduleuse': 9, u'interrogatrice': 1, u'bêcheuse': 2, u'temporelle': 4, u'réitérante': 0, u'combative': 7, u'spectatrice': 1, u'suspicieuse': 9, u'heideggerienne': 4, u'persécutante': 0, u'dispendieuse': 9, u'moissonneuse': 2, u'microlocale': 3, u'saline': 0, u'mahoraise': 0, u'harponneuse': 2, u'rimeuse': 2, u'inconstante': 0, u'clémente': 0, u'vergée': 0, u'fabricante': 0, u'fourvoyante': 0, u'cordelière': 6, u'extra-conjugale': 3, u'carotidienne': 4, u'suborbitale': 3, u'discontinue': 0, u'excessive': 7, u'intracontinentale': 3, u'infante': 0, u'tutorielle': 4, u'confraternelle': 4, u'compulsive': 7, u'argumentatrice': 1, u'amasseuse': 2, u'assiégeante': 0, u'suicidée': 0, u'relevante': 0, u'lavandière': 6, u'jupitérienne': 4, u'tunisienne': 4, u'franco-islandaise': 0, u'carotteuse': 2, u'résomptive': 7, u'antimusulmane': 0, u'crêpière': 6, u'jamaïcaine': 0, u'impatiente': 0, u'fatigante': 0, u'détersive': 7, u'vigésimale': 3, u'sablière': 6, u'orthogonale': 3, u'bourreuse': 2, u'repoussante': 0, u'terreuse': 9, u'préretraitée': 0, u'chambrière': 6, u'bustière': 6, u'godichonne': 4, u'mésadaptée': 0, u'jargonnante': 0, u'fracassante': 0, u'traumatisée': 0, u'infirmative': 7, u'buffetière': 6, u'shogounale': 3, u'élective': 7, u'lunettière': 6, u'douce': 9, u'préfrontale': 3, u'albuminée': 0, u'destructive': 7, u'laciniée': 0, u'politico-financière': 6, u'sphénoïdale': 3, u'barbelée': 0, u'mortifiante': 0, u'magasinière': 6, u'zénithale': 3, u'saoule': 0, u'rennaise': 0, u'procédurière': 6, u'beauvaisienne': 4, u'digitale': 3, u'contraventionnelle': 4, u'licencieuse': 9, u'réticuloendothéliale': 3, u'censée': 0, u'rageuse': 2, u'carminative': 7, u'applicatrice': 1, u'discursive': 7, u'chômeuse': 2, u'mesquine': 0, u'doctorale': 3, u'mulâtresse': 16, u'tridentine': 0, u'intimidante': 0, u'post-doctorante': 0, u'régionale': 3, u'iléo-cæcale': 3, u'infondée': 0, u'raveuse': 2, u'moralisatrice': 1, u'réductive': 7, u'bovine': 0, u'fêtarde': 0, u'mauritanienne': 4, u'graveuse': 2, u'verveuse': 9, u'lingère': 6, u'franco-bolivienne': 4, u'aptienne': 4, u'salée': 0, u'coliqueuse': 9, u'néonazie': 0, u'immatérielle': 4, u'éteigneuse': 2, u'croulante': 0, u'glapissante': 0, u'ambitieuse': 9, u'prescriptive': 7, u'paulinienne': 4, u'flagorneuse': 2, u'émailleuse': 2, u'décrypteuse': 2, u'pleurale': 3, u'néogrecque': 29, u'grésillante': 0, u'étripée': 0, u'vergetée': 0, u'éblouissante': 0, u'louviéroise': 0, u'figurante': 0, u'rieuse': 2, u'barbante': 0, u'vocifératrice': 1, u'roguée': 0, u'pharmacienne': 4, u'intrusive': 7, u'sacro-sainte': 0, u'merveilleuse': 9, u'bahaïe': 0, u'sommeillante': 0, u'mulassière': 6, u'délassante': 0, u'quincaillère': 6, u'prurigineuse': 9, u'venimeuse': 9, u'accordée': 0, u'garçonne': 4, u'sablonneuse': 9, u'piquée': 0, u'guérisseuse': 2, u'imprimitive': 7, u'ultraplate': 0, u'diffusive': 7, u'chapelière': 6, u'ondée': 0, u'télévisuelle': 4, u'lunée': 0, u'oscillante': 0, u'enquêteuse': 2, u'managériale': 3, u'pro-européenne': 4, u'pustuleuse': 9, u'postnatale': 0, u'ahurissante': 0, u'sino-australienne': 4, u'citernière': 6, u'polygonale': 3, u'burineuse': 2, u'joyeuse': 9, u'noueuse': 9, u'blonde': 0, u'suppurative': 7, u'augurale': 3, u'laveuse': 2, u'prédictive': 7, u'planétésimale': 3, u'quadripartie': 0, u'autoexcitatrice': 1, u'rhéto-romane': 0, u'imméritée': 0, u'féminine': 0, u'franco-québecoise': 0, u'franco-luxembourgeoise': 0, u'ventée': 0, u'païenne': 4, u'druidesse': 16, u'ouralienne': 4, u'magicienne': 4, u'vexatrice': 1, u'informative': 7, u'affriquée': 0, u'sédative': 7, u'gonzesse': 16, u'coéquipière': 6, u'décapante': 0, u'cristallière': 6, u'extrême-orientale': 3, u'nivale': 3, u'bitumineuse': 9, u'stratificationnelle': 4, u'arpenteuse': 2, u'géocacheuse': 2, u'gouteuse': 9, u'cellophanée': 0, u'rafraichie': 0, u'autoévaluante': 0, u'médusante': 0, u'manouvrière': 6, u'bridgeuse': 2, u'frustrée': 0, u'semi-finie': 0, u'pétulante': 0, u'réviseuse': 2, u'relanceuse': 2, u'soûlarde': 0, u'franco-japonaise': 0, u'instructive': 7, u'mecquoise': 0, u'béate': 0, u'brève': 23, u'internée': 0, u'lessiveuse': 2, u'recéleuse': 2, u'reminéralisante': 0, u'husserlienne': 4, u'rubanière': 6, u'équipollente': 0, u'pluvionivale': 3, u'chapitrale': 3, u'pompeuse': 9, u'langoustière': 6, u'entérorénale': 3, u'pelucheuse': 9, u'thermorésistante': 0, u'démobilisatrice': 1, u'claudicante': 0, u'chausseuse': 2, u'bi-dimensionnelle': 4, u'faustienne': 4, u'lignagère': 6, u'caverneuse': 9, u'vulgarisatrice': 1, u'filtreuse': 2, u'matrimoniale': 3, u'rafraichissante': 0, u'focale': 3, u'contentieuse': 9, u'révolue': 0, u'crétacée': 0, u'libéro-ligneuse': 9, u'compétitrice': 1, u'vengeresse': 20, u'goûteuse': 9, u'furibonde': 0, u'fangeuse': 9, u'surdéterminée': 0, u'pédaleuse': 2, u'longuette': 4, u'chorale': 0, u'conseillère': 6, u'courrière': 6, u'concrète': 25, u'vétillarde': 0, u'bi-active': 7, u'gazouillante': 0, u'mauvaise': 0, u'moucharde': 0, u'terre-neuvienne': 4, u'corrective': 7, u'décussée': 0, u'uligineuse': 9, u'transandine': 0, u'tressaillante': 0, u'post-prandiale': 3, u'oiseleuse': 2, u'mutilante': 0, u'panarde': 0, u'batifoleuse': 2, u'covendeuse': 2, u'givrante': 0, u'stylisticienne': 4, u'sommitale': 3, u'séductrice': 1, u'rubéfiante': 0, u'lancéolée': 0, u'poussive': 7, u'ahurie': 0, u'électromécanicienne': 4, u'suceuse': 2, u'courante': 0, u'blagueuse': 2, u'noceuse': 2, u'égayante': 0, u'notionnelle': 4, u'pavillonnée': 0, u'rabelaisienne': 4, u'machinatrice': 1, u'flegmoneuse': 9, u'entraineuse': 2, u'falote': 0, u'suppurée': 0, u'pseudo-riemannienne': 4, u'hexagonale': 3, u'pestilente': 0, u'hurlante': 0, u'trinervée': 0, u'goménolée': 0, u'rituelle': 4, u'laïusseuse': 2, u'niveleuse': 2, u'subordonnante': 0, u'narquoise': 0, u'amorale': 3, u'paraphraseuse': 2, u'scabreuse': 9, u'enrageante': 0, u'inactuelle': 4, u'unificatrice': 1, u'sri-lankaise': 0, u'réunionnaise': 0, u'vaine': 0, u'inverseuse': 2, u'chaîneuse': 2, u'ultracourte': 0, u'posturale': 3, u'nilo-saharienne': 4, u'trépidante': 0, u'sublinguale': 3, u'mistonne': 4, u'caissonnée': 0, u'albumineuse': 9, u'essayeuse': 2, u'rastafarienne': 4, u'consolante': 0, u'déshonorante': 0, u'aiguiseuse': 2, u'exploiteuse': 2, u'remontante': 0, u'dévolue': 0, u'contreproductive': 7, u'variétale': 3, u'rabatteuse': 2, u'inaltérée': 0, u'flabellée': 0, u'pontifiante': 0, u'radio-amatrice': 1, u'multiplicative': 7, u'anglo-portugaise': 0, u'mitochondriale': 3, u'traumatisante': 0, u'rappée': 0, u'plumassière': 6, u'shamane': 0, u'éradicatrice': 1, u'antéro-ventrale': 3, u'glaiseuse': 9, u'non-croyante': 0, u'arrimeuse': 2, u'montagneuse': 9, u'cardeuse': 2, u'ladine': 0, u'surpeuplée': 0, u'bisexuée': 0, u'intramodale': 3, u'confusionnelle': 4, u'désobéissante': 0, u'copieuse': 9, u'conciliante': 0, u'piaillarde': 0, u'passementière': 6, u'lustreuse': 2, u'moyenâgeuse': 9, u'thésauriseuse': 2, u'ambleuse': 2, u'fascée': 0, u'maraîchine': 0, u'méridionale': 3, u'gratuite': 0, u'amareyeuse': 2, u'interculturelle': 4, u'glaucomateuse': 9, u'rémunératrice': 1, u'semoulière': 6, u'multicatégorielle': 4, u'éphésienne': 4, u'galloise': 0, u'gazée': 0, u'taraudeuse': 2, u'malodorante': 0, u'uninominale': 3, u'irritée': 0, u'colporteuse': 2, u'ogivale': 3, u'irako-syrienne': 4, u'printanière': 6, u'gratifiante': 0, u'correspondancière': 6, u'vacante': 0, u'exiguë': 0, u'bidonnante': 0, u'russo-allemande': 0, u'cranio-faciale': 3, u'réplétive': 7, u'phlegmoneuse': 9, u'gigoteuse': 2, u'nutricière': 6, u'méliorative': 7, u'pleine': 0, u'fériale': 3, u'agglomérante': 0, u'photocompositrice': 1, u'imprécatrice': 1, u'cisailleuse': 2, u'canoteuse': 2, u'cervicale': 3, u'membrée': 0, u'damasquineuse': 2, u'hongkongaise': 0, u'intraveineuse': 9, u'douteuse': 9, u'adjutrice': 1, u'stridente': 0, u'codemanderesse': 20, u'partageuse': 9, u'galvaniseuse': 2, u'sporulée': 0, u'peléenne': 4, u'exinscrite': 0, u'farceuse': 2, u'codébitrice': 1, u'fluente': 0, u'violoneuse': 9, u'désobligeante': 0, u'collaboratrice': 1, u'ferrugineuse': 9, u'minotière': 6, u'mélanésienne': 4, u'chiante': 0, u'acoquinante': 0, u'quadragésimale': 3, u'serbo-monténégrine': 0, u'caractérielle': 4, u'bruxelloise': 0, u'autrichienne': 4, u'maîtresse': 16, u'dénaturante': 0, u'littorale': 3, u'crochue': 0, u'génératrice': 1, u'unitive': 7, u'manifestante': 0, u'rétroéclairée': 0, u'synclinale': 3, u'biostatisticienne': 4, u'décimale': 3, u'préparatrice': 1, u'bijoutière': 6, u'murissante': 0, u'matonne': 4, u'inespérée': 0, u'pensante': 0, u'inventrice': 1, u'perceptrice': 1, u'essénienne': 4, u'installatrice': 1, u'envieuse': 9, u'israélo-palestinienne': 4, u'panière': 6, u'négrillonne': 4, u'tutoyeuse': 2, u'sénatorienne': 4, u'laryngée': 0, u'ornaise': 0, u'dépensière': 6, u'aguicheuse': 2, u'effaceuse': 2, u'vasodilatatrice': 1, u'résolvante': 0, u'linière': 6, u'rêvasseuse': 2, u'ressuscitée': 0, u'gibbeuse': 9, u'viticultrice': 1, u'efférente': 0, u'trancheuse': 2, u'détorse': 0, u'soulaude': 0, u'synodale': 3, u'trekkeuse': 2, u'suppliciante': 0, u'patrouilleuse': 2, u'mobilière': 6, u'hypothético-déductive': 7, u'effervescente': 0, u'califale': 3, u'harassante': 0, u'collatérale': 3, u'annale': 3, u'gueulante': 0, u'accompagnante': 0, u'ordinale': 3, u'butinante': 0, u'hôtelière': 6, u'descendante': 0, u'haineuse': 9, u'socioaffective': 7, u'ligneuse': 9, u'ombrienne': 4, u'lasse': 4, u'transgressive': 7, u'fainéante': 0, u'prétentieuse': 9, u'blondine': 0, u'contigüe': 12, u'casinotière': 6, u'justicière': 6, u'tricarde': 0, u'artisane': 0, u'escrimeuse': 2, u'hyposodée': 0, u'corailleuse': 2, u'répondante': 0, u'inéduquée': 0, u'torturée': 0, u'fautrice': 1, u'bûcheuse': 2, u'épépineuse': 2, u'corrélationnelle': 4, u'épicurienne': 4, u'entarteuse': 2, u'tournante': 0, u'jetsetteuse': 2, u'licenciée': 0, u'markovienne': 4, u'enragée': 0, u'superprédatrice': 1, u'franco-équatorienne': 4, u'médico-sociale': 3, u'affirmative': 7, u'cancanière': 6, u'défectueuse': 9, u'compacteuse': 2, u'burundaise': 0, u'pimpante': 0, u'déferlante': 0, u'consternante': 0, u'offrante': 0, u'gambadeuse': 2, u'foraine': 0, u'évaluatrice': 1, u'onglée': 0, u'malawienne': 4, u'pédonculée': 0, u'procédurale': 3, u'valvée': 0, u'russo-suédoise': 0, u'subsistante': 0, u'aquacultrice': 1, u'maigriotte': 4, u'italo-néerlandaise': 0, u'coccygienne': 4, u'abuseuse': 2, u'défective': 7, u'procréatrice': 1, u'ronceuse': 9, u'avalancheuse': 9, u'guerrière': 6, u'fielleuse': 9, u'anversoise': 0, u'roucouleuse': 2, u'vénale': 3, u'andorrane': 0, u'instantanée': 0, u'municipale': 3, u'liquidative': 7, u'gaillarde': 0, u'exaspérante': 0, u'célibattante': 0, u'résinière': 6, u'confabulatrice': 1, u'malsaine': 0, u'bastillée': 0, u'fluocompacte': 0, u'lombricultrice': 1, u'berrichonne': 4, u'miséreuse': 9, u'froide': 0, u'périgourdine': 0, u'intempestive': 7, u'alvine': 0, u'argentine': 0, u'normée': 0, u'fugueuse': 2, u'moustérienne': 4, u'appétissante': 0, u'conjurée': 0, u'grainetière': 6, u'squameuse': 9, u'rabattante': 0, u'corroyeuse': 2, u'ourlienne': 4, u'freinante': 0, u'rejointoyeuse': 2, u'idéelle': 4, u'pénitentiale': 3, u'froufrouteuse': 9, u'laiteuse': 9, u'postcoloniale': 3, u'fructueuse': 9, u'changeuse': 2, u'ouvreuse': 2, u'convaincue': 0, u'alumineuse': 9, u'raisonneuse': 2, u'ignifugeante': 0, u'soulante': 0, u'ferromagnésienne': 4, u'infatuée': 0, u'régleuse': 2, u'oursonne': 4, u'fictive': 7, u'adsorbante': 0, u'analysée': 0, u'raideuse': 2, u'mi-close': 0, u'vichyssoise': 0, u'animatrice': 1, u'hennuyère': 6, u'escaliéteuse': 2, u'afflictive': 7, u'bilabiale': 3, u'dépeceuse': 2, u'trainarde': 0, u'palmiséquée': 0, u'répétitive': 7, u'tchatcheuse': 2, u'chroniqueuse': 2, u'botoxée': 0, u'afficheuse': 2, u'persifleuse': 2, u'paradoxale': 3, u'disubstituée': 0, u'susnommée': 0, u'sémioticienne': 4, u'maroquinière': 6, u'vitrière': 6, u'cabote': 0, u'gouacheuse': 2, u'anfractueuse': 9, u'sylvicultrice': 1, u'assaillie': 0, u'sûre': 0, u'plénière': 6, u'renvoyée': 0, u'jardineuse': 9, u'translative': 7, u'guerroyeuse': 2, u'incrustatrice': 1, u'triviale': 3, u'adiposogénitale': 3, u'renonciatrice': 1, u'chuchoteuse': 2, u'algonkine': 0, u'captatrice': 1, u'ardente': 0, u'abasourdissante': 0, u'fumante': 0, u'couverte': 0, u'tapissière': 6, u'nominatrice': 1, u'inexperte': 0, u'reluisante': 0, u'sorcière': 6, u'néorurale': 3, u'trisectrice': 1, u'innovatrice': 1, u'lorentzienne': 4, u'refoulée': 0, u'égale': 3, u'quarteronne': 4, u'pyramidée': 0, u'accusée': 0, u'automotrice': 1, u'donjonnée': 0, u'spumescente': 0, u'palliative': 7, u'marmiteuse': 9, u'népalaise': 0, u'artérioscléreuse': 9, u'transfrontalière': 6, u'ostrogote': 0, u'vice-rectrice': 1, u'traînarde': 0, u'future': 0, u'peinarde': 0, u'duelle': 4, u'confédératrice': 1, u'thanatopractrice': 1, u'dégénérative': 7, u'capricieuse': 9, u'rouilleuse': 9, u'mafflue': 0, u'brigantine': 0, u'phosphoreuse': 9, u'passionnée': 0, u'commutatrice': 1, u'douée': 0, u'égypto-américaine': 0, u'ultrarésistante': 0, u'quadridimensionnelle': 4, u'sémanticienne': 4, u'culpabilisante': 0, u'labio-dentale': 3, u'franco-argentine': 0, u'émanée': 0, u'insoumise': 0, u'tripolitaine': 0, u'alcyonienne': 4, u'faiseuse': 2, u'déserteuse': 2, u'infuse': 0, u'banqueroutière': 6, u'discriminatrice': 1, u'inentendue': 0, u'réflective': 7, u'gravitante': 0, u'phénicienne': 4, u'conormale': 3, u'néodarwinienne': 4, u'fignoleuse': 2, u'ineffective': 7, u'tréfileuse': 2, u'superflue': 0, u'abyssale': 3, u'urgente': 0, u'cingalaise': 0, u'interloquée': 0, u'ténébreuse': 9, u'dominicaine': 0, u'processuelle': 4, u'mucilagineuse': 9, u'discriminative': 7, u'alliacée': 0, u'nue': 0, u'trépignante': 0, u'déprimée': 0, u'observantine': 0, u'papetière': 6, u'brunissante': 0, u'mécheuse': 9, u'gladiatrice': 1, u'gainière': 6, u'neurochirurgicale': 3, u'spiritueuse': 9, u'vérificatrice': 1, u'aliénée': 0, u'bronzeuse': 2, u'précuite': 0, u'singapourienne': 4, u'causative': 7, u'sous-traitante': 0, u'inviolée': 0, u'syntacticienne': 4, u'politicienne': 4, u'tannante': 0, u'fonctionnelle': 4, u'signalisatrice': 1, u'nominée': 0, u'effectrice': 1, u'pestilentielle': 4, u'lascive': 7, u'vrillante': 0, u'sous-pubienne': 4, u'hurleuse': 2, u'sarcomateuse': 9, u'commençante': 0, u'commentatrice': 1, u'continue': 0, u'totalisante': 0, u'adimensionnelle': 4, u'poursuivante': 0, u'conciliatrice': 1, u'tôlière': 6, u'locomotrice': 1, u'gondolante': 0, u'vieillotte': 4, u'surplombante': 0, u'décélérée': 0, u'piémontaise': 0, u'béguine': 0, u'fendeuse': 2, u'inquisitrice': 1, u'ainée': 0, u'développementale': 3, u'configurationnelle': 4, u'pudibonde': 0, u'bouillante': 0, u'expérimentatrice': 1, u'superlative': 7, u'romancière': 6, u'virulente': 0, u'hachichée': 0, u'pectinée': 0, u'arabo-palestinienne': 4, u'scrofuleuse': 9, u'codificatrice': 1, u'enseignante': 0, u'matriarcale': 3, u'intervenante': 0, u'franco-irakienne': 4, u'anti-microbienne': 4, u'lysosomiale': 3, u'coloniale': 3, u'idéalisatrice': 1, u'antécédente': 0, u'trempeuse': 2, u'expectative': 7, u'hémorroïdale': 3, u'savoureuse': 9, u'béguë': 0, u'franchisée': 0, u'déverse': 0, u'introublée': 0, u'sermonneuse': 2, u'catcheuse': 2, u'industrieuse': 9, u'essentielle': 4, u'prébendée': 0, u'ségrégative': 7, u'approbative': 7, u'pistillée': 0, u'blasonneuse': 2, u'affectueuse': 9, u'bidimensionnelle': 4, u'rebondissante': 0, u'millerandée': 0, u'vitrificatrice': 1, u'parafiscale': 3, u'piscicultrice': 1, u'vicieuse': 9, u'fanatisante': 0, u'septentrionale': 3, u'globuleuse': 9, u'russo-japonaise': 0, u'rénitente': 0, u'aigrette': 4, u'dessiccative': 7, u'récidivante': 0, u'ovine': 0, u'aiglonne': 4, u'bio-indicatrice': 1, u'tournoyante': 0, u'broussarde': 0, u'grignoteuse': 2, u'néoformée': 0, u'bilieuse': 9, u'déstructurante': 0, u'centrale': 3, u'libérienne': 4, u'nord-américaine': 0, u'suspecte': 0, u'cafteuse': 2, u'abaissante': 0, u'dépanneuse': 2, u'cliquetante': 0, u'adnée': 0, u'cabochée': 0, u'piétinante': 0, u'oscillocoulissante': 0, u'pfaffienne': 4, u'blette': 4, u'fondue': 0, u'opérative': 7, u'ondulante': 0, u'balayeuse': 2, u'huguenote': 0, u'orofaciale': 3, u'accréditive': 7, u'improbative': 7, u'sahélienne': 4, u'coercitive': 7, u'prédatrice': 1, u'shampouineuse': 2, u'faubourienne': 4, u'impénitente': 0, u'périe': 0, u'caillouteuse': 9, u'scripte': 0, u'handicapante': 0, u'grondante': 0, u'dérangeante': 0, u'singulière': 6, u'électronicienne': 4, u'semi-développée': 0, u'gaie': 0, u'embryonnée': 0, u'impaire': 0, u'reloue': 0, u'margeuse': 2, u'repentante': 0, u'carottière': 6, u'antiaméricaine': 0, u'enregistreuse': 2, u'trichineuse': 9, u'attaquante': 0, u'expirante': 0, u'bronzante': 0, u'dinandière': 6, u'ionisante': 0, u'affixale': 3, u'veuve': 7, u'subéreuse': 9, u'guyanienne': 4, u'polytransfusée': 0, u'flottante': 0, u'commerçante': 0, u'vieillarde': 0, u'exemptée': 0, u'affineuse': 2, u'accusatrice': 1, u'fraternelle': 4, u'organomagnésienne': 4, u'lavandicultrice': 1, u'immobilière': 6, u'hellénisante': 0, u'solénoïdale': 3, u'cupro-ammoniacale': 3, u'neurosensorielle': 4, u'réelle': 4, u'labiée': 0, u'crépitante': 0, u'paridigitidée': 0, u'claustrale': 3, u'virtuelle': 4, u'rhumatisante': 0, u'cellérière': 6, u'spirituelle': 4, u'raciale': 3, u'riveraine': 0, u'relictuelle': 4, u'entêtante': 0, u'jacobienne': 4, u'sud-coréenne': 4, u'déchevelante': 0, u'présente': 0, u'coracobrachiale': 3, u'franco-italienne': 4, u'montagnarde': 0, u'composée': 0, u'entraînante': 0, u'décolleteuse': 2, u'extra-territoriale': 3, u'hongroyeuse': 2, u'génitrice': 1, u'fémorale': 3, u'bienaimée': 0, u'spécificative': 7, u'apprenante': 0, u'montoise': 0, u'équivalente': 0, u'malienne': 4, u'vitruvienne': 4, u'incantatrice': 1, u'communicative': 7, u'arsénieuse': 9, u'ligueuse': 2, u'réanimatrice': 1, u'photosensibilisante': 0, u'post-natale': 0, u'fraudeuse': 2, u'canotière': 6, u'persévérante': 0, u'exagérative': 7, u'cap-hornière': 6, u'injectrice': 1, u'forgeuse': 2, u'snobinarde': 0, u'bretonnante': 0, u'anticancéreuse': 9, u'houligane': 0, u'psychomotrice': 1, u'territoriale': 3, u'renégate': 0, u'providentielle': 4, u'pentagonale': 3, u'achéenne': 4, u'exciseuse': 2, u'diverse': 0, u'vice-présidente': 0, u'transhumante': 0, u'mal-baisée': 0, u'dotale': 3, u'rigoleuse': 2, u'distrayante': 0, u'afro-brésilienne': 4, u'agrarienne': 4, u'trifouilleuse': 2, u'appontée': 0, u'faïencée': 0, u'hébergeuse': 2, u'moussue': 0, u'explosée': 0, u'occurrente': 0, u'jambière': 6, u'manufacturière': 6, u'prospective': 7, u'milicienne': 4, u'rivale': 3, u'jobarde': 0, u'beuglante': 0, u'subaigüe': 12, u'guillotineuse': 2, u'désopilante': 0, u'parcimonieuse': 9, u'chiropractrice': 1, u'intégrale': 3, u'aryenne': 4, u'esthétisante': 0, u'gribouilleuse': 2, u'coquette': 4, u'séminale': 3, u'caressante': 0, u'rééducatrice': 1, u'globale': 3, u'arbustive': 7, u'nivernaise': 0, u'imprudente': 0, u'déductive': 7, u'entrepreneuriale': 3, u'ultramarine': 0, u'chéloïdienne': 4, u'myxœdémateuse': 9, u'routière': 6, u'faramineuse': 9, u'layetière': 6, u'cartésienne': 4, u'rhétoricienne': 4, u'revendicatrice': 1, u'preneuse': 2, u'détimbrée': 0, u'octogonale': 3, u'diluvienne': 4, u'bifurquée': 0, u'décagonale': 3, u'mitrailleuse': 2, u'procuratrice': 1, u'zwanzeuse': 2, u'lésineuse': 2, u'saure': 0, u'piquante': 0, u'redondante': 0, u'malotrue': 0, u'discoïdale': 3, u'sanguinolente': 0, u'argumentée': 0, u'bedonnante': 0, u'locomotive': 7, u'mutilatrice': 1, u'réflectrice': 1, u'buveuse': 2, u'protectionnelle': 4, u'attablée': 0, u'mobilisatrice': 1, u'ultrafine': 0, u'sous-préfète': 25, u'interurbaine': 0, u'vachère': 6, u'soixante-huitarde': 0, u'naissante': 0, u'proprette': 4, u'estropiée': 0, u'souveraine': 0, u'assommante': 0, u'laineuse': 9, u'rémanente': 0, u'coalisée': 0, u'effractive': 7, u'autoadjointe': 0, u'pennée': 0, u'équisingulière': 6, u'épileuse': 2, u'commerciale': 3, u'unisexuée': 0, u'emphysémateuse': 9, u'effrontée': 0, u'déclarative': 7, u'poseuse': 2, u'ourse': 0, u'corrigeuse': 2, u'sacrosainte': 0, u'rotatrice': 1, u'nargueuse': 2, u'codante': 0, u'quémandeuse': 2, u'longévive': 7, u'siccative': 7, u'parcheminière': 6, u'sévrienne': 4, u'eulérienne': 4, u'luxurieuse': 9, u'inquiétante': 0, u'épanouissante': 0, u'bichromatée': 0, u'démesurée': 0, u'haute-lissière': 6, u'habituelle': 4, u'néocomienne': 4, u'régulière': 6, u'fermentée': 0, u'lubrificatrice': 1, u'rentoileuse': 2, u'descendeuse': 2, u'paléo-environnementale': 3, u'interro-négative': 7, u'malandreuse': 9, u'soudante': 0, u'triumvirale': 3, u'agressive': 7, u'malaise': 0, u'galvanisante': 0, u'manuscrite': 0, u'héraultaise': 0, u'flagrante': 0, u'oratorienne': 4, u'intuitive': 7, u'laborantine': 0, u'bâcleuse': 2, u'gastro-duodénale': 3, u'expiratrice': 1, u'équine': 0, u'auteure': 0, u'trapue': 0, u'sommative': 7, u'épilatrice': 1, u'aisée': 0, u'plaidante': 0, u'cristallophyllienne': 4, u'houspilleuse': 2, u'ciseleuse': 2, u'préfixale': 3, u'loco-régionale': 3, u'postulatrice': 1, u'pronostiqueuse': 2, u'monaurale': 3, u'faiblissante': 0, u'vraie': 0, u'sismale': 3, u'prestidigitatrice': 1, u'haute': 0, u'bavarde': 0, u'murrhine': 0, u'métalliseuse': 2, u'ginguette': 4, u'ramenarde': 0, u'stative': 7, u'travestie': 0, u'irlandaise': 0, u'tinctoriale': 3, u'virilisante': 0, u'famennoise': 0, u'multifonctionnelle': 4, u'assistée': 0, u'préputiale': 0, u'baiseuse': 2, u'auboise': 0, u'ambassadrice': 1, u'anti-sous-marine': 0, u'convective': 7, u'recasquante': 0, u'diligente': 0, u'couvreuse': 2, u'péroreuse': 2, u'dératée': 0, u'circulante': 0, u'cumularde': 0, u'huitrière': 6, u'coursière': 6, u'insurrectionnelle': 4, u'saccareuse': 9, u'fraiche': 27, u'prestigieuse': 9, u'imageuse': 2, u'saboteuse': 2, u'croquante': 0, u'complaisante': 0, u'tapageuse': 2, u'chançarde': 0, u'dure': 0, u'calqueuse': 2, u'zaïroise': 0, u'fallacieuse': 9, u'autoréférentielle': 4, u'transcaspienne': 4, u'micacée': 0, u'différentielle': 4, u'bienfaitrice': 1, u'membrue': 0, u'employeuse': 2, u'saumonée': 0, u'mégissière': 6, u'saônoise': 0, u'transcriptionnelle': 4, u'assomptive': 7, u'barbette': 4, u'prôneuse': 2, u'impédante': 0, u'méplate': 0, u'urétrale': 3, u'angineuse': 9, u'assombrissante': 0, u'épicycloïdale': 3, u'peule': 0, u'phalanstérienne': 4, u'électorale': 3, u'épicentrale': 3, u'récréationnelle': 4, u'barakie': 0, u'ostentatrice': 1, u'satisfaisante': 0, u'négro-africaine': 0, u'réformatrice': 1, u'cybernéticienne': 4, u'câbleuse': 2, u'immunodépressive': 7, u'lâcheuse': 2, u'néo-calédonienne': 4, u'marnaise': 0, u'folkeuse': 9, u'inusuelle': 4, u'arbitrale': 3, u'colo-rectale': 3, u'temporisatrice': 1, u'chancelière': 6, u'initiale': 3, u'ravine': 0, u'mireuse': 2, u'dynamiteuse': 2, u'processive': 7, u'gazouilleuse': 2, u'handicapée': 0, u'staturo-pondérale': 3, u'féringienne': 4, u'surexcitante': 0, u'molletonneuse': 9, u'élavée': 0, u'agroforestière': 6, u'avant-courrière': 6, u'maigrelette': 4, u'reconquérante': 0, u'simultanée': 0, u'sanctifiante': 0, u'cheloue': 0, u'copulatrice': 1, u'développante': 0, u'malveillante': 0, u'prosternée': 0, u'mordante': 0, u'percutante': 0, u'éreinteuse': 2, u'sagittée': 0, u'procrastinatrice': 1, u'dominicale': 3, u'acridienne': 4, u'belgo-néerlandaise': 0, u'vogoule': 0, u'liante': 0, u'enrhumée': 0, u'inertielle': 4, u'retoucheuse': 2, u'exportatrice': 1, u'cachotière': 6, u'prépalatale': 3, u'wallingante': 0, u'corticospinale': 3, u'réprobatrice': 1, u'acclamatrice': 1, u'goutteuse': 9, u'absorbante': 0, u'minutieuse': 9, u'écharneuse': 2, u'vertigineuse': 9, u'graissante': 0, u'oscillobattante': 0, u'stayeuse': 2, u'imitatrice': 1, u'cambiale': 3, u'maquisarde': 0, u'mamelue': 0, u'multimodale': 3, u'urbaine': 0, u'planeuse': 2, u'staminale': 3, u'antiseigneuriale': 3, u'pagayeuse': 2, u'vaseuse': 9, u'auto-adjointe': 0, u'consciencieuse': 9, u'inhibitrice': 1, u'communicationnelle': 4, u'corsetière': 6, u'indifférente': 0, u'socioéducative': 7, u'accusative': 7, u'attachée': 0, u'sous-intendante': 0, u'barbotine': 0, u'constituante': 0, u'mongole': 0, u'marquante': 0, u'décompteuse': 2, u'récitante': 0, u'sportive': 7, u'mouillante': 0, u'infichue': 0, u'orléanaise': 0, u'résignée': 0, u'vaginale': 3, u'tondue': 0, u'subéquatoriale': 3, u'prépensionnée': 0, u'avant-dernière': 6, u'fréquentielle': 4, u'badaude': 0, u'attristante': 0, u'conoïdale': 3, u'concédante': 0, u'fragrante': 0, u'acromiale': 3, u'déchaumeuse': 2, u'appauvrissante': 0, u'vicariale': 3, u'cheminote': 0, u'allumettière': 6, u'salésienne': 4, u'bibeloteuse': 2, u'surgie': 0, u'végane': 0, u'légumineuse': 9, u'blanchisseuse': 2, u'thermopropulsée': 0, u'purgative': 7, u'coffretière': 6, u'populacière': 6, u'soudière': 6, u'nazaréenne': 4, u'affranchie': 0, u'aoutienne': 4, u'libanaise': 0, u'écervelée': 0, u'incroyante': 0, u'cohéritière': 6, u'teinte': 0, u'caricaturale': 3, u'papillotante': 0, u'montante': 0, u'rasante': 0, u'prééminente': 0, u'subventionnelle': 4, u'fermentative': 7, u'bollywoodienne': 4, u'affective': 7, u'giboyeuse': 9, u'arillée': 0, u'profonde': 0, u'immédiate': 0, u'bipartie': 0, u'lithinée': 0, u'traineuse': 2, u'périnéale': 3, u'gambienne': 4, u'justifiante': 0, u'matraqueuse': 2, u'combattive': 7, u'simplificatrice': 1, u'ravineuse': 9, u'environnante': 0, u'métayère': 6, u'déguenillée': 0, u'arrangeante': 0, u'essencière': 6, u'salvadorienne': 4, u'diviseuse': 2, u'scrupuleuse': 9, u'émolliente': 0, u'structurante': 0, u'adscrite': 0, u'épaississante': 0, u'pédiculée': 0, u'ammoniacée': 0, u'fontainière': 6, u'distale': 3, u'jongleuse': 2, u'antipaludéenne': 4, u'ajusteuse': 2, u'rotative': 7, u'endocrânienne': 4, u'archimédienne': 4, u'comburante': 0, u'altière': 6, u'chicanière': 6, u'débineuse': 2, u'bitumeuse': 9, u'blogueuse': 2, u'bourrue': 0, u'recrudescente': 0, u'inhalatrice': 1, u'mécréante': 0, u'monoskieuse': 2, u'cagoulée': 0, u'tyrolienne': 4, u'clapoteuse': 9, u'intercurrente': 0, u'hydraulicienne': 4, u'pluvio-nivale': 3, u'haveuse': 2, u'malavisée': 0, u'renifleuse': 2, u'beurrière': 6, u'reprise': 0, u'tchadienne': 4, u'opérée': 0, u'caissière': 6, u'assortisseuse': 2, u'cobelligérante': 0, u'florale': 3, u'infructueuse': 9, u'réglante': 0, u'palière': 6, u'déménageuse': 2, u'associatrice': 1, u'boulonnière': 6, u'scandaleuse': 9, u'séditieuse': 9, u'ensommeillée': 0, u'leveuse': 2, u'succenturiée': 0, u'interastrale': 3, u'inscrivante': 0, u'manchoise': 0, u'dérobeuse': 2, u'marmitonne': 4, u'accélératrice': 1, u'pédégée': 0, u'effrayante': 0, u'videuse': 2, u'fluorée': 0, u'pro-occidentale': 3, u'marieuse': 2, u'renforçatrice': 1, u'peuhle': 0, u'infranationale': 3, u'affûteuse': 2, u'flambante': 0, u'brinquebalante': 0, u'orthonormale': 3, u'dominante': 0, u'hilbertienne': 4, u'péribuccale': 3, u'dissipatrice': 1, u'pétrifiante': 0, u'lamarckienne': 4, u'finno-ougrienne': 4, u'rostrale': 3, u'inaboutie': 0, u'sifflotante': 0, u'équilibrante': 0, u'plombeuse': 9, u'levalloisienne': 4, u'processionnelle': 4, u'liquidatrice': 1, u'directoriale': 3, u'stoloniale': 3, u'effilocheuse': 2, u'exécutrice': 1, u'picturale': 3, u'fascinatrice': 1, u'euthanasiée': 0, u'équatoriale': 3, u'embauchée': 0, u'gerbeuse': 2, u'rouquine': 0, u'putrescente': 0, u'fuégienne': 4, u'socioprofessionnelle': 4, u'himalayenne': 4, u'lointaine': 0, u'amoureuse': 9, u'savonneuse': 9, u'franco-péruvienne': 4, u'conseilleuse': 2, u'tibiale': 3, u'anglo-suédoise': 0, u'bienvoulue': 0, u'euscarienne': 4, u'amorceuse': 2, u'intoxicante': 0, u'capétienne': 4, u'pasticheuse': 2, u'inconsciente': 0, u'ferblantière': 6, u'émergée': 0, u'électrotechnicienne': 4, u'schwarzienne': 4, u'dominotière': 6, u'parotidienne': 4, u'placette': 4, u'princière': 6, u'baladodiffuseuse': 2, u'écraseuse': 2, u'glaceuse': 9, u'hypogée': 0, u'génitale': 3, u'transylvaine': 0, u'surjective': 7, u'décalqueuse': 2, u'pilosébacée': 0, u'hottentote': 0, u'romanichelle': 4, u'techniverrière': 6, u'andine': 0, u'quadrifoliée': 0, u'gravettienne': 4, u'noircissante': 0, u'noyauteuse': 2, u'salissante': 0, u'non-résidente': 0, u'jaïne': 0, u'grossissante': 0, u'élégante': 0, u'bêtabloquante': 0, u'malnutrie': 0, u'bonifiante': 0, u'médicale': 3, u'gargotière': 6, u'psycho-affective': 7, u'réverbérante': 0, u'marinière': 6, u'pollicitante': 0, u'suffixale': 3, u'oppressante': 0, u'ringarde': 0, u'conjuratrice': 1, u'idéale': 3, u'coquillière': 6, u'synoviale': 3, u'dormante': 0, u'malapprise': 0, u'dégrossisseuse': 2, u'dépurative': 7, u'instauratrice': 1, u'désambiguïsatrice': 1, u'camphrée': 0, u'nipponne': 4, u'graduée': 0, u'brochante': 0, u'expressive': 7, u'coéditrice': 1, u'pâtissière': 6, u'télédéclarante': 0, u'crouteuse': 9, u'assyrienne': 4, u'primatiale': 3, u'rarescente': 0, u'salpingienne': 4, u'connivente': 0, u'émancipatrice': 1, u'franco-uruguayenne': 4, u'attirante': 0, u'fleuronnée': 0, u'polyvalente': 0, u'fautive': 7, u'cuisante': 0, u'marcescente': 0, u'délibérée': 0, u'laborante': 0, u'auto-évaluante': 0, u'efféminée': 0, u'paléoenvironnementale': 3, u'moyenne': 4, u'franco-libanaise': 0, u'patineuse': 2, u'persécutrice': 1, u'démonstrative': 7, u'bissectrice': 1, u'interprétative': 7, u'grasse': 4, u'newtonienne': 4, u'coxale': 3, u'exclue': 0, u'relaxante': 0, u'pythagoricienne': 4, u'kantienne': 4, u'marathonienne': 4, u'raccrocheuse': 2, u'grimaude': 0, u'affréteuse': 2, u'sino-américaine': 0, u'incontestée': 0, u'chantourneuse': 2, u'vermoulue': 0, u'royale': 3, u'intouchée': 0, u'socio-culturelle': 4, u'contrattaquante': 0, u'caramélée': 0, u'chanteuse': 2, u'choroïdienne': 4, u'coobligée': 0, u'hindie': 0, u'staphyline': 0, u'tarnaise': 0, u'francilienne': 4, u'contre-attaquante': 0, u'seigneuriale': 3, u'jappeuse': 2, u'sabreuse': 2, u'extensive': 7, u'reconventionnelle': 4, u'dépendeuse': 2, u'bonimenteuse': 2, u'étuveuse': 2, u'frangine': 0, u'nullarde': 0, u'bêlante': 0, u'administrée': 0, u'républicaine': 0, u'présanctifiée': 0, u'véhémente': 0, u'progressive': 7, u'sanskrite': 0, u'précolombienne': 4, u'vasomotrice': 1, u'semi-publique': 13, u'intraépithéliale': 3, u'carreleuse': 2, u'interpellée': 0, u'incivile': 0, u'didacticienne': 4, u'mesureuse': 2, u'ultra-courte': 0, u'finnoise': 0, u'discourtoise': 0, u'provirale': 3, u'haschichine': 0, u'sécrétrice': 1, u'loculée': 0, u'insignifiante': 0, u'modalisatrice': 1, u'arrondisseuse': 2, u'voteuse': 2, u'éplucheuse': 2, u'capitée': 0, u'pilleuse': 2, u'affligée': 0, u'laqueuse': 9, u'bisaïeule': 0, u'rabouilleuse': 2, u'messagère': 6, u'préventive': 7, u'crépisseuse': 2, u'poudroyante': 0, u'silencieuse': 9, u'limbourgeoise': 0, u'inquisitive': 7, u'enveloppante': 0, u'bucheuse': 2, u'tintinnabulante': 0, u'lacuneuse': 9, u'circumnavigatrice': 1, u'éminatie': 0, u'discale': 3, u'embaucheuse': 2, u'compendieuse': 9, u'stendhalienne': 4, u'nasilleuse': 2, u'plutonienne': 4, u'extrarénale': 3, u'envoûteuse': 2, u'alkylante': 0, u'équipementière': 6, u'amplective': 7, u'buissonnière': 6, u'cambrienne': 4, u'prostitutionnelle': 4, u'alarmante': 0, u'brumeuse': 9, u'vitreuse': 9, u'antiguaise': 0, u'directive': 7, u'protutrice': 1, u'débaucheuse': 2, u'ténorisante': 0, u'moutonneuse': 9, u'sinusoïdale': 3, u'arrivante': 0, u'faillie': 0, u'inexploitée': 0, u'gasconne': 4, u'confectionneuse': 2, u'algale': 3, u'permienne': 4, u'trouveuse': 2, u'fouisseuse': 2, u'carmélitaine': 0, u'prostrée': 0, u'hastée': 0, u'excédante': 0, u'situationnelle': 4, u'sonneuse': 2, u'aimée': 0, u'étrangère': 6, u'courtière': 6, u'saturante': 0, u'footeuse': 9, u'fripière': 6, u'pustulée': 0, u'staturopondérale': 3, u'repasseuse': 2, u'intestine': 0, u'archidiocésaine': 0, u'inconséquente': 0, u'jointée': 0, u'ventrue': 0, u'débitrice': 1, u'spatulée': 0, u'voyeuse': 2, u'glossopharyngienne': 4, u'capricante': 0, u'inappréciée': 0, u'superovariée': 0, u'hasardeuse': 9, u'traverse': 0, u'enveloppeuse': 2, u'cokéfiante': 0, u'bédouine': 0, u'acupunctrice': 1, u'mozambicaine': 0, u'sarthoise': 0, u'dalembertienne': 4, u'historienne': 4, u'riante': 0, u'expéditive': 7, u'potinière': 6, u'désespérante': 0, u'combattante': 0, u'poirée': 0, u'appareilleuse': 2, u'répondeuse': 2, u'baigneuse': 2, u'cimmérienne': 4, u'active': 7, u'arminienne': 4, u'uniovulée': 0, u'majestueuse': 9, u'dandinante': 0, u'compétente': 0, u'rôtie': 0, u'farfouilleuse': 2, u'exécutante': 0, u'tolérante': 0, u'testimoniale': 3, u'chialeuse': 2, u'excrétrice': 1, u'sabordeuse': 2, u'dégoulinante': 0, u'camée': 0, u'embrouilleuse': 2, u'bretessée': 0, u'bourgeoisiale': 3, u'implosive': 7, u'sébacée': 0, u'bolivienne': 4, u'feue': 0, u'papuleuse': 9, u'coïnculpée': 0, u'gazetière': 6, u'néandertalienne': 4, u'emmerdeuse': 2, u'lupuline': 0, u'recolleuse': 2, u'rapeuse': 2, u'emboutisseuse': 2, u'cotylédonée': 0, u'lacérante': 0, u'ballante': 0, u'photoconductrice': 1, u'vagale': 3, u'occupante': 0, u'indolente': 0, u'évacuante': 0, u'étroite': 0, u'démystificatrice': 1, u'bivariante': 0, u'chinoise': 0, u'houlomotrice': 1, u'multidirectionnelle': 4, u'haschischine': 0, u'coupante': 0, u'nazie': 0, u'sacramentelle': 4, u'ferronnière': 6, u'frottante': 0, u'comminutive': 7, u'ingénue': 0, u'matineuse': 9, u'sémillante': 0, u'transsibérienne': 4, u'onéreuse': 9, u'douanière': 6, u'nécromante': 0, u'bullée': 0, u'fédérale': 3, u'aspirante': 0, u'complimenteuse': 2, u'irréfléchie': 0, u'percluse': 0, u'amusante': 0, u'râlante': 0, u'catégorielle': 4, u'surreprésentée': 0, u'florentine': 0, u'infortunée': 0, u'laceuse': 2, u'officière': 6, u'calmante': 0, u'antibourgeoise': 0, u'revenante': 0, u'hachischine': 0, u'pacificatrice': 1, u'radiotélévisée': 0, u'dartreuse': 9, u'hauturière': 6, u'élévatrice': 1, u'cotonnière': 6, u'antidiagonale': 3, u'compétitive': 7, u'œdémateuse': 9, u'gemmeuse': 2, u'libératrice': 1, u'cléricale': 3, u'creusoise': 0, u'crènelée': 0, u'truande': 0, u'captivante': 0, u'lampassée': 0, u'retombante': 0, u'bimestrielle': 4, u'bêtifiante': 0, u'protubérante': 0, u'magistrale': 3, u'autotractée': 0, u'inondée': 0, u'contre-électromotrice': 1, u'incongrue': 0, u'mienne': 4, u'dépréciatrice': 1, u'oreillarde': 0, u'sublétale': 3, u'éluviale': 3, u'galopante': 0, u'indo-aryenne': 4, u'oxydoréductrice': 1, u'pâlotte': 4, u'alexandrine': 0, u'fraiseuse': 2, u'accommodante': 0, u'toroïdale': 3, u'mycénienne': 4, u'plucheuse': 9, u'autosomale': 3, u'purpurine': 0, u'fistulisante': 0, u'folichonne': 4, u'désinfectrice': 1, u'deutérée': 0, u'bourdonnée': 0, u'orfévrée': 0, u'centennale': 3, u'sino-tibétaine': 0, u'précancéreuse': 9, u'canine': 0, u'planteuse': 2, u'ratiocineuse': 2, u'opérationnelle': 4, u'akkadienne': 4, u'hongroise': 0, u'effective': 7, u'accommodatrice': 1, u'sénescente': 0, u'confidente': 0, u'bassecourière': 6, u'quinquennale': 3, u'pareille': 4, u'pochetronne': 4, u'écotée': 0, u'sursaturante': 0, u'précitée': 0, u'codétentrice': 1, u'silurienne': 4, u'saccageuse': 2, u'avitailleuse': 2, u'anticyclonale': 3, u'asservissante': 0, u'morfale': 0, u'nommée': 0, u'incompréhensive': 7, u'adroite': 0, u'verdissante': 0, u'socioculturelle': 4, u'jacasseuse': 2, u'sous-peuplée': 0, u'activatrice': 1, u'kazakhe': 0, u'officiante': 0, u'déficiente': 0, u'vineuse': 9, u'lactéale': 3, u'tétravalente': 0, u'grenouillante': 0, u'léonarde': 0, u'inappropriée': 0, u'persuasive': 7, u'postprandiale': 3, u'urogénitale': 3, u'péritonéale': 3, u'intragénérationnelle': 4, u'foxée': 0, u'intertextuelle': 4, u'ravaudeuse': 2, u'pointue': 0, u'perturbative': 7, u'aimante': 0, u'nitrifiante': 0, u'nécrosante': 0, u'anecdotière': 6, u'rétractive': 7, u'inexpérimentée': 0, u'phraseuse': 2, u'unipersonnelle': 4, u'lamineuse': 2, u'trufficultrice': 1, u'chargée': 0, u'révérencieuse': 9, u'mordeuse': 2, u'troqueuse': 2, u'montréalaise': 0, u'ronronnante': 0, u'persane': 0, u'lacédémonienne': 4, u'métacarpienne': 4, u'introspective': 7, u'autogérée': 0, u'trinidadienne': 4, u'componée': 0, u'communiante': 0, u'incommodante': 0, u'herbue': 0, u'classicisante': 0, u'réticente': 0, u'grillette': 4, u'ravisseuse': 2, u'cuivreuse': 9, u'triomphatrice': 1, u'esbroufeuse': 2, u'gigotante': 0, u'pèlerine': 0, u'embarrassante': 0, u'dravidienne': 4, u'talentueuse': 9, u'succincte': 0, u'bimodale': 3, u'subjuguante': 0, u'betteravière': 6, u'wagnérienne': 4, u'turco-mongole': 0, u'subordonnée': 0, u'cartonnière': 6, u'languissante': 0, u'parqueteuse': 2, u'drummeuse': 2, u'avaricieuse': 9, u'liminale': 3, u'conventionnelle': 4, u'démone': 0, u'sympathisante': 0, u'polycristalline': 0, u'céphalée': 0, u'négociante': 0, u'résidentielle': 4, u'rancuneuse': 9, u'gouailleuse': 2, u'valorisante': 0, u'tortureuse': 2, u'égyptienne': 4, u'ardoisière': 6, u'grappilleuse': 2, u'haillonneuse': 9, u'subexponentielle': 4, u'pédieuse': 9, u'joufflue': 0, u'extérieure': 0, u'intégrante': 0, u'inspirante': 0, u'croisée': 0, u'déchue': 0, u'commandante': 0, u'tubéracée': 0, u'coopérative': 7, u'psychologisante': 0, u'consommatrice': 1, u'revendeuse': 2, u'laxative': 7, u'pétillante': 0, u'historisante': 0, u'professionnalisante': 0, u'laïcarde': 0, u'faneuse': 2, u'rubigineuse': 9, u'monophasée': 0, u'estonienne': 4, u'belliqueuse': 9, u'voltigeuse': 2, u'désinsectiseuse': 2, u'oasienne': 4, u'lesbienne': 4, u'curiale': 3, u'franco-danoise': 0, u'ganguée': 0, u'halitueuse': 9, u'casquante': 0, u'insatisfaite': 0, u'indivise': 0, u'gallo-romaine': 0, u'formalisante': 0, u'paonne': 4, u'antérieure': 0, u'prairiale': 3, u'réhabilitatrice': 1, u'fluidifiante': 0, u'mouleuse': 2, u'mielleuse': 9, u'jeteuse': 2, u'érysipélateuse': 9, u'dépressive': 7, u'clermontoise': 0, u'perforante': 0, u'galicienne': 4, u'botteuse': 2, u'gentille': 4, u'plate': 0, u'bâilleuse': 2, u'languée': 0, u'tremblée': 0, u'foutue': 0, u'brouteuse': 2, u'cahoteuse': 9, u'floricultrice': 1, u'embosseuse': 2, u'boscotte': 4, u'antiamarile': 0, u'omnidirectionnelle': 4, u'lacrymale': 3, u'auriculée': 0, u'arrogante': 0, u'plagale': 3, u'attachante': 0, u'classieuse': 9, u'jamaïquaine': 0, u'polytonale': 3, u'zouloue': 0, u'radicale': 3, u'franco-québécoise': 0, u'galonnière': 6, u'mandarine': 0, u'stupéfiante': 0, u'excitomotrice': 1, u'guérissante': 0, u'amandée': 0, u'flâneuse': 2, u'envoutée': 0, u'masseuse': 2, u'autocentrée': 0, u'potière': 6, u'sarcleuse': 2, u'cardiale': 3, u'inexplorée': 0, u'jouissante': 0, u'inexécutée': 0, u'obsédée': 0, u'bureauticienne': 4, u'oblitératrice': 1, u'alcalinisante': 0, u'sous-maitresse': 16, u'taurine': 0, u'acconière': 6, u'ambisexuée': 0, u'shampooineuse': 2, u'lissante': 0, u'impolie': 0, u'foncière': 6, u'dorsale': 3, u'précordiale': 3, u'intrafamiliale': 3, u'franco-néerlandaise': 0, u'visionneuse': 2, u'tyrrhénienne': 4, u'ethnocentrée': 0, u'non-violente': 0, u'pétreuse': 9, u'joviale': 3, u'divagante': 0, u'lubrifiante': 0, u'computationnelle': 4, u'replète': 25, u'pelliculeuse': 9, u'motivante': 0, u'tubérisée': 0, u'confédérale': 3, u'motarde': 0, u'paraphernale': 3, u'bestiale': 3, u'surréservée': 0, u'pyriteuse': 9, u'inassouvie': 0, u'examinatrice': 1, u'maboule': 0, u'rechigneuse': 9, u'égressive': 7, u'déconcertante': 0, u'dictatrice': 1, u'galbante': 0, u'réalisatrice': 1, u'aubaine': 0, u'germano-chinoise': 0, u'luxembourgeoise': 0, u'apicultrice': 1, u'arénacée': 0, u'tambourinante': 0, u'fusante': 0, u'paripennée': 0, u'tacticienne': 4, u'bourgeoise': 0, u'vendangeuse': 2, u'interactionnelle': 4, u'gangreneuse': 9, u'tribunitienne': 4, u'fourmillante': 0, u'cyberdélinquante': 0, u'syncytiale': 3, u'cinghalaise': 0, u'obstructive': 7, u'collaborative': 7, u'bajocienne': 4, u'dévaliseuse': 2, u'tordue': 0, u'paramédicale': 3, u'infinitésimale': 3, u'encenseuse': 2, u'ferrailleuse': 2, u'nauséabonde': 0, u'révisionnelle': 4, u'adverbiale': 3, u'vacancière': 6, u'étancheuse': 2, u'vigoureuse': 9, u'cintrière': 6, u'anglo-israélienne': 4, u'codirectrice': 1, u'urgonienne': 4, u'dynamicienne': 4, u'caloporteuse': 2, u'brésilienne': 4, u'féline': 0, u'remonteuse': 2, u'liliale': 3, u'calculeuse': 9, u'affixée': 0, u'dégénérée': 0, u'assujettissante': 0, u'irrévélée': 0, u'envahissante': 0, u'cylindro-ogivale': 3, u'médisante': 0, u'tombante': 0, u'élue': 0, u'pullulante': 0, u'verticale': 3, u'olécranienne': 4, u'capacitive': 7, u'trépassée': 0, u'casseuse': 2, u'exposante': 0, u'ovulée': 0, u'cabocharde': 0, u'vendue': 0, u'tokharienne': 4, u'tubarde': 0, u'repreneuse': 2, u'suspensive': 7, u'estuarienne': 4, u'inusitée': 0, u'poissarde': 0, u'exterminatrice': 1, u'compositrice': 1, u'envoutante': 0, u'prédécoupée': 0, u'pédante': 0, u'dodelinante': 0, u'ergotée': 0, u'surjacente': 0, u'omanaise': 0, u'chipoteuse': 2, u'performeuse': 2, u'pluvieuse': 9, u'errante': 0, u'discriminante': 0, u'influente': 0, u'mutine': 0, u'dative': 7, u'découvreuse': 2, u'facturière': 6, u'mercaticienne': 4, u'phosphorescente': 0, u'monoclonale': 3, u'suprahumaine': 0, u'importante': 0, u'bootleggeuse': 2, u'blondinette': 4, u'kidnappeuse': 2, u'extrudeuse': 2, u'compassionnelle': 4, u'bocagère': 6, u'semi-ouvrée': 0, u'fidjienne': 4, u'rhénane': 0, u'attrapeuse': 2, u'comtale': 3, u'ambulante': 0, u'testacée': 0, u'pluviale': 3, u'munichoise': 0, u'microbienne': 4, u'dalmatienne': 4, u'latino-américaine': 0, u'herbeuse': 9, u'résistive': 7, u'jurisprudentielle': 4, u'infinitive': 7, u'gabonaise': 0, u'soufie': 0, u'étincelante': 0, u'nette': 4, u'thésarde': 0, u'orgueilleuse': 9, u'glandeuse': 2, u'néo-grammairienne': 4, u'rubescente': 0, u'ressasseuse': 2, u'pochtronne': 4, u'divorcée': 0, u'croupière': 6, u'lucrative': 7, u'pseudomembraneuse': 9, u'subliminale': 3, u'hautaine': 0, u'costale': 3, u'étourdie': 0, u'colombienne': 4, u'muqueuse': 9, u'sultane': 0, u'précautionneuse': 9, u'strip-teaseuse': 2, u'accédante': 0, u'septembrale': 3, u'cadavéreuse': 9, u'fontenière': 6, u'mutilée': 0, u'dévote': 0, u'verrière': 6, u'vénusienne': 4, u'boréale': 3, u'validatrice': 1, u'oblative': 7, u'paludéenne': 4, u'bicipitale': 3, u'olympienne': 4, u'lagrangienne': 4, u'épatante': 0, u'précipitée': 0, u'impétigineuse': 9, u'poussine': 0, u'fondante': 0, u'bourdonneuse': 2, u'griffue': 0, u'trachéale': 3, u'première': 6, u'tempéramentale': 3, u'fumerolienne': 4, u'édificatrice': 1, u'sergente': 0, u'épicière': 6, u'préhilbertienne': 4, u'bronzière': 6, u'sagittale': 3, u'lombosacrée': 0, u'convertisseuse': 2, u'masculine': 0, u'gaussienne': 4, u'borélienne': 4, u'expérimentale': 3, u'indo-européenne': 4, u'rabroueuse': 2, u'inventive': 7, u'insoupçonnée': 0, u'explétive': 7, u'embarquée': 0, u'brossière': 6, u'rugissante': 0, u'biophysicienne': 4, u'médiaplanneuse': 2, u'judéo-chrétienne': 4, u'rugueuse': 9, u'morte': 0, u'transcriptrice': 1, u'existentielle': 4, u'rémoise': 0, u'lombo-sacrée': 0, u'plurielle': 4, u'attendrissante': 0, u'revendicative': 7, u'geôlière': 6, u'boulote': 0, u'mosellane': 0, u'décervelée': 0, u'sécurisante': 0, u'abondée': 0, u'bi-infinie': 0, u'préencollée': 0, u'luxueuse': 9, u'fistuleuse': 9, u'instigatrice': 1, u'empalée': 0, u'pleurarde': 0, u'abroutie': 0, u'psychoéducative': 7, u'hurdleuse': 2, u'socio-professionnelle': 4, u'captante': 0, u'courtisée': 0, u'normalisatrice': 1, u'attentionnelle': 4, u'mictionnelle': 4, u'hydrolienne': 4, u'inséminée': 0, u'avenante': 0, u'diapositive': 7, u'individuelle': 4, u'rimailleuse': 2}

ASCIIZERO = 48  # 48 is the ASCII code for "0"

def isFemForm (s):
    "returns True if s exists in dMasFW"
    if s in _dMasFW:
        return True
    return False

def getMasForm (s, bPlur):
    "returns masculine form with feminine form"
    if s in _dMasFW:
        return _modifyStringWithSuffixCode(s, _whatSuffixCode(_lTags[_dMasFW[s]], bPlur))
    return ""

def isMiscPlural (s):
    "returns True if s exists in dPlurMisc"
    if s in _dPlurMisc:
        return True
    return False

def getMiscPlural (s):
    "returns plural form with singular form"
    if s in _dPlurMisc:
        return _modifyStringWithSuffixCode(s, _lTags[_dPlurMisc[s]])
    return ""

def _whatSuffixCode (sAff, bPlur):
    "necessary only for dMasFW"
    if bPlur:
        if "|" not in sAff:
            if sAff.endswith(("x", "s")):
                return sAff
            return sAff+"s"
        return sAff[sAff.find("|")+1:]
    if "|" in sAff:
        return sAff[:sAff.find("|")]
    return sAff

def _modifyStringWithSuffixCode (s, sAff):
    "returns s modified by sAff"
    if not sAff:
        return ""
    if sAff == "0":
        return s
    try:
        return s[:-(ord(sAff[0])-ASCIIZERO)] + sAff[1:]  if sAff[0] != '0'  else  s + sAff[1:]
    except:
        return u"## erreur, code : " + str(sAff) + " ##"
